// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: int16.cpp 
// Compiler Used: MSVC40, DJGPP 2.7.2.1, GCC 2.7.2.1, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 09/05/97  
// Date Last Modified: 12/03/97
// Copyright (c) 1997 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBD C++ classes are copyright (c) 1997, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The INT16 class is used to represent 16 bit signed integers
independently of the operating system or hardware platform
used. It was created to overcome the big and little endian
problem encountered when writing integer values to a common
database file accessed by several different types of machines.

The term "endian" is used to describe the order in which
multibyte numbers are stored in the computers's memory. This
is specifc to each microprocessor. The Intel x86 family is
little-endian, meaning the lowest-order byte is stored first.
Hewlett-Packard's PA-RISC and Sun's SuperSPARC are big-endian,
meaning the highest-order byte is stored first. The Silicon
Graphics MIPS and IBM/Motorola PowerPC processors are both
little and big endian (bi-endian).

 Multibyte Value     Big-Endian    Little-Endian
   0x12345678        0x12345678    0x78563412
   0x1234            0x1234        0x3412
   0x5678            0x5678        0x7856
   "ABC"             41 42 42 00   41 42 43 00

With big-endian ordering, the address of the multibyte value
is its most significant byte (its big end). With little-endian
ordering, the address is its least significant byte (its little
end). In a struct the order of bytes in memory will differ
depending on the byte ordering and the particular data type.
*/
// ----------------------------------------------------------- // 
#include <string.h>
#include <memory.h>
#include "int16.hpp"
#include "ehandler.hpp"

INT16::INT16(SWORD val)
{
  UnPackBits(val);
}

INT16::INT16(const INT16& ob)
{
  memmove((void *)byte, (const void *)ob.byte, 2);
}

INT16& INT16::operator=(const INT16& ob)
{
  memmove((void *)byte, (const void *)ob.byte, 2);
  return *this;
}

INT16& INT16::operator=(const SWORD val)
{
  UnPackBits(val);
  return *this;
}

INT16::operator SWORD() const
{
  return PackBits();
}

SWORD INT16::PackBits() const
{
  SWORD a, b;
  
  a = (SWORD)byte[0];
  b = (SWORD)byte[1];

  a = a & 0xFF;
  b = (b<<8) & 0xFF00;

  return a + b;
}

void INT16::UnPackBits(SWORD val)
{
  byte[0] = val & 0xFF;
  byte[1] = (val & 0xFF00)>>8;
}

INT16 INT16::operator++(int) // Postfix
{
  INT16 val_before(*this); 
  operator=(*this + 1);
  return val_before;
}
 
INT16 INT16::operator--(int) // Postfix
{
  INT16 val_before(*this); 
  operator=(*this - 1);
  return val_before;
}

void INT16::operator/=(const INT16 &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void INT16::operator/=(const LWORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

    operator=(*this / i);
}

void INT16::operator/=(const ULWORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void INT16::operator/=(const WORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void INT16::operator/=(const SWORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void INT16::operator/=(const UWORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void INT16::operator/=(const USWORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void INT16::operator/=(const SBYTE &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / (SWORD)i);
}

void INT16::operator/=(const UBYTE &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / (SWORD)i);
}

int operator==(const INT16 &a, const INT16 &b)
{
  return a.PackBits() == b.PackBits();
}

int operator==(const INT16 &a, const LWORD &bs)
{
  return a.PackBits() == bs;
}

int operator==(const LWORD &as, const INT16 &b)
{
  return as == b.PackBits(); 
}

int operator==(const INT16 &a, const ULWORD &bs)
{
  return a.PackBits() == bs;
}

int operator==(const ULWORD &as, const INT16 &b)
{
  return as == b.PackBits(); 
}

int operator==(const INT16 &a, const WORD &bs)
{
  return a.PackBits() == bs;
}

int operator==(const WORD &as, const INT16 &b)
{
  return as == b.PackBits();
}

int operator==(const INT16 &a, const SWORD &bs)
{
  return a.PackBits() == bs;
}

int operator==(const SWORD &as, const INT16 &b)
{
  return as == b.PackBits(); 
}

int operator==(const INT16 &a, const UWORD &bs)
{
  return a.PackBits() == bs;
}

int operator==(const UWORD &as, const INT16 &b)
{
  return as == b.PackBits(); 
}

int operator==(const INT16 &a, const USWORD &bs)
{
  return  a.PackBits() == bs;
}

int operator==(const USWORD &as, const INT16 &b)
{
  return as == b.PackBits(); 
}

int operator==(const INT16 &a, const SBYTE &bs)
{
  return a.PackBits() == (SWORD)bs;
}

int operator==(const SBYTE &as, const INT16 &b)
{
  return (SWORD)as == b.PackBits(); 
}

int operator==(const INT16 &a, const UBYTE &bs)
{
  return a.PackBits() == (SWORD)bs;
}

int operator==(const UBYTE &as, const INT16 &b)
{
  return (SWORD)as == b.PackBits(); 
}

int operator!=(const INT16 &a, const INT16 &b)
{
  return a.PackBits() != b.PackBits();
}

int operator!=(const INT16 &a, const LWORD &bs)
{
  return a.PackBits() != bs;
}

int operator!=(const LWORD &as, const INT16 &b)
{
  return as != b.PackBits();
}

int operator!=(const INT16 &a, const ULWORD &bs)
{
  return a.PackBits() != bs;
}

int operator!=(const ULWORD &as, const INT16 &b)
{
  return as != b.PackBits();
}

int operator!=(const INT16 &a, const WORD &bs)
{
  return a.PackBits() != bs;
}

int operator!=(const WORD &as, const INT16 &b)
{
  return as != b.PackBits();
}

int operator!=(const INT16 &a, const SWORD &bs)
{
  return a.PackBits() != bs;
}

int operator!=(const SWORD &as, const INT16 &b)
{
  return as != b.PackBits();
}

int operator!=(const INT16 &a, const UWORD &bs)
{
  return a.PackBits() != bs;
}

int operator!=(const UWORD &as, const INT16 &b)
{
  return as != b.PackBits();
}

int operator!=(const INT16 &a, const USWORD &bs)
{
  return a.PackBits() != bs;
}

int operator!=(const USWORD &as, const INT16 &b)
{
  return as != b.PackBits();
}

int operator!=(const INT16 &a, const SBYTE &bs)
{
  return a.PackBits() != (SWORD)bs;
}

int operator!=(const SBYTE &as, const INT16 &b)
{
  return (SWORD)as != b.PackBits();
}

int operator!=(const INT16 &a, const UBYTE &bs)
{
  return a.PackBits() != (SWORD)bs;
}

int operator!=(const UBYTE &as, const INT16 &b)
{
  return (SWORD)as != b.PackBits();
}

int operator<(const INT16 &a, const INT16 &b)
{
  return a.PackBits() < b.PackBits();
}

int operator<(const INT16 &a, const LWORD &bs)
{
  return a.PackBits() < bs;
}

int operator<(const LWORD &as, const INT16 &b)
{
  return as < b.PackBits();
}

int operator<(const INT16 &a, const ULWORD &bs)
{
  return a.PackBits() < bs;
}

int operator<(const ULWORD &as, const INT16 &b)
{
  return as < b.PackBits();
}

int operator<(const INT16 &a, const WORD &bs)
{
  return a.PackBits() < bs;
}

int operator<(const WORD &as, const INT16 &b)
{
  return as < b.PackBits();
}

int operator<(const INT16 &a, const SWORD &bs)
{
  return a.PackBits() < bs;
}

int operator<(const SWORD &as, const INT16 &b)
{
  return as < b.PackBits();
}

int operator<(const INT16 &a, const UWORD &bs)
{
  return a.PackBits() < bs;
}

int operator<(const UWORD &as, const INT16 &b)
{
  return as < b.PackBits();
}

int operator<(const INT16 &a, const USWORD &bs)
{
  return a.PackBits() < bs;
}

int operator<(const USWORD &as, const INT16 &b)
{
  return as < b.PackBits();
}

int operator<(const INT16 &a, const SBYTE &bs)
{
  return a.PackBits() < (SWORD)bs;
}

int operator<(const SBYTE &as, const INT16 &b)
{
  return (SWORD)as < b.PackBits();
}

int operator<(const INT16 &a, const UBYTE &bs)
{
  return a.PackBits() < (SWORD)bs;
}

int operator<(const UBYTE &as, const INT16 &b)
{
  return (SWORD)as < b.PackBits();
}

int operator>(const INT16 &a, const INT16 &b)
{
  return a.PackBits() > b.PackBits();
}

int operator>(const INT16 &a, const LWORD &bs)
{
  return a.PackBits() > bs;
}

int operator>(const LWORD &as, const INT16 &b)
{
  return as > b.PackBits();
}

int operator>(const INT16 &a, const ULWORD &bs)
{
  return a.PackBits() > bs;
}

int operator>(const ULWORD &as, const INT16 &b)
{
  return as > b.PackBits();
}

int operator>(const INT16 &a, const WORD &bs)
{
  return a.PackBits() > bs;
}

int operator>(const WORD &as, const INT16 &b)
{
  return as > b.PackBits();
}

int operator>(const INT16 &a, const SWORD &bs)
{
  return a.PackBits() > bs;
}

int operator>(const SWORD &as, const INT16 &b)
{
  return as > b.PackBits();
}

int operator>(const INT16 &a, const UWORD &bs)
{
  return a.PackBits() > bs;
}

int operator>(const UWORD &as, const INT16 &b)
{
  return as > b.PackBits();
}

int operator>(const INT16 &a, const USWORD &bs)
{
  return a.PackBits() > bs;
}

int operator>(const USWORD &as, const INT16 &b)
{
  return as > b.PackBits();
}

int operator>(const INT16 &a, const SBYTE &bs)
{
  return a.PackBits() > (SWORD)bs;
}

int operator>(const SBYTE &as, const INT16 &b)
{
  return (SWORD)as > b.PackBits();
}

int operator>(const INT16 &a, const UBYTE &bs)
{
  return a.PackBits() > (SWORD)bs;
}

int operator>(const UBYTE &as, const INT16 &b)
{
  return (SWORD)as > b.PackBits();
}

int operator<=(const INT16 &a, const INT16 &b)
{
  return a.PackBits() == b.PackBits() || a.PackBits() < b.PackBits();
}

int operator<=(const INT16 &a, const LWORD &bs)
{
  return a.PackBits() == bs || a.PackBits() < bs;
}

int operator<=(const LWORD &as, const INT16 &b)
{
  return as == b.PackBits() || as < b.PackBits();
}

int operator<=(const INT16 &a, const ULWORD &bs)
{
  return a.PackBits() == bs || a.PackBits() < bs;
}

int operator<=(const ULWORD &as, const INT16 &b)
{
  return as == b.PackBits() || as < b.PackBits();
}

int operator<=(const INT16 &a, const WORD &bs)
{
  return a.PackBits() == bs || a.PackBits() < bs;
}

int operator<=(const WORD &as, const INT16 &b)
{
  return as == b.PackBits() || as < b.PackBits();
}

int operator<=(const INT16 &a, const SWORD &bs)
{
  return a.PackBits() == bs || a.PackBits() < bs;
}

int operator<=(const SWORD &as, const INT16 &b)
{
  return as == b.PackBits() || as < b.PackBits();
}

int operator<=(const INT16 &a, const UWORD &bs)
{
  return a.PackBits() == bs || a.PackBits() < bs;
}

int operator<=(const UWORD &as, const INT16 &b)
{
  return as == b.PackBits() || as < b.PackBits();
}

int operator<=(const INT16 &a, const USWORD &bs)
{
  return a.PackBits() == bs || a.PackBits() < bs;
}

int operator<=(const USWORD &as, const INT16 &b)
{
  return as == b.PackBits() || as < b.PackBits();
}

int operator<=(const INT16 &a, const SBYTE &bs)
{
  return a.PackBits() == (SWORD)bs ||  a.PackBits() < (SWORD)bs;
}

int operator<=(const SBYTE &as, const INT16 &b)
{
  return (SWORD)as == b.PackBits() || (SWORD)as < b.PackBits();
}

int operator<=(const INT16 &a, const UBYTE &bs)
{
  return a.PackBits() == (SWORD)bs ||  a.PackBits() < (SWORD)bs;
}

int operator<=(const UBYTE &as, const INT16 &b)
{
  return (SWORD)as == b.PackBits() || (SWORD)as < b.PackBits();
}

int operator>=(const INT16 &a, const INT16 &b)
{
  return a.PackBits() == b.PackBits() || a.PackBits() > b.PackBits();
}

int operator>=(const INT16 &a, const LWORD &bs)
{
  return a.PackBits() == bs || a.PackBits() > bs;
}

int operator>=(const LWORD &as, const INT16 &b)
{
  return as == b.PackBits() || as > b.PackBits();
}

int operator>=(const INT16 &a, const ULWORD &bs)
{
  return a.PackBits() == bs || a.PackBits() > bs;
}

int operator>=(const ULWORD &as, const INT16 &b)
{
  return as == b.PackBits() || as > b.PackBits();
}

int operator>=(const INT16 &a, const WORD &bs)
{
  return a.PackBits() == bs || a.PackBits() > bs;
}

int operator>=(const WORD &as, const INT16 &b)
{
  return as == b.PackBits() || as > b.PackBits();
}

int operator>=(const INT16 &a, const SWORD &bs)
{
  return a.PackBits() == bs || a.PackBits() > bs;
}

int operator>=(const SWORD &as, const INT16 &b)
{
  return as == b.PackBits() || as > b.PackBits();
}

int operator>=(const INT16 &a, const UWORD &bs)
{
  return a.PackBits() == bs || a.PackBits() > bs;
}

int operator>=(const UWORD &as, const INT16 &b)
{
  return as == b.PackBits() || as > b.PackBits();
}

int operator>=(const INT16 &a, const USWORD &bs)
{
  return a.PackBits() == bs || a.PackBits() > bs;
}

int operator>=(const USWORD &as, const INT16 &b)
{
  return as == b.PackBits() || as > b.PackBits();
}

int operator>=(const INT16 &a, const SBYTE &bs)
{
  return a.PackBits() == (SWORD)bs || a.PackBits() > (SWORD)bs;
}

int operator>=(const SBYTE &as, const INT16 &b)
{
  return (SWORD)as == b.PackBits() || (SWORD)as > b.PackBits();
}

int operator>=(const INT16 &a, const UBYTE &bs)
{
  return a.PackBits() == (SWORD)bs || a.PackBits() > (SWORD)bs;
}

int operator>=(const UBYTE &as, const INT16 &b)
{
  return (SWORD)as == b.PackBits() || (SWORD)as > b.PackBits();
}
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
