// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: uint16.cpp 
// Compiler Used: MSVC40, DJGPP 2.7.2.1, GCC 2.7.2.1, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 09/05/97  
// Date Last Modified: 12/03/97
// Copyright (c) 1997 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBD C++ classes are copyright (c) 1997, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The UINT16 class is used to represent 16 bit unsigned integers
independently of the operating system or hardware platform
used. It was created to overcome the big and little endian
problem encountered when writing integer values to a common
database file accessed by several different types of machines.

The term "endian" is used to describe the order in which
multibyte numbers are stored in the computers's memory. This
is specifc to each microprocessor. The Intel x86 family is
little-endian, meaning the lowest-order byte is stored first.
Hewlett-Packard's PA-RISC and Sun's SuperSPARC are big-endian,
meaning the highest-order byte is stored first. The Silicon
Graphics MIPS and IBM/Motorola PowerPC processors are both
little and big endian (bi-endian).

 Multibyte Value     Big-Endian    Little-Endian
   0x12345678        0x12345678    0x78563412
   0x1234            0x1234        0x3412
   0x5678            0x5678        0x7856
   "ABC"             41 42 42 00   41 42 43 00

With big-endian ordering, the address of the multibyte value
is its most significant byte (its big end). With little-endian
ordering, the address is its least significant byte (its little
end). In a struct the order of bytes in memory will differ
depending on the byte ordering and the particular data type.
*/
// ----------------------------------------------------------- // 
#include <string.h>
#include <memory.h>
#include "uint16.hpp"
#include "ehandler.hpp"

UINT16::UINT16(USWORD val)
{
  UnPackBits(val);
}

UINT16::UINT16(const UINT16& ob)
{
  memmove((void *)byte, (const void *)ob.byte, 2);
}

UINT16& UINT16::operator=(const UINT16& ob)
{
  memmove((void *)byte, (const void *)ob.byte, 2);
  return *this;
}

UINT16& UINT16::operator=(const USWORD val)
{
  UnPackBits(val);
  return *this;
}

UINT16::operator USWORD() const
{
  return PackBits();
}

USWORD UINT16::PackBits() const
{
  USWORD a, b;
  
  a = (USWORD)byte[0];
  b = (USWORD)byte[1];

  a = a & 0xFF;
  b = (b<<8) & 0xFF00;

  return a + b;
}

void UINT16::UnPackBits(USWORD val)
{
  byte[0] = val & 0xFF;
  byte[1] = (val & 0xFF00)>>8;
}

UINT16 UINT16::operator++(int) // Postfix
{
  UINT16 val_before(*this); 
  operator=(*this + 1);
  return val_before;
}
 
UINT16 UINT16::operator--(int) // Postfix
{
  UINT16 val_before(*this); 
  operator=(*this - 1);
  return val_before;
}

void UINT16::operator/=(const UINT16 &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void UINT16::operator/=(const LWORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

    operator=(*this / i);
}

void UINT16::operator/=(const ULWORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void UINT16::operator/=(const WORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void UINT16::operator/=(const SWORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void UINT16::operator/=(const UWORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void UINT16::operator/=(const USWORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void UINT16::operator/=(const SBYTE &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / (USWORD)i);
}

void UINT16::operator/=(const UBYTE &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / (USWORD)i);
}

int operator==(const UINT16 &a, const UINT16 &b)
{
  return a.PackBits() == b.PackBits();
}

int operator==(const UINT16 &a, const LWORD &bs)
{
  return a.PackBits() == bs;
}

int operator==(const LWORD &as, const UINT16 &b)
{
  return as == b.PackBits(); 
}

int operator==(const UINT16 &a, const ULWORD &bs)
{
  return a.PackBits() == bs;
}

int operator==(const ULWORD &as, const UINT16 &b)
{
  return as == b.PackBits(); 
}

int operator==(const UINT16 &a, const WORD &bs)
{
  return a.PackBits() == bs;
}

int operator==(const WORD &as, const UINT16 &b)
{
  return as == b.PackBits();
}

int operator==(const UINT16 &a, const SWORD &bs)
{
  return a.PackBits() == bs;
}

int operator==(const SWORD &as, const UINT16 &b)
{
  return as == b.PackBits(); 
}

int operator==(const UINT16 &a, const UWORD &bs)
{
  return a.PackBits() == bs;
}

int operator==(const UWORD &as, const UINT16 &b)
{
  return as == b.PackBits(); 
}

int operator==(const UINT16 &a, const USWORD &bs)
{
  return  a.PackBits() == bs;
}

int operator==(const USWORD &as, const UINT16 &b)
{
  return as == b.PackBits(); 
}

int operator==(const UINT16 &a, const SBYTE &bs)
{
  return a.PackBits() == (USWORD)bs;
}

int operator==(const SBYTE &as, const UINT16 &b)
{
  return (USWORD)as == b.PackBits(); 
}

int operator==(const UINT16 &a, const UBYTE &bs)
{
  return a.PackBits() == (USWORD)bs;
}

int operator==(const UBYTE &as, const UINT16 &b)
{
  return (USWORD)as == b.PackBits(); 
}

int operator!=(const UINT16 &a, const UINT16 &b)
{
  return a.PackBits() != b.PackBits();
}

int operator!=(const UINT16 &a, const LWORD &bs)
{
  return a.PackBits() != bs;
}

int operator!=(const LWORD &as, const UINT16 &b)
{
  return as != b.PackBits();
}

int operator!=(const UINT16 &a, const ULWORD &bs)
{
  return a.PackBits() != bs;
}

int operator!=(const ULWORD &as, const UINT16 &b)
{
  return as != b.PackBits();
}

int operator!=(const UINT16 &a, const WORD &bs)
{
  return a.PackBits() != bs;
}

int operator!=(const WORD &as, const UINT16 &b)
{
  return as != b.PackBits();
}

int operator!=(const UINT16 &a, const SWORD &bs)
{
  return a.PackBits() != bs;
}

int operator!=(const SWORD &as, const UINT16 &b)
{
  return as != b.PackBits();
}

int operator!=(const UINT16 &a, const UWORD &bs)
{
  return a.PackBits() != bs;
}

int operator!=(const UWORD &as, const UINT16 &b)
{
  return as != b.PackBits();
}

int operator!=(const UINT16 &a, const USWORD &bs)
{
  return a.PackBits() != bs;
}

int operator!=(const USWORD &as, const UINT16 &b)
{
  return as != b.PackBits();
}

int operator!=(const UINT16 &a, const SBYTE &bs)
{
  return a.PackBits() != (USWORD)bs;
}

int operator!=(const SBYTE &as, const UINT16 &b)
{
  return (USWORD)as != b.PackBits();
}

int operator!=(const UINT16 &a, const UBYTE &bs)
{
  return a.PackBits() != (USWORD)bs;
}

int operator!=(const UBYTE &as, const UINT16 &b)
{
  return (USWORD)as != b.PackBits();
}

int operator<(const UINT16 &a, const UINT16 &b)
{
  return a.PackBits() < b.PackBits();
}

int operator<(const UINT16 &a, const LWORD &bs)
{
  return a.PackBits() < bs;
}

int operator<(const LWORD &as, const UINT16 &b)
{
  return as < b.PackBits();
}

int operator<(const UINT16 &a, const ULWORD &bs)
{
  return a.PackBits() < bs;
}

int operator<(const ULWORD &as, const UINT16 &b)
{
  return as < b.PackBits();
}

int operator<(const UINT16 &a, const WORD &bs)
{
  return a.PackBits() < bs;
}

int operator<(const WORD &as, const UINT16 &b)
{
  return as < b.PackBits();
}

int operator<(const UINT16 &a, const SWORD &bs)
{
  return a.PackBits() < bs;
}

int operator<(const SWORD &as, const UINT16 &b)
{
  return as < b.PackBits();
}

int operator<(const UINT16 &a, const UWORD &bs)
{
  return a.PackBits() < bs;
}

int operator<(const UWORD &as, const UINT16 &b)
{
  return as < b.PackBits();
}

int operator<(const UINT16 &a, const USWORD &bs)
{
  return a.PackBits() < bs;
}

int operator<(const USWORD &as, const UINT16 &b)
{
  return as < b.PackBits();
}

int operator<(const UINT16 &a, const SBYTE &bs)
{
  return a.PackBits() < (USWORD)bs;
}

int operator<(const SBYTE &as, const UINT16 &b)
{
  return (USWORD)as < b.PackBits();
}

int operator<(const UINT16 &a, const UBYTE &bs)
{
  return a.PackBits() < (USWORD)bs;
}

int operator<(const UBYTE &as, const UINT16 &b)
{
  return (USWORD)as < b.PackBits();
}

int operator>(const UINT16 &a, const UINT16 &b)
{
  return a.PackBits() > b.PackBits();
}

int operator>(const UINT16 &a, const LWORD &bs)
{
  return a.PackBits() > bs;
}

int operator>(const LWORD &as, const UINT16 &b)
{
  return as > b.PackBits();
}

int operator>(const UINT16 &a, const ULWORD &bs)
{
  return a.PackBits() > bs;
}

int operator>(const ULWORD &as, const UINT16 &b)
{
  return as > b.PackBits();
}

int operator>(const UINT16 &a, const WORD &bs)
{
  return a.PackBits() > bs;
}

int operator>(const WORD &as, const UINT16 &b)
{
  return as > b.PackBits();
}

int operator>(const UINT16 &a, const SWORD &bs)
{
  return a.PackBits() > bs;
}

int operator>(const SWORD &as, const UINT16 &b)
{
  return as > b.PackBits();
}

int operator>(const UINT16 &a, const UWORD &bs)
{
  return a.PackBits() > bs;
}

int operator>(const UWORD &as, const UINT16 &b)
{
  return as > b.PackBits();
}

int operator>(const UINT16 &a, const USWORD &bs)
{
  return a.PackBits() > bs;
}

int operator>(const USWORD &as, const UINT16 &b)
{
  return as > b.PackBits();
}

int operator>(const UINT16 &a, const SBYTE &bs)
{
  return a.PackBits() > (USWORD)bs;
}

int operator>(const SBYTE &as, const UINT16 &b)
{
  return (USWORD)as > b.PackBits();
}

int operator>(const UINT16 &a, const UBYTE &bs)
{
  return a.PackBits() > (USWORD)bs;
}

int operator>(const UBYTE &as, const UINT16 &b)
{
  return (USWORD)as > b.PackBits();
}

int operator<=(const UINT16 &a, const UINT16 &b)
{
  return a.PackBits() == b.PackBits() || a.PackBits() < b.PackBits();
}

int operator<=(const UINT16 &a, const LWORD &bs)
{
  return a.PackBits() == bs || a.PackBits() < bs;
}

int operator<=(const LWORD &as, const UINT16 &b)
{
  return as == b.PackBits() || as < b.PackBits();
}

int operator<=(const UINT16 &a, const ULWORD &bs)
{
  return a.PackBits() == bs || a.PackBits() < bs;
}

int operator<=(const ULWORD &as, const UINT16 &b)
{
  return as == b.PackBits() || as < b.PackBits();
}

int operator<=(const UINT16 &a, const WORD &bs)
{
  return a.PackBits() == bs || a.PackBits() < bs;
}

int operator<=(const WORD &as, const UINT16 &b)
{
  return as == b.PackBits() || as < b.PackBits();
}

int operator<=(const UINT16 &a, const SWORD &bs)
{
  return a.PackBits() == bs || a.PackBits() < bs;
}

int operator<=(const SWORD &as, const UINT16 &b)
{
  return as == b.PackBits() || as < b.PackBits();
}

int operator<=(const UINT16 &a, const UWORD &bs)
{
  return a.PackBits() == bs || a.PackBits() < bs;
}

int operator<=(const UWORD &as, const UINT16 &b)
{
  return as == b.PackBits() || as < b.PackBits();
}

int operator<=(const UINT16 &a, const USWORD &bs)
{
  return a.PackBits() == bs || a.PackBits() < bs;
}

int operator<=(const USWORD &as, const UINT16 &b)
{
  return as == b.PackBits() || as < b.PackBits();
}

int operator<=(const UINT16 &a, const SBYTE &bs)
{
  return a.PackBits() == (USWORD)bs ||  a.PackBits() < (USWORD)bs;
}

int operator<=(const SBYTE &as, const UINT16 &b)
{
  return (USWORD)as == b.PackBits() || (USWORD)as < b.PackBits();
}

int operator<=(const UINT16 &a, const UBYTE &bs)
{
  return a.PackBits() == (USWORD)bs ||  a.PackBits() < (USWORD)bs;
}

int operator<=(const UBYTE &as, const UINT16 &b)
{
  return (USWORD)as == b.PackBits() || (USWORD)as < b.PackBits();
}

int operator>=(const UINT16 &a, const UINT16 &b)
{
  return a.PackBits() == b.PackBits() || a.PackBits() > b.PackBits();
}

int operator>=(const UINT16 &a, const LWORD &bs)
{
  return a.PackBits() == bs || a.PackBits() > bs;
}

int operator>=(const LWORD &as, const UINT16 &b)
{
  return as == b.PackBits() || as > b.PackBits();
}

int operator>=(const UINT16 &a, const ULWORD &bs)
{
  return a.PackBits() == bs || a.PackBits() > bs;
}

int operator>=(const ULWORD &as, const UINT16 &b)
{
  return as == b.PackBits() || as > b.PackBits();
}

int operator>=(const UINT16 &a, const WORD &bs)
{
  return a.PackBits() == bs || a.PackBits() > bs;
}

int operator>=(const WORD &as, const UINT16 &b)
{
  return as == b.PackBits() || as > b.PackBits();
}

int operator>=(const UINT16 &a, const SWORD &bs)
{
  return a.PackBits() == bs || a.PackBits() > bs;
}

int operator>=(const SWORD &as, const UINT16 &b)
{
  return as == b.PackBits() || as > b.PackBits();
}

int operator>=(const UINT16 &a, const UWORD &bs)
{
  return a.PackBits() == bs || a.PackBits() > bs;
}

int operator>=(const UWORD &as, const UINT16 &b)
{
  return as == b.PackBits() || as > b.PackBits();
}

int operator>=(const UINT16 &a, const USWORD &bs)
{
  return a.PackBits() == bs || a.PackBits() > bs;
}

int operator>=(const USWORD &as, const UINT16 &b)
{
  return as == b.PackBits() || as > b.PackBits();
}

int operator>=(const UINT16 &a, const SBYTE &bs)
{
  return a.PackBits() == (USWORD)bs || a.PackBits() > (USWORD)bs;
}

int operator>=(const SBYTE &as, const UINT16 &b)
{
  return (USWORD)as == b.PackBits() || (USWORD)as > b.PackBits();
}

int operator>=(const UINT16 &a, const UBYTE &bs)
{
  return a.PackBits() == (USWORD)bs || a.PackBits() > (USWORD)bs;
}

int operator>=(const UBYTE &as, const UINT16 &b)
{
  return (USWORD)as == b.PackBits() || (USWORD)as > b.PackBits();
}
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
