// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: testprog.cpp 
// Compiler Used: MSVC40, DJGPP 2.7.2.1, GCC 2.7.2.1, HP CPP 10.24
// Produced By: Doug Gaer  
// File Creation Date: 09/18/97  
// Date Last Modified: 12/03/97
// Copyright (c) 1997 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
This is a test program use to test the (P)ersistent base class.

The VBD C++ classes are copyright (c) 1997, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.
*/
// ----------------------------------------------------------- // 
#include "part.hpp"

const char *FileName = "part";


void pause()
{
  cout << endl;
  cout << "Press enter to continue..." << endl;
  cin.get();
}

void SkipToEol(istream &s)
// Used to clear istream
{
  char c;
  s.clear();
  while(s.get(c) && c != '\n') { ; }
}

int Quit()
{
  cout << "Exiting..." << endl;
  return 0;
}

void DisplayStats(const VBDFilePtr &f)
// Passing VBDFilePtr by reference to avoid calling the Refcount
// copy constructor. This will avoid any unnecessary aliasing.
{
  const char *FName = f->VBDFileName();
  SBYTE FileStatus = f->GetFileStatus(); // Get the current file status
  cout << "----- VBD file statistics -----" << endl;
  cout << endl;
  cout << "----- Static information -----" << endl;
  cout << "VBD File Name:          " << FName << endl;
  cout << "VBDFile Signature:      " << f->GetSignature() << endl; 
  cout << "VBDFile Version Number: " << f->GetVersion() << endl;
  cout << "File Header Size :      " << f->FileHeaderSize() << endl;
  cout << "VB Header Size:         " << f->VBHeaderSize() << endl;
  cout << "Static area size:       " << f->StaticArea() << endl;
  cout << "Static file size:       " << f->FileSize(FName) << endl;
  cout << endl;
  cout << "----- Dynamic Information -----" << endl;
  cout << "File status Byte:       ";
  if(FileStatus & 0x01) cout << "Good "; else cout << "Bad ";
  if(FileStatus & 0x02) cout << "Open "; else cout << "Closed ";
  if(FileStatus & 0x04) cout << "Read/Write "; else cout << "Read/Only ";
  cout << endl;
  cout << "VBDFile Free Space:     " << f->GetFreeSpace() << endl;
  cout << "VBDFile End of File:    " << f->GetEOF() << endl;
  cout << "VBDFile Heap Start:     " << f->GetHeapStart() << endl;
  cout << "VBDFile Highest VB:     " << f->GetHighestVB() << endl;
  cout << "Total VBs in file:      " << f->VBTotal() << endl;
  cout << "Total deleted VBs:      " << f->VBDeleted() << endl;
  pause();
}
    
void InputData(UString &s)
{
  cout << "Enter the name of the part:    ";
  cin >> s;
}

void DisplayPart(Part &part)
{
  cout << endl;
  cout << "Item's Name  = " << part.GetName() << endl;
  cout << "Stock Number = " << part.GetID() << endl;
  cout << "Price        = $" << part.GetPrice() << endl;
  cout << endl;
}

void AddPart(const POD *DB)
{
  UString buf;
  InputData(buf);

  Part part(DB);
  part.SetName(buf);
  FAU exists = part.FindObject();

  if(exists) {
    cout << "Part: " << buf << " already exists at address "
	 << exists << endl;
    return;
  }

  int id;
  double price;

  cout << "Enter the part's stock number: ";
  cin >> id;
  if(cin) {
    part.SetID(id);
    cout << "Enter the part's price:       $";
    cin >> price;
  } 
  else {
    cout << "Invalid entry. Object not added!" << endl;
    return;
  }
  if(cin) {
    part.SetPrice(price);
  }
  else {
    cout << "Invalid entry. Object not added!" << endl;
    return;
  }
  
  SkipToEol(cin);
  
  FAU addr = part.AddObject();
  if(!addr) {
    cout << "Could not add object to database" << endl;
    return;
  }
  return;
}

void Delete(const POD *DB)
{
  UString buf;
  InputData(buf);

  Part part(DB);
  part.SetName(buf);
  FAU addr = part.DeleteObject();
  
  if(!addr) {
    cout << "Could not find part: " << buf << " in the database" << endl;
    return;
  }

  cout << "Deleted part: " << buf << " at address " << addr << endl;
}

void Remove(const POD *DB)
{
  UString buf;
  InputData(buf);

  Part part(DB);
  part.SetName(buf);
  FAU addr = part.RemoveObject();
  
  if(!addr) {
    cout << "Could not find part: " << buf << " in the database" << endl;
    return;
  }

  cout << "Deleted part: " << buf << " at address " << addr << endl;
}

void ListInOrder(const POD *DB)
{
  FAU addr = 0;
  Part part(DB);
  unsigned count = 0;
  SkipToEol(cin); // Clear input stream
  while(1)
    {
      addr = DB->OpenDatabase()->FindFirstObject(addr);
      if(!addr) break;
      part.ReadObject(addr);
      DisplayPart(part);
      count++;
      if(count==2) {
	pause();
	count = 0;
      }
    }
}

void FindPart(const POD *DB)
{
  UString buf;
  InputData(buf);

  Part part(DB);
  part.SetName(buf);
  FAU addr = part.FindObject();
  
  if(!addr) {
    cout << "Could not find part: " << buf << " in the database" << endl;
    return;
  }

  cout << "Found part " << buf << " at address " << addr << endl;
  part.ReadObject(addr);
  DisplayPart(part);
}

void Menu(void)
{
  cout << "(A)   Add object to the database" << endl;
  cout << "(D)   Delete object from the database" << endl;
  cout << "(F)   Find part by name" << endl;          
  cout << "(L)   List objects in order" << endl; 
  cout << "(S)   Display VBD file stats" << endl;
  cout << "(H)   Help (prints this menu)" << endl;
  cout << "(Q)   Quit" << endl;
  cout << "(R)   Remove object object from the database" << endl;
}

int main()
{
  UString FN(FileName);
  POD pod(FN+".pod");
  POD *DB = &pod;

  char key;
  Menu();
  
  int rv = 1;
  while(rv) {
    if (!cin) { // Input is in fail state
      SkipToEol(cin); // Go to end of line
      if (!cin) {  // Can't fix
	cout << "Input stream is broken" << endl;
	return 0;
      }
    }
    cout << '>';
    cin >> key;
    if (!cin) continue; // Fix at top of loop
    switch(key) {
      case 'a' : case 'A' : SkipToEol(cin); AddPart(DB); break;
      case 'd' : case 'D' : Delete(DB); break;
      case 'f' : case 'F' : FindPart(DB); break;	
      case 'l' : case 'L' : ListInOrder(DB); break;	 
      case 's' : case 'S' : DisplayStats(DB->OpenDatabase()); break;	
      case 'h' : case 'H' : Menu(); break;
      case 'q' : case 'Q' : rv = Quit(); break;
      case 'r' : case 'R' : Remove(DB); break;
	
      default:
        cout << "Unrecognized command" << endl;
    }
  }
  return 0;
}
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
