// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: testprog.cpp 
// Compiler Used: MSVC40, DJGPP 2.7.2.1, GCC 2.7.2.1, HP CPP 10.24
// Produced By: Doug Gaer 
// File Creation Date: 02/04/97  
// Date Last Modified: 12/04/97
// Copyright (c) 1997 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
This is a test program for the VBDFile class.

The VBD C++ classes are copyright (c) 1997, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.
*/
// ----------------------------------------------------------- // 
#include <iostream.h>
#include "vbdfile.hpp"
#include "float64.hpp"

class Part
{ 
public:
  Part(INT32 i = 0, FLOAT64 p = 0) { id = i; price = p; }

public:
  INT32 id; 
  FLOAT64 price;
};

void pause()
{
  cout << endl;
  cout << "Press enter to continue..." << endl;
  cin.get();
}

void DisplayStats(const VBDFilePtr &f)
// Passing VBDFilePtr by reference to avoid calling the Refcount
// copy constructor. This will avoid any unnecessary aliasing.
{
  const char *FName = f->VBDFileName();
  SBYTE FileStatus = f->GetFileStatus(); // Get the current file status
  cout << "----- VBD file statistics -----" << endl;
  cout << endl;
  cout << "----- Static information -----" << endl;
  cout << "VBD File Name:          " << FName << endl;
  cout << "VBDFile Signature:      " << f->GetSignature() << endl; 
  cout << "VBDFile Version Number: " << f->GetVersion() << endl;
  cout << "File Header Size :      " << f->FileHeaderSize() << endl;
  cout << "VB Header Size:         " << f->VBHeaderSize() << endl;
  cout << "Static area size:       " << f->StaticArea() << endl;
  cout << "Static file size:       " << f->FileSize(FName) << endl;
  cout << endl;
  cout << "----- Dynamic Information -----" << endl;
  cout << "File status Byte:       ";
  if(FileStatus & 0x01) cout << "Good "; else cout << "Bad ";
  if(FileStatus & 0x02) cout << "Open "; else cout << "Closed ";
  if(FileStatus & 0x04) cout << "Read/Write "; else cout << "Read/Only ";
  cout << endl;
  cout << "VBDFile Free Space:     " << f->GetFreeSpace() << endl;
  cout << "VBDFile End of File:    " << f->GetEOF() << endl;
  cout << "VBDFile Heap Start:     " << f->GetHeapStart() << endl;
  cout << "VBDFile Highest VB:     " << f->GetHighestVB() << endl;
  cout << "Total VBs in file:      " << f->VBTotal() << endl;
  cout << "Total deleted VBs:      " << f->VBDeleted() << endl;
  pause();
}

void VBStats(const VBDFilePtr &f, FAU ObjectAddress)
// Passing VBDFilePtr by reference to avoid calling the Refcount
// copy constructor. This will avoid any unnecessary aliasing.
{
  VBHeader vb;
  FAU VBAddress = ObjectAddress - sizeof(VBHeader);
  f->Read(&vb, sizeof(VBHeader), VBAddress);

  cout << endl;
  cout << "----- (V)ariable (D)ata (B)lock statistics -----" << endl;
  cout << "vb check word = " << vb.CkWord << endl;
  cout << "vb length     = " << vb.Length << endl;
  cout << "vb status     = " << (SBYTE)vb.Status << endl;
  cout << "vb NDB        = " << vb.NextDeletedVB << endl;
  cout << endl;
  cout << "----- Object statistics -----" << endl;
  cout << "Object length  = " << vb.Length - sizeof(VBHeader) << endl;
  cout << endl;
}
    
int main()
{
  VBDFilePtr f(new VBDFile);   // Should always create dynamically
  const char *FName = "testfile.vbd";
  int exists;
  
  if(!VBDFile::Exists(FName)) {
    cout << "Creating new file..." << endl;
    f->Create(FName);
    exists = 0;
    cout << "Reference count = " << f->Count() << endl;
  }
  else {
    cout << "Opening existing file..." << endl;
    f->Open(FName);
    exists = 1;
    cout << "Reference count = " << f->Count() << endl;
  }
  
  pause();
  DisplayStats(f);

  FAU addr;
  
  if(!exists) {
    cout << "Adding object to VBD file..." << endl;
    Part part(17, 42.97);   // In-memory copy of part to be stored
    cout << "Size of object = " << sizeof(Part) << endl;
    addr = f->Alloc(sizeof(Part)); // Allocate room for part #1
    f->Write(&part, sizeof(Part), addr);
    cout << "Reference count = " << f->Count() << endl;
    VBStats(f, addr);
    pause();
    cout << "Deleting the object from the VBD file..." << endl;
    f->Delete(addr);
    cout << "Reference count = " << f->Count() << endl;
  }
  else {
    f->Open(FName); // Open() closes first
    cout << "Reference count = " << f->Count() << endl;
    Part buf;
    cout << "Reading the first object from the VBD file..." << endl;
    addr = f->FindFirstVB();
    f->Read(&buf, sizeof(Part), addr + sizeof(VBHeader));
    cout << "Reference count = " << f->Count() << endl;
    cout << "Part: <" << buf.id << ", " << buf.price << ">" << endl;
  }
  
  pause();
  
  DisplayStats(f);

  cout << "Testing vb find functions..." << endl;

  FAU vb = f->FindFirstVB();
  FAU obj = f->FindFirstObject(); // First object that has not been deleted
  if(vb) cout << "First vb address =      " << vb << endl;
  if(obj) cout << "First Object's address = " << obj << endl;
  
  vb = f->FindNextVB(vb); 
  obj = f->FindNextObject(vb); // Next object that has not been deleted
  if(vb) cout << "Next vb address =       " << vb << endl;
  if(obj) cout << "Next Object's address =  " << obj << endl;

  cout << endl;
  cout << "Closing file automatically using VBDFile destructor..." << endl;
  return 0; 
}
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
