// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: ehandler.cpp (terminal interface version)
// Compiler Used: MSVC40, DJGPP 2.7.2.1, GCC 2.7.2.1, HP CPP 10.24
// Produced By: Doug Gaer  
// File Creation Date: 02/14/96  
// Date Last Modified: 12/04/97
// Copyright (c) 1997 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBD C++ classes are copyright (c) 1997, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The Ehandler class is used to catch program exceptions that
occur at run-time. This implementation can be used with or
with out C++ built-in exception handling. If C++ exception
handling is turned off with the CPP_EXCEPTIONS macro then
the EHandler class can be used to display program errors,
exit the program, or trap a program error with a user defined
Action Function.

This version has been modified to work the independent terminal
interface in both UNIX and DOS.
*/
// ----------------------------------------------------------- // 
#include <stdlib.h>
#include "ehandler.hpp"
#include "terminal.hpp"

EHandler ExceptionHandler;
EHandler *Error = &ExceptionHandler; // Global exception handler pointer

const char *ExceptionMessage[MessageCount] = {
    "No exception reported",                     // None
    "File not ready (failed or closed file)",    // FileNotReady
    "Could not write to file",                   // FileNotWriteable
    "Trying to write to a read-only file",       // ReadOnlyFile
    "Trying to use a closed file",               // FileNotOpenError
    "Error creating file",                       // FileCreationError
    "Error opening file",                        // FileOpenError
    "Error closing file",                        // FileCloseError
    "Error seeking in file",                     // FileSeekError
    "Error reading from file",                   // FileReadError
    "Error writing to file",                     // FileWriteError
    "Unexpected end of file",                    // EOFError
    "Wrong file type",                           // WrongFileType
    "File corrupted",                            // FileCorrupt
    "File already exists",                       // FileExists
    "No such file exists",                       // NoFileExists
    "Invalid path",                              // PathError
    "Dangling 'reference counted pointer'",      // DanglingPtr
    "Accessing a null 'smart pointer'",          // NullPtr
    "Cache full",                                // CacheFull
    "Stack full",                                // StkFull
    "Stack empty",                               // StkEmpty
    "Assertion failed",                          // AssertError
    "Zero key length",                           // ZeroLengthKey
    "No database open",                          // NoDatabaseOpen
    "Object not loaded",                         // ObjectNotLoaded
    "Object not saved",                          // ObjectNotSaved
    "Object must be destroyed",                  // MustDestroyObject
    "Object not in constructor",                 // NotInConstructor
    "Object not in destructor",                  // NotInDestructor
    "Object already exists",                     // ObjectExists
    "Bad Object Address",                        // BadObjectAddress
    "Bad key length",                            // BadKeyLength
    "Bad Reference",                             // BadReference
    "Divide By Zero Error"                       // DivideByZero
};

EHandler::~EHandler()
{
  // Destructor provided for virtuality
} 

void EHandler::DisplayException()
{
  terminal->ClearScreen();
  terminal->Write("Caught Exception: ", 0, 0);
  terminal->Write(ExceptionMessage[ExceptionCode]);
  terminal->AnyKey(0, 2);
}

void EHandler::DisplayException(int ECode)
{
  terminal->ClearScreen();
  terminal->Write("Caught Exception: ");
  terminal->Write(ExceptionMessage[ECode]);
  terminal->AnyKey(0, 2);
}

void EHandler::SignalException(int ECode, int Level, int DisplayError)
{
  SetException(ECode); // Set the exception code

  // By default a message will be displayed
  if(DisplayError) DisplayException(); 

  switch(Level)
    {
      case FATAL:    // Fatal error, terminate program immediately
	Terminate();

      case DISPLAY: // Dummy handler that does nothing
	return;

      default:
	Terminate();
    }
}

void EHandler::Terminate()
{
  // Terminate the program abnormally
  terminal->finish();
  exit(FatalErrorLevel);
}

void EHandler::TrapException(AF ActionFunction)
{
  // Call the user defined routine
  (*ActionFunction)();
}
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
