// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- //
// C++ Header File Name: ehandler.hpp (terminal interface version)
// Compiler Used: MSVC40, DJGPP 2.7.2.1, GCC 2.7.2.1, HP CPP 10.24
// Produced By: Doug Gaer    
// File Creation Date: 02/14/96   
// Date Last Modified: 12/04/97
// Copyright (c) 1997 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ---------- Include File Description and Details  ---------- // 
// ----------------------------------------------------------- // 
/*
The VBD C++ classes are copyright (c) 1997, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The Ehandler class is used to catch program exceptions that
occur at run-time. This implementation can be used with or
with out C++ built-in exception handling. If C++ exception
handling is turned off with the CPP_EXCEPTIONS macro then
the EHandler class can be used to display program errors,
exit the program, or trap a program error with a user defined
Action Function.

This version has been modified to work the independent terminal
interface in both UNIX and DOS.
*/
// ----------------------------------------------------------- //   
#ifndef __EHANDLER_HPP
#define __EHANDLER_HPP

// Macro used turn on/off C++'s built-in exception handling. 
// #define CPP_EXCEPTIONS

const int FatalErrorLevel = 1; // Return value to Operating System
const int DisplayOn = 1;       // Display the exception
const int DisplayOff = 0;      // Do not display the exception
const int MessageCount = 35;   // Number of exception messages

typedef void (*AF)(); // Function pointer for action function

class EHandler
{
public:
  enum { // Exception Handler error codes
    None,              // No exception 
    FileNotReady,      // File not ready (failed or closed file)
    FileNotWriteable,  // Could not write to file 
    ReadOnlyFile,      // Trying to write to read-only file
    FileNotOpenError,  // Trying to use a closed file
    FileCreationError, // Error creating file
    FileOpenError,     // Error opening file
    FileCloseError,    // Error closing file
    FileSeekError,     // Error seeking in file
    FileReadError,     // Error reading file
    FileWriteError,    // Error writing to file
    EOFError,          // Unexpected end of file
    WrongFileType,     // Wrong file type
    FileCorrupt,       // File corrupted
    FileExists,        // File already exists
    NoFileExists,      // No such file exists
    PathError,         // Invalid path
    DanglingPtr,       // Dangling reference counted pointer
    NullPtr,           // Accessing a null pointer 
    CacheFull,         // Cache full
    StkFull,           // Stack full
    StkEmpty,          // Stack empty
    AssertError,       // Assertion failed
    ZeroLengthKey,     // Zero length key
    NoDatabaseOpen,    // No database open
    ObjectNotLoaded,   // Object not loaded
    ObjectNotSaved,    // Object not saved
    MustDestroyObject, // Object must be destroyed
    NotInConstructor,  // Object not in class constructor
    NotInDestructor,   // Object not in class destructor
    ObjectExists,      // Object already exists
    BadObjectAddress,  // Bad object address
    BadKeyLength,      // Bad key length
    BadReference,      // Bad Reference
    DivideByZero       // DivideByZeroError
  };

  enum { // Exception Handler error levels
    FATAL = 0x0010,    // Fatal error, terminate program immediately
    DISPLAY = 0x0020   // Dummy handler that does nothing
  };
    
public:
  EHandler() { ExceptionCode = None; }
  virtual ~EHandler();

public:
  virtual void DisplayException();
  virtual void DisplayException(int ECode);

  // Display exception, exit the program, do nothing, or handle exception
  virtual void SignalException 
  (int ECode, int Level = FATAL, int DisplayError = DisplayOn);

  // Exception handler used to handle user defined routines to trap
  // a program error 
  void TrapException(AF ActionFunction);
  
  void SetException(int ECode) { ExceptionCode = ECode; }
  int Exception() { return ExceptionCode; }
  void ClearException() { ExceptionCode = None; }

private:
  virtual void Terminate();
  
private:
  int ExceptionCode;
};

extern EHandler ExceptionHandler;
extern EHandler *Error; // Global exception handler pointer

// Class declarations for exceptions representing program errors. 
// This implementation is provided for use C++'s built-in exception
// handling routines.
class CNone : public EHandler { };              // No exception
class CFileNotReady : public EHandler { };      // File not ready
class CFileNotWriteable : public EHandler { };  // Could not write to file
class CReadOnlyFile : public EHandler { };      // Writing to read-only file
class CFileNotOpenError : public EHandler { };  // Using a closed file
class CFileCreationError : public EHandler { }; // Error creating file
class CFileOpenError : public EHandler { };     // Error opening file
class CFileCloseError : public EHandler { };    // Error closing file
class CFileSeekError : public EHandler { };     // Error seeking in file
class CFileReadError : public EHandler { };     // Error reading file
class CFileWriteError : public EHandler { };    // Error writing to file
class CEOFError : public EHandler { };          // Unexpected end of file
class CWrongFileType : public EHandler { };     // Wrong file type
class CFileCorrupt : public EHandler { };       // File corrupted
class CFileExists : public EHandler { };        // File already exists
class CNoFileExists : public EHandler { };      // No such file exists
class CPathError : public EHandler { };         // Invalid path
class CDanglingPtr : public EHandler { };       // Dangling ref counted ptr
class CNullPtr : public EHandler { };           // Accessing a Null pointer 
class CCacheFull : public EHandler { };         // Cache full
class CStkFull : public EHandler { };           // Stack full
class CStkEmpty : public EHandler { };          // Stack empty
class CAssertError : public EHandler { };       // Assertion failed
class CZeroLengthKey : public EHandler { };     // Zero length key
class CNoDatabaseOpen : public EHandler { };    // No database
class CObjectNotLoaded : public EHandler { };   // Object not loaded
class CObjectNotSaved : public EHandler { };    // Object not saved
class CMustDestroyObject : public EHandler { }; // Must destroy object
class CNotInConstructor : public EHandler { };  // Not in Constructor
class CNotInDestructor : public EHandler { };   // Not in Destructor
class CObjectExists : public EHandler { };      // ObjectExists
class CBadObjectAddress : public EHandler { };  // Bad object address
class CBadKeyLength : public EHandler { };      // Bad key length
class CBadReference : public EHandler { };      // Bad Reference
class CDivideByZero : public EHandler { };      // Divide By Zero Error

#endif  // __EHANDLER_HPP 
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
