// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: main.cpp 
// Compiler Used: MSVC40, DJGPP 2.7.2.1, GCC 2.7.2.1, HP CPP 10.24
// Produced By: Doug Gaer  
// File Creation Date: 09/18/97  
// Date Last Modified: 12/04/97
// Copyright (c) 1997 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBD C++ classes are copyright (c) 1997, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

This is a test program used to test the (P)ersistent base class
in a practical database application.
*/
// ----------------------------------------------------------- // 
#include <fstream.h> // Using ofstream to print to a text file
#include <ctype.h>
#include "rbtree.hpp"
#include "twalk.hpp"
#include "grocery.hpp"
#include "terminal.hpp"
#include "config.hpp"

// This version adds a configuration manager and memory cache that
// keeps a copy of the entire database in memory.
const double VersionNumber = 1.04;

static POD *DB;                // Global database pointer
static RBTree<Grocery> *cache; // Global cache pointer
const int KeyBuf = 25;         // Users input limit per string
const int StringOffset = 15;   // Display output limit per string
const int IntOffset = 8;       // Display output limit per number

// Menu menu setup
typedef void (*MMF) (); // Pointer to (M)ain (M)enu (F)unctions
const char *MMTitle = "------- GROCERY LIST PROGRAM'S MAIN MENU -------" ;
const int MMSelections = 6; // Main menu selections

// Main menu functions
void Exit();
void ISRMenu();     // Insertion/Removal menu
void FindMenu();    // Query menu
void DisplayMenu(); // Display menu
void FileMenu();    // File operations menu
void PrintMenu();   // Print menu

const char *mainmenu[MMSelections] = {
  "(0) - Exit this program",
  "(1) - Add, Remove, or Change items in the database",
  "(2) - Find items in the database",
  "(3) - Display items in the database",
  "(4) - File Operations",
  "(5) - Print Menu"
};

MMF MMFunctions[MMSelections] = {
  &Exit,
  &ISRMenu,
  &FindMenu,
  &DisplayMenu,
  &FileMenu,
  &PrintMenu
};

// Insertion/Removal menu setup
typedef void (*ISRMF) (); // Pointer to (A)dd (M)enu (F)unctions 
const char *ISRMTitle = "--------- DATABASE INSERTION/REMOVAL MENU ---------";
const int ISRMSelections = 5; // Insertion/Removal menu selections

// Insertion/Removal menu functions
void Return();
void Add();
void Change();
void Delete();
void Remove();

const char *isrmenu[ISRMSelections] = {
  "(0) - Return to main menu",
  "(1) - Add items to the database",
  "(2) - Change an item in the database",
  "(3) - Delete an item (keep object intact after deletion)",
  "(4) - Remove an item (permanently remove object)"
};

ISRMF ISRMFunctions[ISRMSelections] = {
  &Return,
  &Add,
  &Change,
  &Delete,
  &Remove
};

// Find menu setup
typedef void (*FMF) (); // Pointer to (F)ind (M)enu (F)unctions 
const char *FMTitle = "---------- DATABASE QUERY MENU ----------";
const int FMSelections = 2; // Find menu selections

// Find menu functions
void Return();
void FindByName();

const char *findmenu[FMSelections] = {
  "(0) - Return to main menu",
  "(1) - Find objects by name"
};

FMF FMFunctions[FMSelections] = {
  &Return,
  &FindByName
};

// Display menu setup
typedef void (*DMF) (); // Pointer to (D)isplay (M)enu (F)unctions 
const char *DMTitle = "---------- DATABASE DISPLAY MENU ----------";
const int DMSelections = 3; // Display menu selections

// Display menu functions
void DisplayLBL(); // Display all objects line by line
void DisplayAll(); // Display all objects one at a time
  
const char *displaymenu[DMSelections] = {
  "(0) - Return to main menu",
  "(1) - Display all items line by line",
  "(2) - Display all items one a time"
};

DMF DMFunctions[DMSelections] = {
  &Return,
  &DisplayLBL,
  &DisplayAll
};

// File operations menu setup
typedef void (*FOMF) (); // Pointer to (F)ile (M)enu (F)unctions 
const char *FOMTitle = "---------- DATABASE FILE OPERATIONS MENU ----------";
const int FOMSelections = 3; // File operations menu selections

// File operations menu functions
void FileStats();
void LoadCache();

const char *filemenu[FOMSelections] = {
  "(0) - Return to main menu",
  "(1) - Display Variable Block Database statistics",
  "(2) - Reload memory cache"
};

FOMF FOMFunctions[FOMSelections] = {
  &Return,
  &FileStats,
  &LoadCache
};

// Print menu setup
typedef void (*PMF) (); // Pointer to (P)rint (M)enu (F)unctions 
const char *PMTitle = "---------- DATABASE PRINTING MENU ----------";
const int PMSelections = 2; // Print menu selections

// Print menu functions
void ASCIIPrintAll(); // Print entire database to an ASCII file 
  
const char *printmenu[PMSelections] = {
  "(0) - Return to main menu",
  "(1) - Print entire database to a text file" 
};

PMF PMFunctions[PMSelections] = {
  &Return,
  &ASCIIPrintAll
};

// Function prototypes for functions use by all menu functions
void AddObject(Grocery &grocery, Coords *p);
void ChangeObject(Grocery &grocery, Coords *p, int adding = 0);
void DisplayObject(Grocery &grocery, Coords *p);
void LineByLine(Grocery &grocery, Coords *p);
void DisplayStats(const VBDFilePtr &f);
void DisplayString(const char *s);

// Function prototypes, includes, and constants for print functions
const int PrintCols = 80;
const int PrintRows = 25;
const char *LineFeed = "\n";
const char *FormFeed = "\012";
void FPrint(const char *s, ofstream &stream, int offset, int pos = 0);
void FPrint(const char c, ofstream &stream, int offset, int pos = 0);
void FPrint(int val, ofstream &stream, int offset, int pos = 0);
void FPrint(long val, ofstream &stream, int offset, int pos = 0);
void FPrint(double val, ofstream &stream, int offset, int pos = 0);
void PrintItemBar(ofstream &stream);
void PrintLineByLine(Grocery &grocery, ofstream &stream);

void MainMenu()
{
  int c;
  Coords *p = new Coords(0, 0);

  while(1) {
    p->SetXY(0, 0);
    p->SetX(terminal->Center(MMTitle));
    p->SetY(terminal->ScreenCenter(MMSelections));
    terminal->ClearScreen();
    terminal->Write(MMTitle, p->XPos(), p->YPrev());

    for(int i = 0; i < MMSelections; i++)
      terminal->Write(mainmenu[i], p->XPos(), p->YOffset(1));
    
    terminal->Write("Press the number of your selection >",
		    p->XPos(), p->YOffset(2));
    
    c = terminal->GetChar();
    switch(tolower(c)) {
      case '0': 
	(*MMFunctions[0]) (); // Call the appropriate menu function
	return;
      case '1':
	(*MMFunctions[1]) (); // Call the appropriate menu function
	break;
      case '2':
	(*MMFunctions[2]) (); // Call the appropriate menu function
	break;
      case '3':
	(*MMFunctions[3]) (); // Call the appropriate menu function
	break;
      case '4':
	(*MMFunctions[4]) (); // Call the appropriate menu function
	break;
      case '5':
	(*MMFunctions[5]) (); // Call the appropriate menu function
	break;
      default:
	break;
    }
  }
}

void ISRMenu()
{
  int c;
  Coords *p = new Coords(0, 0);

  while(1) {
    p->SetXY(0, 0);
    p->SetX(terminal->Center(ISRMTitle));
    p->SetY(terminal->ScreenCenter(ISRMSelections));
    terminal->ClearScreen();
    terminal->Write(ISRMTitle, p->XPos(), p->YPrev());

    for(int i = 0; i < ISRMSelections; i++)
      terminal->Write(isrmenu[i], p->XPos(), p->YOffset(1));
    
    terminal->Write("Press the number of your selection >",
		    p->XPos(), p->YOffset(2));
    
    c = terminal->GetChar();
    switch(tolower(c)) {
      case '0': 
	(*ISRMFunctions[0]) (); // Call the appropriate menu function
	return;
      case '1':
	(*ISRMFunctions[1]) (); // Call the appropriate menu function
	break;
      case '2':
	(*ISRMFunctions[2]) (); // Call the appropriate menu function
	break;
      case '3':
	(*ISRMFunctions[3]) (); // Call the appropriate menu function
	break;
      case '4':
	(*ISRMFunctions[4]) (); // Call the appropriate menu function
	break;
      default:
	break;
    }
  }
}

void FindMenu()
{
  int c;
  Coords *p = new Coords(0, 0);

  while(1) {
    p->SetXY(0, 0);
    p->SetX(terminal->Center(FMTitle));
    p->SetY(terminal->ScreenCenter(FMSelections));
    terminal->ClearScreen();
    terminal->Write(FMTitle, p->XPos(), p->YPrev());

    for(int i = 0; i < FMSelections; i++)
      terminal->Write(findmenu[i], p->XPos(), p->YOffset(1));
    
    terminal->Write("Press the number of your selection >",
		    p->XPos(), p->YOffset(2));
    
    c = terminal->GetChar();
    switch(tolower(c)) {
      case '0': 
	(*FMFunctions[0]) (); // Call the appropriate menu function
	return;
      case '1':
	(*FMFunctions[1]) (); // Call the appropriate menu function
	break;
      default:
	break;
    }
  }
}

void DisplayMenu()
{
  int c;
  Coords *p = new Coords(0, 0);

  while(1) {
    p->SetXY(0, 0);
    p->SetX(terminal->Center(DMTitle));
    p->SetY(terminal->ScreenCenter(DMSelections));
    terminal->ClearScreen();
    terminal->Write(DMTitle, p->XPos(), p->YPrev());

    for(int i = 0; i < DMSelections; i++)
      terminal->Write(displaymenu[i], p->XPos(), p->YOffset(1));
    
    terminal->Write("Press the number of your selection >",
		    p->XPos(), p->YOffset(2));
    
    c = terminal->GetChar();
    switch(tolower(c)) {
      case '0': 
	(*DMFunctions[0]) (); // Call the appropriate menu function
	return;
      case '1':
	(*DMFunctions[1]) (); // Call the appropriate menu function
	break;

      case '2':
	(*DMFunctions[2]) (); // Call the appropriate menu function
	break;
	
      default:
	break;
    }
  }
}

void FileMenu()
{
  int c;
  Coords *p = new Coords(0, 0);

  while(1) {
    p->SetXY(0, 0);
    p->SetX(terminal->Center(FOMTitle));
    p->SetY(terminal->ScreenCenter(FOMSelections));
    terminal->ClearScreen();
    terminal->Write(FOMTitle, p->XPos(), p->YPrev());

    for(int i = 0; i < FOMSelections; i++)
      terminal->Write(filemenu[i], p->XPos(), p->YOffset(1));
    
    terminal->Write("Press the number of your selection >",
		    p->XPos(), p->YOffset(2));
    
    c = terminal->GetChar();
    switch(tolower(c)) {
      case '0': 
	(*FOMFunctions[0]) (); // Call the appropriate menu function
	return;
      case '1':
	(*FOMFunctions[1]) (); // Call the appropriate menu function
	break;

      case '2':
	(*FOMFunctions[2]) (); // Call the appropriate menu function
	break;

      default:
	break;
    }
  }
}

void PrintMenu()
{
  int c;
  Coords *p = new Coords(0, 0);

  while(1) {
    p->SetXY(0, 0);
    p->SetX(terminal->Center(PMTitle));
    p->SetY(terminal->ScreenCenter(PMSelections));
    terminal->ClearScreen();
    terminal->Write(PMTitle, p->XPos(), p->YPrev());

    for(int i = 0; i < PMSelections; i++)
      terminal->Write(printmenu[i], p->XPos(), p->YOffset(1));
    
    terminal->Write("Press the number of your selection >",
		    p->XPos(), p->YOffset(2));
    
    c = terminal->GetChar();
    switch(tolower(c)) {
      case '0': 
	(*PMFunctions[0]) (); // Call the appropriate menu function
	return;
      case '1':
	(*PMFunctions[1]) (); // Call the appropriate menu function
	break;
	
      default:
	break;
    }
  }
}

void Exit()
{
  // Nothing to do
}

void Return()
{
  // Nothing to do
}

void DisplayAll()
{
  Grocery grocery(DB);
  Coords *p = new Coords(0, 0);
  int c;
  BNodeBase *nxt; // BSTree base node pointer

  // Using iterator object to walk throught the tree
  TreeWalkerb tw(cache->GetRoot(), INORDER);

  while((nxt = tw.Next()) != 0) {
    terminal->ClearScreen();      
    p->SetXY(0, 0); // Reset x and y positions
    RBNode<Grocery> *ptr = (RBNode<Grocery> *)nxt;
    DisplayObject(ptr->Data, p);
    terminal->Write("(C)-Change, (D)-Delete, (R)-Remove, (X)-Exit",
		    p->XPos(), p->YOffset(2));
    terminal->Write(", Any other key to continue");
    c = terminal->GetChar();
    switch(tolower(c)) {
      case 'c': 
	terminal->ClearScreen();
	p->SetXY(0, 0);
	grocery.Copy(ptr->Data);
	grocery.ReadObject(grocery.ObjectAddress());
	ChangeObject(grocery, p);
	break;
	
      case 'd':
	grocery.Copy(ptr->Data);
	cache->Delete(grocery); // Update the memory cache
	grocery.DeleteObject(grocery.ObjectAddress());
	break;
	
      case 'r':
	grocery.Copy(ptr->Data);
	cache->Delete(grocery); // Update the memory cache
	grocery.RemoveObject(grocery.ObjectAddress());
	break;

      case 'x':
	return;

      default:
	break;
    }
  }
}

void ItemBar(Coords *p)
{
  const char *col1 = "Number";
  const char *col2 = "Name";
  const char *col3 = "Store";
  const char *col4 = "Price";
  const char *col5 = "Quantity";
  const char *col6 = "Buying";
  const char *col7 = "Total";
  
  int x = p->XPos(); 
  terminal->Write(col2, p->XPos(), p->YPos());
  terminal->Write(col3, p->XOffset(StringOffset+1), p->YPos());
  terminal->Write(col4, p->XOffset(StringOffset+1), p->YPos());
  terminal->Write(col5, p->XOffset(IntOffset+1), p->YPos());
  terminal->Write(col6, p->XOffset(IntOffset+1), p->YPos());
  terminal->Write(col7, p->XOffset(IntOffset+1), p->YPos());
  terminal->MoveCursor(0, p->YOffset(1));
  int i = 0;
  while (i++ < terminal->MaxCols()-1) terminal->Write('=');
  p->YOffset(1);
  p->SetX(x);
}

void DisplayLBL()
{
  Grocery grocery(DB);
  int c;
  unsigned count = 0;
  Coords *p = new Coords(0, 0);
  terminal->ClearScreen();
  ItemBar(p);
  BNodeBase *nxt; // BSTree base node pointer

  // Using iterator object to walk throught the tree
  TreeWalkerb tw(cache->GetRoot(), INORDER);

  while((nxt = tw.Next()) != 0) {
    RBNode<Grocery> *ptr = (RBNode<Grocery> *)nxt;
    LineByLine(ptr->Data, p);
    p->YOffset(1);
    count++;
    if(count > 5) {
    terminal->Write("(X)-Exit", p->XPos(), p->YOffset(2));
    terminal->Write(", Any other key to continue");
    c = terminal->GetChar();
    switch(tolower(c)) {
      case 'x':
	return;
	
      default:
	break;
    }
    terminal->ClearScreen();
    p->SetXY(0, 0);
    ItemBar(p);
    count = 0;
    }
  }
  terminal->AnyKey(p->XPos(), p->YOffset(2));
  delete p; // Free Coords pointer
}

void Delete()
{
  terminal->ClearScreen();
  Coords *p = new Coords(0, 0);
  char buf[KeyBuf]; 
  Grocery grocery(DB);
  terminal->Write("Enter the name of the item to delete: ",
		  p->XPos(), p->YPos());
  terminal->GetString(buf);
  grocery.SetName(buf);

  if(!cache->GetMember(grocery)) { // Search the memory cache for the object
    terminal->Write("Could not find item: ", p->XPos(), p->YOffset(2));
    terminal->Write(buf);
    terminal->Write(" in the database");
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    delete p; // Free the Coord pointer
    return;
  }
  
  cache->Delete(grocery); // Update the memory cache
  grocery.DeleteObject(grocery.ObjectAddress());
  
  terminal->Write("Deleted item ", p->XPos(), p->YOffset(2));
  terminal->Write(buf);

  terminal->AnyKey(p->XPos(), p->YOffset(2));
  delete p; // Free Coord pointer
}

void Remove()
{
  terminal->ClearScreen();
  Coords *p = new Coords(0, 0);
  char buf[KeyBuf]; 
  Grocery grocery(DB);
  terminal->Write("Enter the name of the item to remove: ",
		  p->XPos(), p->YPos());
  terminal->GetString(buf);
  grocery.SetName(buf);

  if(!cache->GetMember(grocery)) { // Search the memory cache for the object
    terminal->Write("Could not find item: ", p->XPos(), p->YOffset(2));
    terminal->Write(buf);
    terminal->Write(" in the database");
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    delete p; // Free the Coord pointer
    return;
  }
  
  cache->Delete(grocery); // Update the memory cache
  grocery.RemoveObject(grocery.ObjectAddress());
  
  terminal->Write("Removed item ", p->XPos(), p->YOffset(2));
  terminal->Write(buf);

  terminal->AnyKey(p->XPos(), p->YOffset(2));
  delete p; // Free Coord pointer
}

void FileStats()
{
  DisplayStats(DB->OpenDatabase());
}

void FPrint(const char *s, ofstream &stream, int offset, int pos)
{
  int i = 0;
  int len = strlen(s);
  char *buf = new char[len + 1];
  buf[len] = '\0';
  strcpy(buf, s);
  if(len > offset) len = offset-1; // Subtract 1 to avoid overlaps
  if(pos) for(i = 0; i < pos; i++) stream << ' '; // Position the text
  for(i = 0; i < len; i++) stream << buf[i];
  if(len < offset) {
    offset = offset - len;
    for(i = 0; i < offset; i++) stream << ' ';
  }
  delete[] buf;
}

void FPrint(const char c, ofstream &stream, int offset, int pos)
{
  char s[1];
  s[0] = c;    // Copy char into a string buffer
  s[1] = '\0'; // Null terminate the string
  FPrint(s, stream, offset, pos);
}
  
void FPrint(int val, ofstream &stream, int offset, int pos)
{
  char *buf = new char[offset+1];
  buf[offset] = '\0';
  sprintf(buf, "%d", val);
  FPrint(buf, stream, offset, pos);
  delete[] buf; 
}

void FPrint(long val, ofstream &stream, int offset, int pos)
{
  char *buf = new char[offset+1];
  buf[offset] = '\0';
  sprintf(buf, "%d", val);
  FPrint(buf, stream, offset, pos);
  delete[] buf; 
}

void FPrint(double val, ofstream &stream, int offset, int pos)
{
  char *buf = new char[offset+1];
  buf[offset] = '\0';
  sprintf(buf, "%g", val);
  FPrint(buf, stream, offset, pos);
  delete[] buf; 
}

void PrintItemBar(ofstream &stream)
{
  const char *col1 = "Number";
  const char *col2 = "Name";
  const char *col3 = "Store";
  const char *col4 = "Price";
  const char *col5 = "Quantity";
  const char *col6 = "Buying";
  const char *col7 = "Total";
  int i = 0;

  FPrint(col2, stream, StringOffset+1);
  FPrint(col3, stream, StringOffset+1);
  FPrint(col4, stream, IntOffset+1);
  FPrint(col5, stream, IntOffset+1);
  FPrint(col6, stream, IntOffset+1);
  FPrint(col7, stream, IntOffset+1, 2);
  
  stream << LineFeed;

  i = 0;
  while (i++ < PrintCols-1) stream << '=';
  stream << LineFeed;
}

void PrintLineByLine(Grocery &grocery, ofstream &stream)
{
  int i = 0;
  
  FPrint(grocery.GetName(), stream, StringOffset+1);
  FPrint(grocery.GetStore(), stream, StringOffset+1);
  FPrint(grocery.GetPrice(), stream, IntOffset+1);
  FPrint(grocery.GetQuantity(), stream, IntOffset-4, 4);
  FPrint(grocery.GetPriority(), stream, IntOffset, 4);
  FPrint(grocery.GetTotal(), stream, IntOffset+1);
  stream << LineFeed;
  
  i = 0;
  while (i++ < PrintCols-1) stream << '-';
  stream << LineFeed;
}

void ASCIIPrintAll()
{
  const char *fname = "grocery.txt";
  ofstream stream(fname, ios::out); // Open file and truncate it
  terminal->ClearScreen();
  Coords *p = new Coords(0, 0);
  terminal->Write("Printing the entire database to the: ",
		  p->XPos(), p->YPos());
  terminal->Write(fname);
  terminal->Write(" file");
  
  if(!stream) { // Could not open the stream
    terminal->Write("Could not open the spool file: ",
		    p->XPos(), p->YOffset(2));
    terminal->Write(fname);
    terminal->Write(" for writing.");
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    delete p; // Free the Coords pointer
    return; 
  }

  terminal->Write("Printing...", p->XPos(), p->YOffset(2));
		  
  PrintItemBar(stream);

  Grocery grocery(DB);
  unsigned count = 0;
  BNodeBase *nxt; // BSTree base node pointer

  // Using iterator object to walk throught the tree
  TreeWalkerb tw(cache->GetRoot(), INORDER);

  while((nxt = tw.Next()) != 0) {
    RBNode<Grocery> *ptr = (RBNode<Grocery> *)nxt;
    PrintLineByLine(ptr->Data, stream);
    count++;
    if(count > PrintRows) {
      stream << FormFeed;
      count = 0;
    }
  }
  
  terminal->Write("Finished", p->XPos(), p->YOffset(2));
  terminal->AnyKey(p->XPos(), p->YOffset(2));
}

void Add()
{
  Coords *p = new Coords(0, 0);
  char buf[KeyBuf];
  Grocery grocery(DB);
  terminal->ClearScreen();
  terminal->Write("Enter the name of item to add: ", p->XPos(), p->YPos());
  terminal->GetString(buf);
  grocery.SetName(buf);
  
  if(cache->GetMember(grocery)) { // Search the memory cache for the object
    terminal->Write("Item: ", p->XPos(), p->YOffset(2));
    terminal->Write(buf);
    terminal->Write(" already exists in the database."); 
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    delete p; // Free Coords pointer
    return;
  }

  AddObject(grocery, p);
}

void AddObject(Grocery &grocery, Coords *p)
{
  char buf[KeyBuf];
  double price = 0;
  int quantity = 0;
  char priority = ' ';
  double total = 0;

  terminal->Write("Enter the name of the store: ",
                  p->XPos(), p->YOffset(1));
  terminal->GetString(buf);
  grocery.SetStore(buf);
  terminal->Write("Enter the item's price: $",
                  p->XPos(), p->YOffset(1));
  price = terminal->GetFloat();
  grocery.SetPrice(price);
  terminal->Write("Enter the quantity: ", p->XPos(), p->YOffset(1));
  quantity = terminal->GetInt();
  grocery.SetQuantity(quantity);
  terminal->Write("Purchasing (y/n): ", p->XPos(), p->YOffset(1));
  if(terminal->GetYesNo()) priority = 'Y'; else priority = 'N';
  grocery.SetPriority(priority);
  FLOAT64 LineTotal = grocery.GetPrice() * grocery.GetQuantity();
  grocery.SetTotal(LineTotal);
  terminal->Write("Line total = $", p->XPos(), p->YOffset(1));
  terminal->Write(grocery.GetTotal());
 
  terminal->Write("(A)-Abort (C)-Change, Any other key to accept entry",
		  p->XPos(), p->YOffset(2));
  int c = terminal->GetChar();
  switch(tolower(c)) {
    case 'c': 
      ChangeObject(grocery, p, 1); 
      break;
    case 'a':
      return;
    default:
      break;
  }

  int find = 0;
  FAU addr = grocery.AddObject(find); // Write the object to the file
  if(!addr) {
    terminal->Write("Could not add item to database",
                    p->XPos(), p->YOffset(2));
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    return;
  }
  cache->Add(grocery); // Update the cache
  
  terminal->Write("Item was added to the database",
                  p->XPos(), p->YOffset(2));

  terminal->Write("Add another (y/n)", p->XPos(), p->YOffset(2));
  if(terminal->GetYesNo()) Add(); else return; 
}

void FindByName()
{
  Coords *p = new Coords(0, 0);
  char buf[KeyBuf]; 
  int c;
  terminal->ClearScreen();
  
  Grocery grocery(DB);
  terminal->Write("Enter the name of the item: ", p->XPos(), p->YPos());
  terminal->GetString(buf);
  grocery.SetName(buf);
  
  if(!cache->GetMember(grocery)) { // Search the memory cache for the object
    terminal->Write("Could not find item: ", p->XPos(), p->YOffset(2));
    terminal->Write(buf);
    terminal->Write(" in the database");
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    delete p; // Free the Coord pointer
    return;
  }

  terminal->Write("Found item ", p->XPos(), p->YOffset(2));
  terminal->Write(buf);

  grocery.ReadObject(grocery.ObjectAddress());
  p->YOffset(2);
  DisplayObject(grocery, p);
  terminal->Write("(C)-Change, (D)-Delete, (R)-Remove",
		  p->XPos(), p->YOffset(2));
  terminal->Write(", Any other key to continue");
  c = terminal->GetChar();
  switch(tolower(c)) {
    case 'c': 
      terminal->ClearScreen();
      p->SetXY(0, 0);
      ChangeObject(grocery, p);
      break;
      
    case 'd':
      cache->Delete(grocery); // Update the memory cache
      grocery.DeleteObject(grocery.ObjectAddress());
      break;
      
    case 'r':
      cache->Delete(grocery); // Update the memory cache
      grocery.RemoveObject(grocery.ObjectAddress());
      break;
	
    default:
      break;
  }
  delete p; // Free Coord pointer
}

void Change()
{
  Grocery grocery(DB);
  Coords *p = new Coords(0, 0);
  char buf[KeyBuf];

  terminal->ClearScreen();
  terminal->Write("Enter name of item to change: ", p->XPos(), p->YPos());
  terminal->GetString(buf);
  grocery.SetName(buf);
  
  if(!cache->GetMember(grocery)) { // Search the memory cache for the object
    terminal->Write("Could not find item: ", p->XPos(), p->YOffset(2));
    terminal->Write(buf);
    terminal->Write(" in the database");
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    delete p; // Free the Coord pointer
    return;
  }

  grocery.ReadObject(grocery.ObjectAddress());
  ChangeObject(grocery, p);
  delete p; // Free Coords pointer
}

void ChangeObject(Grocery &grocery, Coords *p, int adding)
{
  char buf[KeyBuf];

  // Buffer the object's data to allow changes to be canceled
  Grocery ob;
  ob.Copy(grocery);

  // Temp buffers used to record changes
  Price pr = 0;
  Quantity qty = 0;
  Priority pri = ' ';
  FLOAT64 LineTotal;
  Grocery changed(DB);
  
  char *prompt = "Press the number of your selection >";  
  int c, x, y, x_prev, y_prev, x_prompt, y_prompt;
  x_prev = p->XPos(); y_prev = p->YPos();

  while(1) {
    p->SetXY(x_prev, y_prev);
    terminal->Write("(0) Cancel change", p->XPos(), p->YOffset(2));
    if(adding) {
    terminal->Write("(1) Item's Name  =  ", p->XPos(), p->YOffset(1));
    terminal->Write(grocery.GetName());
    terminal->ClearLine();
    }
    else {
    terminal->Write("(-) Item's Name  =  ", p->XPos(), p->YOffset(1));
    terminal->Write(grocery.GetName());
    terminal->ClearLine();
    }
    terminal->Write("(2) Store        =  ", p->XPos(), p->YOffset(1));
    terminal->Write(grocery.GetStore());
    terminal->ClearLine();
    terminal->Write("(3) Price        = $", p->XPos(), p->YOffset(1));
    terminal->Write(grocery.GetPrice());
    terminal->ClearLine();
    terminal->Write("(4) Quantity     =  ", p->XPos(), p->YOffset(1));
    terminal->Write(grocery.GetQuantity());
    terminal->ClearLine();
    terminal->Write("(5) Purchasing   =  ", p->XPos(), p->YOffset(1));
    terminal->Write(grocery.GetPriority());
    terminal->ClearLine();
    terminal->Write("(-) Total        =  ", p->XPos(), p->YOffset(1));
    terminal->Write(grocery.GetTotal());
    terminal->ClearLine();
    terminal->Write("(7) Commit changes", p->XPos(), p->YOffset(1));

    terminal->Write(prompt, p->XPos(), p->YOffset(2));
    x_prompt = p->XPos() + strlen(prompt);
    y_prompt = p->YPos();
  
    x = p->XPos();
    y = p->YPos() + 2;

    c = terminal->GetChar();
    switch(tolower(c)) {
      case '0': 
	grocery.Copy(ob);
	return;

      case '1':
	// User not allowed to change name unless adding a new object
	if(!adding) break; 
	terminal->Write("Enter new name: ", x, y);
	terminal->GetString(buf);
	grocery.SetName(buf);
	if(grocery.FindObject()) {
	  terminal->Write("Item already exists", x, y+1);
	  terminal->AnyKey();
	  terminal->ClearLine(x, y+1);
	  terminal->ClearLine(x, y);
	  terminal->MoveCursor(x_prev, y_prev);
	  break;
	}
	terminal->ClearLine(x, y);
	terminal->MoveCursor(x_prev, y_prev);
	break;

      case '2':
	terminal->Write("Enter new store: ", x, y);
	terminal->GetString(buf);
	grocery.SetStore(buf);
	terminal->ClearLine(x, y);
	terminal->MoveCursor(x_prev, y_prev);
	break;

      case '3':
	terminal->Write("Enter new price: $", x, y);
	pr = terminal->GetFloat();
	grocery.SetPrice(pr);
	terminal->ClearLine(x, y);
	terminal->MoveCursor(x_prev, y_prev);
	LineTotal = grocery.GetPrice() * grocery.GetQuantity();
	grocery.SetTotal(LineTotal);
	break;

      case '4':
	terminal->Write("Enter new quantity: ", x, y);
	qty = terminal->GetInt();
	grocery.SetQuantity(qty);
	terminal->ClearLine(x, y);
	terminal->MoveCursor(x_prev, y_prev);
	LineTotal = grocery.GetPrice() * grocery.GetQuantity();
	grocery.SetTotal(LineTotal);
	break;

      case '5':
	terminal->Write("Purchasing (y/n): ", x, y);
	if(terminal->GetYesNo()) pri = 'Y'; else pri = 'N'; 
	grocery.SetPriority(pri);
	terminal->ClearLine(x, y);
	terminal->MoveCursor(x_prev, y_prev);
	break;

      case '6':
	break;
	
      case '7':
	if(adding) return; // New object, no need to reallocate space
	if(grocery.FullCompare(ob)) return; // The object has not changed
	changed.Copy(grocery);  // Make a copy of the changes
	cache->Delete(grocery); // Update the memory cache
	// Remove the original object 
	grocery.DeleteObject(grocery.ObjectAddress()); 
	changed.AddObject(); // Add the changed object back to the database
	cache->Add(changed); // Update the memory cache   
	return;

      default:
	break;
    }
  }
}

void DisplayStats(const VBDFilePtr &f)
// Passing VBDFilePtr by reference to avoid calling the Refcount
// copy constructor. This will avoid any unnecessary aliasing.
{
  terminal->ClearScreen();
  int tab = 5;
  const char *FName = f->VBDFileName();
  SBYTE FileStatus = f->GetFileStatus(); // Get the current file status
  char *s1 = "----- (V)ariable (B)lock (D)atabase file statistics -----";
  terminal->Write(s1, terminal->Center(s1), 0);
  terminal->Write("----- Static information -----", 0, 2);
  terminal->Write("VBD File Name:          ", tab, 3);
  terminal->Write(FName);
  terminal->Write("VBDFile Signature:      ", tab, 4);
  terminal->Write(f->GetSignature());
  terminal->Write("VBDFile Version Number: ", tab, 5);
  terminal->Write(f->GetVersion());
  terminal->Write("File Header Size :      ", tab, 6);
  terminal->Write((int)f->FileHeaderSize());
  terminal->Write("VB Header Size:         ", tab, 7);
  terminal->Write((int)f->VBHeaderSize());
  terminal->Write("Static area size:       ", tab, 8);
  terminal->Write(f->StaticArea());
  terminal->Write("Static file size:       ", tab, 9);
  terminal->Write(f->FileSize(FName));
  terminal->Write("----- Dynamic Information -----", 0, 11);
  terminal->Write("File status Byte:       ", tab, 12);
  if(FileStatus & 0x01)
    terminal->Write("Good ");
  else
    terminal->Write("Bad ");
  if(FileStatus & 0x02)
    terminal->Write("Open ");
  else
    terminal->Write("Closed ");
  if(FileStatus & 0x04)
    terminal->Write("Read/Write ");
  else
    terminal->Write("Read/Only ");
  terminal->Write("VBDFile Free Space:     ", tab, 13);
  terminal->Write(f->GetFreeSpace());
  terminal->Write("VBDFile End of File:    ", tab, 14);
  terminal->Write(f->GetEOF());
  terminal->Write("VBDFile Heap Start:     ", tab, 15);
  terminal->Write(f->GetHeapStart());
  terminal->Write("VBDFile Highest VB:     ", tab, 16);
  terminal->Write(f->GetHighestVB());
  terminal->Write("Total VBs in file:      ", tab, 17);
  terminal->Write((int)f->VBTotal());
  terminal->Write("Total deleted VBs:      ", tab, 18);
  terminal->Write((int)f->VBDeleted());
  terminal->AnyKey(0, 20);
  terminal->ClearScreen();
}

void DisplayObject(Grocery &grocery, Coords *p)
// Display's the complete object with all details.
{
  terminal->Write("Item's Name  =  ", p->XPos(), p->YPos());
  terminal->Write(grocery.GetName());
  terminal->Write("Store        =  ", p->XPos(), p->YOffset(1));
  terminal->Write(grocery.GetStore());
  terminal->Write("Price        = $", p->XPos(), p->YOffset(1));
  terminal->Write(grocery.GetPrice());
  terminal->Write("Quantity     =  ", p->XPos(), p->YOffset(1));
  terminal->Write(grocery.GetQuantity());
  terminal->Write("Purchasing   =  ", p->XPos(), p->YOffset(1));
  terminal->Write(grocery.GetPriority());
  terminal->Write("Line total   =  ", p->XPos(), p->YOffset(1));
  terminal->Write(grocery.GetTotal());
}

void DisplayString(const char *s)
{
  int len = strlen(s);
  int offset = StringOffset;
  char *buf = new char[len + 1];
  buf[len] = '\0';
  strcpy(buf, s);
  if(len < StringOffset) offset = len;
  for(int i = 0; i < offset; i++)
    terminal->Write(buf[i]);
  delete[] buf;
}

void LineByLine(Grocery &grocery, Coords *p)
// Display objects line by line with partial details
{
  int x = p->XPos();

  terminal->MoveCursor(p->XPos(), p->YPos());
  DisplayString(grocery.GetName());
  terminal->MoveCursor(p->XOffset(StringOffset+1), p->YPos()); 
  DisplayString(grocery.GetStore());
  terminal->Write(grocery.GetPrice(),
		  p->XOffset(StringOffset+1), p->YPos()); 
  terminal->Write(grocery.GetQuantity(), p->XOffset(IntOffset+4), p->YPos());
  terminal->Write(grocery.GetPriority(), p->XOffset(IntOffset), p->YPos());
  terminal->Write(grocery.GetTotal(), p->XOffset(IntOffset-1), p->YPos());
  int i = 0;
  terminal->MoveCursor(0, p->YOffset(1));
  while (i++ < terminal->MaxCols()-1) terminal->Write('-');
  p->SetX(x);
}

void LoadCache()
{
  FAU addr;
  Grocery grocery(DB);
  cache->Clear(); // Empty the existing memory buffers
  terminal->ClearScreen();
  const char *mesg = "Loading memory cache...";
  int x = terminal->Center(mesg);
  terminal->Write(mesg, x, terminal->ScreenCenter(1));

  while(1)
    {
      addr = DB->OpenDatabase()->FindFirstObject(addr);
      if(!addr) break;
      grocery.ReadObject(addr);
      cache->Add(grocery);
    }
}

int main(int argc, char **argv)
{
  char *FileName;
  char *DefaultCfgFile = "grocery.ini";
  char *DefaultPODFile = "grocery.pod";
  char *EnvSetting = "GDBCFG"; // Environment setting for CFG file 
  char *CurrentCfgFile;
  char *CfgFile;

  Config *CfgData = new Config;
  
  // Look for CfgFile name in environment
  if((CurrentCfgFile = getenv("GDBCFG")) == 0)
     CfgFile = DefaultCfgFile;
  else
    CfgFile = CurrentCfgFile;

  // Use default file of CFG file name if no file is specifed on 
  if(argc < 2) {
    int FileStatus = CfgData->Load((char *)CfgFile);
    if(!FileStatus)
      FileName = DefaultPODFile;
    else {
      FileName = CfgData->GetStrValue("DBFileName");
      if(!FileName) FileName = DefaultPODFile;
    }
  }
  else {
    FileName = argv[1];
  }

  CfgData->UnLoad(); // Unload the Config file from memory

  UString FN(FileName);
  DB = new POD(FN);            // Initialize global database pointer
  cache = new RBTree<Grocery>; // Initialize global cache pointer
  terminal->init();
  LoadCache();
  MainMenu();
  terminal->finish();
  delete DB;
  delete cache;
  return 0;
}
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
