// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: rbtreeb.cpp
// Compiler Used: MSVC40, DJGPP 2.7.2.1, GCC 2.7.2.1, HP CPP 10.24
// Produced By: Doug Gaer 
// File Creation Date: 01/23/97  
// Date Last Modified: 12/03/97
// Copyright (c) 1997 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
(R)ed (B)lack (T)ree node data independent functions. 

The VBD C++ classes are copyright (c) 1997, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.
*/
// ----------------------------------------------------------- //   
#include "rbtreeb.hpp"
#include "rbtree.hpp"

// Convenience macros
#define T  (pp.t)  // The current node
#define P  (pp.p)  // Parent 
#define G  (pp.g)  // Grandparent 
#define GG (pp.gg) // Great Grandparent
#define S  (pp.s)  // Sibling 
#define RS (pp.rs) // Right children of s 
#define LS (pp.ls) // Left children of s
#define M  (pp.m)  // Matching node 
#define PM (pp.pm) // Parent of matching node

RBNodeBase *InsBalance(RBNodeBase *Root, PtrPack &pp)
// Balance adjusting for top down insertions. Eliminates
// both P and Tree from being red by doing rotations and
// color changes. g, P, Tree ASSUMED not null coming in. 
// gg may be null. At the end of this routine, only Tree 
// and P will be valid wrt each other. g and gg will 
// not reflect the proper ordering.
{
  RBNodeBase *cofgg; // New child of great-grandparent
  int side;

  side = GG && GG->Right == G;

  if (G->Left == P) {
     if (P->Right == T) { // Do double rotate
        G->Left = T->Right;
        T->Right = G;
        P->Right = T->Left;
        T->Left = P;
        P = GG;
        cofgg = T;
     }
     else { // Do single rotate Right
        G->Left = P->Right;
        P->Right = G;
        cofgg = P;
     }
  }
  else { // G->Right == P
    if (P->Left == T) {  // Do double rotate
       G->Right = T->Left;
       T->Left = G;
       P->Left = T->Right;
       T->Right = P;
       P = GG;
       cofgg = T;
    }
    else { // Do single rotate Left
       G->Right = P->Left;
       P->Left = G;
       cofgg = P;
    }
  }
 
  cofgg->color = BlackNode;
  G->color = RedNode;

  if (GG) {
     if (side) GG->Right = cofgg; else GG->Left = cofgg;
  }
  else Root = cofgg;

  return Root;
}

RBNodeBase *DelBalance(RBNodeBase *Root, PtrPack &pp)
// Balancing code during top down deletion
{
  if ((T == 0 || T->color == BlackNode) && S && S->color == RedNode) {

     // Case: Tree black, s red. Tree might be null,
     // but s and P must not be.

     // Fix g child to be s. Also s may become P of m
     if (G) {
        if (G->Left == P) G->Left = S; else G->Right = S;
     }
     else Root = S;
     if (P == M) PM = S;
     // Finish rotating
     if (P->Left == T) {
        // RotateLeft(P);
        P->Right = LS;
        S->Left = P;
        G = S;
        S = LS;
     }
     else {
        // RotateRight(P);
        P->Left = RS;
        S->Right = P;
        G = S;
        S = RS;
     }

     // Fixup children of s
     if (S) { LS = S->GetLeft(); RS = S->GetRight(); }
     // Fixup colors
     P->color = RedNode; G->color = BlackNode;
  }

  if (T && T->color == BlackNode &&
      (T->Left == 0 || T->GetLeft()->color == BlackNode) &&
      (T->Right == 0 || T->GetRight()->color == BlackNode)) {

     // Case: Tree is a black 2 node.

     if (S && S->color == BlackNode &&
         (S->Left == 0 || S->GetLeft()->color == BlackNode) &&
         (S->Right == 0 || S->GetRight()->color == BlackNode)) {

       // Case: Tree and s are black 2 nodes.

       P->color = BlackNode; T->color = RedNode; S->color = RedNode;

     }
     else if (LS && LS->color == RedNode) {

       // Case: Tree is a black 2 node, s is 3 or 4 node,
       // LS is red (RS might be)

       if (P->Left == T) {
          // Fix colors
          LS->color = P->color; P->color = BlackNode; T->color = RedNode;
          // Fix g child to be LS. ALSo LS may become P of m
          if (G) {
             if (G->Left == P) G->Left = LS; else G->Right = LS;
          }
          else Root = LS;
          if (P == M) PM = LS;
          // Finish: DoubleRotateLeft(s, P);
          P->Right = LS->Left;
          LS->Left = P;
          S->Left = LS->Right;
          LS->Right = S;
          G = LS;
          // We won'Tree fix up s, LS, and since they get reassigned
          // at the top of next loop
       }
       else {
          // Fixup colors
          S->color = P->color; LS->color = BlackNode;
          P->color = BlackNode; T->color = RedNode;
          // Fixup g child to be s. ALSo s may become P of m
          if (G) {
             if (G->Left == P) G->Left = S; else G->Right = S;
          }
          else Root = S;
          if (P == M) PM = S;
          // Finish: RotateRight(P);
          P->Left = RS;
          S->Right = P;
          G = S;
          // We won'Tree fix up s, LS, and since they get reassigned
          // at the top of next loop
       }

     }
     else if (RS && RS->color == RedNode) {

       // Case: Tree is a 2 node, s is a 3 node, LS black, RS red

       if (P->Left == T) {
          // Fix colors
          RS->color = BlackNode; S->color = P->color;
          P->color = BlackNode; T->color = RedNode;
          // Fix g child to be s. ALSo, s may become P of m
          if (G) {
             if (G->Left == P) G->Left = S; else G->Right = S;
          }
          else Root = S;
          if (P == M) PM = S;
          // Finish: RotateLeft(P);
          P->Right = LS;
          S->Left = P;
          G = S;
          // We won'Tree fix up s, LS, and since they get reassigned
          // at the top of next loop
       }
       else {
          // Fix colors
          RS->color = P->color; P->color = BlackNode; T->color = RedNode;
          // Fix g child to become RS. ALSo, RS may become P of m.
          if (G) {
             if (G->Left == P) G->Left = RS; else G->Right = RS;
          }
          else Root = RS;
          if (P == M) PM = RS;
          // Finish: DoubleRotateRight(s, P);
          P->Left = RS->Right;
          RS->Right = P;
          S->Right = RS->Left;
          RS->Left = S;
          G = RS;
          // We won'Tree fix up s, LS, and since they get reassigned
          // at the top of next loop
       }

     }
  }

  return Root;
}

RBNodeBase *DoReplacement(RBNodeBase *Root, PtrPack &pp)
{
  // At this point, m is the node to delete, pm is it's parent,
  // P is the replacement node, g is it's parent. If m has no
  // successor, P will = m, and replacement is the non-null
  // child of m.

  if (M) { // Matching node was found
    if (P == M || M->Left == 0 || M->Right == 0) {
       // No successor, and/or at least one child null
       // Get non-null child, if any, else P will be null
       P = (M->Left) ? M->GetLeft() : M->GetRight();
    }
    else { // m has a successor to use as a replacement
       if (G != M) {
         // Successor isn'Tree immediate child of m, so detach
         // from where it is, attach to where m is
         G->Left = P->Right;
         P->Right = M->Right;
       }
       // Finish attaching where m is.
       P->Left = M->Left;
    }
    // P should have same color as m since it's going where m was
    if (P) P->color = M->color;
  }

  // Fixup pm child link to be P
  if (M) {
     if (PM) {
        if (PM->Left == M) PM->Left = P; else PM->Right = P;
     }
     else Root = P; // New Root, possibly null
  }

  return Root;
}

RBNodeBase *DetachMinMax(RBNodeBase *&Root, int mx)
{
  struct PtrPack pp;

  if (Root == 0) return 0;

  T = Root; P = 0; G = 0;
  M = 0; PM = 0;

  while (T) {

    // Pretend we're on the matching node
    M = T; PM = P;

    // Update ancestor pointers
    G = P; P = T;

    // Then go all the way Left or Right

    if (mx) {
       T = P->GetRight(); S = P->GetLeft();
    }
    else {
       T = P->GetLeft(); S = P->GetRight();
    }

    if (S) {
       LS = S->GetLeft(); RS = S->GetRight();
    }
    else {
       LS = 0; RS = 0;
    }

    Root = DelBalance(Root, pp);

  } // end of while loop

  Root = DoReplacement(Root, pp);

  if (Root) Root->color = BlackNode;

  return M; // Node to delete
}
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
