// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- //
// C++ Source Code File Name: float64.cpp 
// Compiler Used: MSVC40, DJGPP 2.7.2.1, GCC 2.7.2.1, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 09/12/97  
// Date Last Modified: 12/03/97
// Copyright (c) 1997 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBD C++ classes are copyright (c) 1997, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The FLOAT64 class is used to represent 64 bit signed double
precision floating point values independently of the operating
system or hardware platform used. It was created to overcome
the big and little endian problem encountered when writing
floating point values to a common database file accessed by
several different types of machines.

The term "endian" is used to describe the order in which
multibyte numbers are stored in the computers's memory. This
is specifc to each microprocessor. The Intel x86 family is
little-endian, meaning the lowest-order byte is stored first.
Hewlett-Packard's PA-RISC and Sun's SuperSPARC are big-endian,
meaning the highest-order byte is stored first. The Silicon
Graphics MIPS and IBM/Motorola PowerPC processors are both
little and big endian (bi-endian).

 Multibyte Value     Big-Endian    Little-Endian
   0x12345678        0x12345678    0x78563412
   0x1234            0x1234        0x3412
   0x5678            0x5678        0x7856
   "ABC"             41 42 42 00   41 42 43 00

With big-endian ordering, the address of the multibyte value
is its most significant byte (its big end). With little-endian
ordering, the address is its least significant byte (its little
end). In a struct the order of bytes in memory will differ
depending on the byte ordering and the particular data type.

For a double precision floating point number, the binary info is
stored within a total of 64 bits. The first bit contains the sign
of the mantissa (0 for positive and 1 for negative). The next 11
bits store the exponent, and the remaining 52 bits provide the
mantissa, giving an approximate decimal precision of 15 digits.
*/
// ----------------------------------------------------------- //   
#include <string.h>
#include <memory.h>
#include "float64.hpp"
#include "ehandler.hpp"
#include "machine.hpp"

FLOAT64::FLOAT64(DPFLOAT val)
{
  UnPackBits(val);
}

FLOAT64::FLOAT64(const FLOAT64& ob)
{
  memmove((void *)byte, (const void *)ob.byte, sizeof(DPFLOAT));
}

FLOAT64& FLOAT64::operator=(const FLOAT64& ob)
{
  memmove((void *)byte, (const void *)ob.byte, sizeof(DPFLOAT));
  return *this;
}

FLOAT64& FLOAT64::operator=(const DPFLOAT val)
{
  UnPackBits(val);
  return *this;
}

FLOAT64::operator DPFLOAT() const
{
  return PackBits();
}

DPFLOAT FLOAT64::PackBits() const
{
  double buf;
  double *nval = new double;

#ifdef __X86__     // Big to littel endian conversion for X86 processors
  FLOAT64 ob;  
  ob.byte[7] = byte[0]; 
  ob.byte[6] = byte[1];
  ob.byte[5] = byte[2];
  ob.byte[4] = byte[3];
  ob.byte[3] = byte[4];
  ob.byte[2] = byte[5];
  ob.byte[1] = byte[6];
  ob.byte[0] = byte[7];
  memmove((void *)nval, (const void *)ob.byte, sizeof(DPFLOAT));
#else
  memmove((void *)nval, (const void *)byte, sizeof(DPFLOAT));
#endif
  
  buf = *nval;
  delete nval;

  return buf;
}

void FLOAT64::UnPackBits(DPFLOAT val)
{
  DPFLOAT *dptr = &val;
  char *ptr = new char[sizeof(DPFLOAT)+1];

  memmove((void *)ptr, (const void *)dptr, sizeof(DPFLOAT));

#ifdef __X86__     // Littel to big endian conversion for X86 processors
  byte[7] = ptr[0];
  byte[6] = ptr[1];
  byte[5] = ptr[2];
  byte[4] = ptr[3];
  byte[3] = ptr[4];
  byte[2] = ptr[5];
  byte[1] = ptr[6];
  byte[0] = ptr[7];
#else                
  byte[0] = ptr[0];
  byte[1] = ptr[1];
  byte[2] = ptr[2];
  byte[3] = ptr[3];
  byte[4] = ptr[4];
  byte[5] = ptr[5];
  byte[6] = ptr[6];
  byte[7] = ptr[7];
#endif
  
  delete [] ptr;
}

FLOAT64 FLOAT64::operator++(int) // Postfix
{
  FLOAT64 val_before(*this); 
  operator=(*this + 1);
  return val_before;
}
 
FLOAT64 FLOAT64::operator--(int) // Postfix
{
  FLOAT64 val_before(*this); 
  operator=(*this - 1);
  return val_before;
}

void FLOAT64::operator/=(const FLOAT64 &fp)
{
  if(fp == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / fp);
}

void FLOAT64::operator/=(const DPFLOAT &fp)
{
  if(fp == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / fp);
}

void FLOAT64::operator/=(const LWORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

    operator=(*this / i);
}

void FLOAT64::operator/=(const ULWORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void FLOAT64::operator/=(const WORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void FLOAT64::operator/=(const SWORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void FLOAT64::operator/=(const UWORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void FLOAT64::operator/=(const USWORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void FLOAT64::operator/=(const SBYTE &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / (DPFLOAT)i);
}

void FLOAT64::operator/=(const UBYTE &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / (DPFLOAT)i);
}

int operator==(const FLOAT64 &a, const FLOAT64 &b)
{
  return a.PackBits() == b.PackBits();
}

int operator==(const FLOAT64 &a, const DPFLOAT &bf)
{
  return a.PackBits() == bf;
}

int operator==(const DPFLOAT &af, const FLOAT64 &b)
{
  return af == b.PackBits(); 
}

int operator==(const FLOAT64 &a, const LWORD &bl)
{
  return a.PackBits() == bl;
}

int operator==(const LWORD &al, const FLOAT64 &b)
{
  return al == b.PackBits(); 
}

int operator==(const FLOAT64 &a, const ULWORD &bl)
{
  return a.PackBits() == bl;
}

int operator==(const ULWORD &al, const FLOAT64 &b)
{
  return al == b.PackBits(); 
}

int operator==(const FLOAT64 &a, const WORD &bl)
{
  return a.PackBits() == bl;
}

int operator==(const WORD &al, const FLOAT64 &b)
{
  return al == b.PackBits();
}

int operator==(const FLOAT64 &a, const SWORD &bl)
{
  return a.PackBits() == bl;
}

int operator==(const SWORD &al, const FLOAT64 &b)
{
  return al == b.PackBits(); 
}

int operator==(const FLOAT64 &a, const UWORD &bl)
{
  return a.PackBits() == bl;
}

int operator==(const UWORD &al, const FLOAT64 &b)
{
  return al == b.PackBits(); 
}

int operator==(const FLOAT64 &a, const USWORD &bl)
{
  return  a.PackBits() == bl;
}

int operator==(const USWORD &al, const FLOAT64 &b)
{
  return al == b.PackBits(); 
}

int operator==(const FLOAT64 &a, const SBYTE &bl)
{
  return a.PackBits() == (DPFLOAT)bl;
}

int operator==(const SBYTE &al, const FLOAT64 &b)
{
  return (DPFLOAT)al == b.PackBits(); 
}

int operator==(const FLOAT64 &a, const UBYTE &bl)
{
  return a.PackBits() == (DPFLOAT)bl;
}

int operator==(const UBYTE &al, const FLOAT64 &b)
{
  return (DPFLOAT)al == b.PackBits(); 
}

int operator!=(const FLOAT64 &a, const FLOAT64 &b)
{
  return a.PackBits() != b.PackBits();
}

int operator!=(const FLOAT64 &a, const DPFLOAT &bf)
{
  return a.PackBits() != bf;
}

int operator!=(const DPFLOAT &af, const FLOAT64 &b)
{
  return af != b.PackBits();
}

int operator!=(const FLOAT64 &a, const LWORD &bl)
{
  return a.PackBits() != bl;
}

int operator!=(const LWORD &al, const FLOAT64 &b)
{
  return al != b.PackBits();
}

int operator!=(const FLOAT64 &a, const ULWORD &bl)
{
  return a.PackBits() != bl;
}

int operator!=(const ULWORD &al, const FLOAT64 &b)
{
  return al != b.PackBits();
}

int operator!=(const FLOAT64 &a, const WORD &bl)
{
  return a.PackBits() != bl;
}

int operator!=(const WORD &al, const FLOAT64 &b)
{
  return al != b.PackBits();
}

int operator!=(const FLOAT64 &a, const SWORD &bl)
{
  return a.PackBits() != bl;
}

int operator!=(const SWORD &al, const FLOAT64 &b)
{
  return al != b.PackBits();
}

int operator!=(const FLOAT64 &a, const UWORD &bl)
{
  return a.PackBits() != bl;
}

int operator!=(const UWORD &al, const FLOAT64 &b)
{
  return al != b.PackBits();
}

int operator!=(const FLOAT64 &a, const USWORD &bl)
{
  return a.PackBits() != bl;
}

int operator!=(const USWORD &al, const FLOAT64 &b)
{
  return al != b.PackBits();
}

int operator!=(const FLOAT64 &a, const SBYTE &bl)
{
  return a.PackBits() != (DPFLOAT)bl;
}

int operator!=(const SBYTE &al, const FLOAT64 &b)
{
  return (DPFLOAT)al != b.PackBits();
}

int operator!=(const FLOAT64 &a, const UBYTE &bl)
{
  return a.PackBits() != (DPFLOAT)bl;
}

int operator!=(const UBYTE &al, const FLOAT64 &b)
{
  return (DPFLOAT)al != b.PackBits();
}

int operator<(const FLOAT64 &a, const FLOAT64 &b)
{
  return a.PackBits() < b.PackBits();
}

int operator<(const FLOAT64 &a, const DPFLOAT &bf)
{
  return a.PackBits() < bf;
}

int operator<(const DPFLOAT &af, const FLOAT64 &b)
{
  return af < b.PackBits();
}

int operator<(const FLOAT64 &a, const LWORD &bl)
{
  return a.PackBits() < bl;
}

int operator<(const LWORD &al, const FLOAT64 &b)
{
  return al < b.PackBits();
}

int operator<(const FLOAT64 &a, const ULWORD &bl)
{
  return a.PackBits() < bl;
}

int operator<(const ULWORD &al, const FLOAT64 &b)
{
  return al < b.PackBits();
}

int operator<(const FLOAT64 &a, const WORD &bl)
{
  return a.PackBits() < bl;
}

int operator<(const WORD &al, const FLOAT64 &b)
{
  return al < b.PackBits();
}

int operator<(const FLOAT64 &a, const SWORD &bl)
{
  return a.PackBits() < bl;
}

int operator<(const SWORD &al, const FLOAT64 &b)
{
  return al < b.PackBits();
}

int operator<(const FLOAT64 &a, const UWORD &bl)
{
  return a.PackBits() < bl;
}

int operator<(const UWORD &al, const FLOAT64 &b)
{
  return al < b.PackBits();
}

int operator<(const FLOAT64 &a, const USWORD &bl)
{
  return a.PackBits() < bl;
}

int operator<(const USWORD &al, const FLOAT64 &b)
{
  return al < b.PackBits();
}

int operator<(const FLOAT64 &a, const SBYTE &bl)
{
  return a.PackBits() < (DPFLOAT)bl;
}

int operator<(const SBYTE &al, const FLOAT64 &b)
{
  return (DPFLOAT)al < b.PackBits();
}

int operator<(const FLOAT64 &a, const UBYTE &bl)
{
  return a.PackBits() < (DPFLOAT)bl;
}

int operator<(const UBYTE &al, const FLOAT64 &b)
{
  return (DPFLOAT)al < b.PackBits();
}

int operator>(const FLOAT64 &a, const FLOAT64 &b)
{
  return a.PackBits() > b.PackBits();
}

int operator>(const FLOAT64 &a, const DPFLOAT &bf)
{
  return a.PackBits() > bf;
}

int operator>(const DPFLOAT &af, const FLOAT64 &b)
{
  return af > b.PackBits();
}

int operator>(const FLOAT64 &a, const LWORD &bl)
{
  return a.PackBits() > bl;
}

int operator>(const LWORD &al, const FLOAT64 &b)
{
  return al > b.PackBits();
}

int operator>(const FLOAT64 &a, const ULWORD &bl)
{
  return a.PackBits() > bl;
}

int operator>(const ULWORD &al, const FLOAT64 &b)
{
  return al > b.PackBits();
}

int operator>(const FLOAT64 &a, const WORD &bl)
{
  return a.PackBits() > bl;
}

int operator>(const WORD &al, const FLOAT64 &b)
{
  return al > b.PackBits();
}

int operator>(const FLOAT64 &a, const SWORD &bl)
{
  return a.PackBits() > bl;
}

int operator>(const SWORD &al, const FLOAT64 &b)
{
  return al > b.PackBits();
}

int operator>(const FLOAT64 &a, const UWORD &bl)
{
  return a.PackBits() > bl;
}

int operator>(const UWORD &al, const FLOAT64 &b)
{
  return al > b.PackBits();
}

int operator>(const FLOAT64 &a, const USWORD &bl)
{
  return a.PackBits() > bl;
}

int operator>(const USWORD &al, const FLOAT64 &b)
{
  return al > b.PackBits();
}

int operator>(const FLOAT64 &a, const SBYTE &bl)
{
  return a.PackBits() > (DPFLOAT)bl;
}

int operator>(const SBYTE &al, const FLOAT64 &b)
{
  return (DPFLOAT)al > b.PackBits();
}

int operator>(const FLOAT64 &a, const UBYTE &bl)
{
  return a.PackBits() > (DPFLOAT)bl;
}

int operator>(const UBYTE &al, const FLOAT64 &b)
{
  return (DPFLOAT)al > b.PackBits();
}

int operator<=(const FLOAT64 &a, const FLOAT64 &b)
{
  return a.PackBits() == b.PackBits() || a.PackBits() < b.PackBits();
}

int operator<=(const FLOAT64 &a, const DPFLOAT &bf)
{
  return a.PackBits() == bf || a.PackBits() < bf;
}

int operator<=(const DPFLOAT &af, const FLOAT64 &b)
{
  return af == b.PackBits() || af < b.PackBits();
}

int operator<=(const FLOAT64 &a, const LWORD &bl)
{
  return a.PackBits() == bl || a.PackBits() < bl;
}

int operator<=(const LWORD &al, const FLOAT64 &b)
{
  return al == b.PackBits() || al < b.PackBits();
}

int operator<=(const FLOAT64 &a, const ULWORD &bl)
{
  return a.PackBits() == bl || a.PackBits() < bl;
}

int operator<=(const ULWORD &al, const FLOAT64 &b)
{
  return al == b.PackBits() || al < b.PackBits();
}

int operator<=(const FLOAT64 &a, const WORD &bl)
{
  return a.PackBits() == bl || a.PackBits() < bl;
}

int operator<=(const WORD &al, const FLOAT64 &b)
{
  return al == b.PackBits() || al < b.PackBits();
}

int operator<=(const FLOAT64 &a, const SWORD &bl)
{
  return a.PackBits() == bl || a.PackBits() < bl;
}

int operator<=(const SWORD &al, const FLOAT64 &b)
{
  return al == b.PackBits() || al < b.PackBits();
}

int operator<=(const FLOAT64 &a, const UWORD &bl)
{
  return a.PackBits() == bl || a.PackBits() < bl;
}

int operator<=(const UWORD &al, const FLOAT64 &b)
{
  return al == b.PackBits() || al < b.PackBits();
}

int operator<=(const FLOAT64 &a, const USWORD &bl)
{
  return a.PackBits() == bl || a.PackBits() < bl;
}

int operator<=(const USWORD &al, const FLOAT64 &b)
{
  return al == b.PackBits() || al < b.PackBits();
}

int operator<=(const FLOAT64 &a, const SBYTE &bl)
{
  return a.PackBits() == (DPFLOAT)bl ||  a.PackBits() < (DPFLOAT)bl;
}

int operator<=(const SBYTE &al, const FLOAT64 &b)
{
  return (DPFLOAT)al == b.PackBits() || (DPFLOAT)al < b.PackBits();
}

int operator<=(const FLOAT64 &a, const UBYTE &bl)
{
  return a.PackBits() == (DPFLOAT)bl ||  a.PackBits() < (DPFLOAT)bl;
}

int operator<=(const UBYTE &al, const FLOAT64 &b)
{
  return (DPFLOAT)al == b.PackBits() || (DPFLOAT)al < b.PackBits();
}

int operator>=(const FLOAT64 &a, const FLOAT64 &b)
{
  return a.PackBits() == b.PackBits() || a.PackBits() > b.PackBits();
}

int operator>=(const FLOAT64 &a, const DPFLOAT &bf)
{
  return a.PackBits() == bf || a.PackBits() > bf;
}

int operator>=(const DPFLOAT &af, const FLOAT64 &b)
{
  return af == b.PackBits() || af > b.PackBits();
}

int operator>=(const FLOAT64 &a, const LWORD &bl)
{
  return a.PackBits() == bl || a.PackBits() > bl;
}

int operator>=(const LWORD &al, const FLOAT64 &b)
{
  return al == b.PackBits() || al > b.PackBits();
}

int operator>=(const FLOAT64 &a, const ULWORD &bl)
{
  return a.PackBits() == bl || a.PackBits() > bl;
}

int operator>=(const ULWORD &al, const FLOAT64 &b)
{
  return al == b.PackBits() || al > b.PackBits();
}

int operator>=(const FLOAT64 &a, const WORD &bl)
{
  return a.PackBits() == bl || a.PackBits() > bl;
}

int operator>=(const WORD &al, const FLOAT64 &b)
{
  return al == b.PackBits() || al > b.PackBits();
}

int operator>=(const FLOAT64 &a, const SWORD &bl)
{
  return a.PackBits() == bl || a.PackBits() > bl;
}

int operator>=(const SWORD &al, const FLOAT64 &b)
{
  return al == b.PackBits() || al > b.PackBits();
}

int operator>=(const FLOAT64 &a, const UWORD &bl)
{
  return a.PackBits() == bl || a.PackBits() > bl;
}

int operator>=(const UWORD &al, const FLOAT64 &b)
{
  return al == b.PackBits() || al > b.PackBits();
}

int operator>=(const FLOAT64 &a, const USWORD &bl)
{
  return a.PackBits() == bl || a.PackBits() > bl;
}

int operator>=(const USWORD &al, const FLOAT64 &b)
{
  return al == b.PackBits() || al > b.PackBits();
}

int operator>=(const FLOAT64 &a, const SBYTE &bl)
{
  return a.PackBits() == (DPFLOAT)bl || a.PackBits() > (DPFLOAT)bl;
}

int operator>=(const SBYTE &al, const FLOAT64 &b)
{
  return (DPFLOAT)al == b.PackBits() || (DPFLOAT)al > b.PackBits();
}

int operator>=(const FLOAT64 &a, const UBYTE &bl)
{
  return a.PackBits() == (DPFLOAT)bl || a.PackBits() > (DPFLOAT)bl;
}

int operator>=(const UBYTE &al, const FLOAT64 &b)
{
  return (DPFLOAT)al == b.PackBits() || (DPFLOAT)al > b.PackBits();
}
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
