// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- //
// C++ Header File Name: float64.hpp 
// Compiler Used: MSVC40, DJGPP 2.7.2.1, GCC 2.7.2.1, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 09/12/97  
// Date Last Modified: 12/03/97
// Copyright (c) 1997 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ---------- Include File Description and Details  ---------- // 
// ----------------------------------------------------------- // 
/*
The VBD C++ classes are copyright (c) 1997, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The FLOAT64 class is used to represent 64 bit signed double
precision floating point values independently of the operating
system or hardware platform used. It was created to overcome
the big and little endian problem encountered when writing
floating point values to a common database file accessed by
several different types of machines.

The term "endian" is used to describe the order in which
multibyte numbers are stored in the computers's memory. This
is specifc to each microprocessor. The Intel x86 family is
little-endian, meaning the lowest-order byte is stored first.
Hewlett-Packard's PA-RISC and Sun's SuperSPARC are big-endian,
meaning the highest-order byte is stored first. The Silicon
Graphics MIPS and IBM/Motorola PowerPC processors are both
little and big endian (bi-endian).

 Multibyte Value     Big-Endian    Little-Endian
   0x12345678        0x12345678    0x78563412
   0x1234            0x1234        0x3412
   0x5678            0x5678        0x7856
   "ABC"             41 42 42 00   41 42 43 00

With big-endian ordering, the address of the multibyte value
is its most significant byte (its big end). With little-endian
ordering, the address is its least significant byte (its little
end). In a struct the order of bytes in memory will differ
depending on the byte ordering and the particular data type.

For a double precision floating point number, the binary info is
stored within a total of 64 bits. The first bit contains the sign
of the mantissa (0 for positive and 1 for negative). The next 11
bits store the exponent, and the remaining 52 bits provide the
mantissa, giving an approximate decimal precision of 15 digits.
*/
// ----------------------------------------------------------- //   
#ifndef __FLOAT64_HPP
#define __FLOAT64_HPP

#include "dtypes.hpp"

// Data structure for signed double precision floating point values.
class FLOAT64
{
public:
  FLOAT64(DPFLOAT val = 0);
  FLOAT64(const FLOAT64& ob);
  FLOAT64& operator=(const FLOAT64& ob);
  FLOAT64& operator=(const DPFLOAT ob);

public:
  void UnPackBits(DPFLOAT val);
  DPFLOAT PackBits() const;
  
public:
  operator DPFLOAT() const;

public: // Arithmetic operators that modify their operand
  FLOAT64 operator++(int);  // Postfix
  FLOAT64 operator--(int);  // Postfix
  FLOAT64 &operator++() { operator=(*this + 1); return *this; } // Prefix
  FLOAT64 &operator--() { operator=(*this - 1); return *this; } // Prefix
  void operator+=(const FLOAT64 &fp) { operator=(*this + fp); }
  void operator-=(const FLOAT64 &fp) { operator=(*this - fp); }
  void operator*=(const FLOAT64 &fp) { operator=(*this * fp); }
  void operator/=(const FLOAT64 &fp);

  void operator+=(const DPFLOAT &fp) { operator=(*this + fp); }
  void operator-=(const DPFLOAT &fp) { operator=(*this - fp); }
  void operator*=(const DPFLOAT &fp) { operator=(*this * fp); }
  void operator/=(const DPFLOAT &fp);
  
  void operator+=(const LWORD &i) { operator=(*this + i); }
  void operator-=(const LWORD &i) { operator=(*this - i); }
  void operator*=(const LWORD &i) { operator=(*this * i); }
  void operator/=(const LWORD &i);

  void operator+=(const ULWORD &i) { operator=(*this + i); }
  void operator-=(const ULWORD &i) { operator=(*this - i); }
  void operator*=(const ULWORD &i) { operator=(*this * i); }
  void operator/=(const ULWORD &i);

  void operator+=(const WORD &i) { operator=(*this + i); }
  void operator-=(const WORD &i) { operator=(*this - i); }
  void operator*=(const WORD &i) { operator=(*this * i); }
  void operator/=(const WORD &i);

  void operator+=(const SWORD &i) { operator=(*this + i); }
  void operator-=(const SWORD &i) { operator=(*this - i); }
  void operator*=(const SWORD &i) { operator=(*this * i); }
  void operator/=(const SWORD &i);

  void operator+=(const UWORD &i) { operator=(*this + i); }
  void operator-=(const UWORD &i) { operator=(*this - i); }
  void operator*=(const UWORD &i) { operator=(*this * i); }
  void operator/=(const UWORD &i);

  void operator+=(const USWORD &i) { operator=(*this + i); }
  void operator-=(const USWORD &i) { operator=(*this - i); }
  void operator*=(const USWORD &i) { operator=(*this * i); }
  void operator/=(const USWORD &i);

  void operator+=(const SBYTE &i) { operator=(*this + (DPFLOAT)i); }
  void operator-=(const SBYTE &i) { operator=(*this - (DPFLOAT)i); }
  void operator*=(const SBYTE &i) { operator=(*this * (DPFLOAT)i); }
  void operator/=(const SBYTE &i);

  void operator+=(const UBYTE &i) { operator=(*this + (DPFLOAT)i); }
  void operator-=(const UBYTE &i) { operator=(*this - (DPFLOAT)i); }
  void operator*=(const UBYTE &i) { operator=(*this * (DPFLOAT)i); }
  void operator/=(const UBYTE &i);

public: // Comparison operators
  friend int operator==(const FLOAT64 &a, const FLOAT64 &b);
  friend int operator==(const FLOAT64 &a, const DPFLOAT &bf);
  friend int operator==(const DPFLOAT &af, const FLOAT64 &b);
  friend int operator==(const FLOAT64 &a, const LWORD &bl);
  friend int operator==(const LWORD &al, const FLOAT64 &b);
  friend int operator==(const FLOAT64 &a, const ULWORD &bl);
  friend int operator==(const ULWORD &al, const FLOAT64 &b);
  friend int operator==(const FLOAT64 &a, const WORD &bl);
  friend int operator==(const WORD &al, const FLOAT64 &b);
  friend int operator==(const FLOAT64 &a, const SWORD &bl);
  friend int operator==(const SWORD &al, const FLOAT64 &b);
  friend int operator==(const FLOAT64 &a, const UWORD &bl);
  friend int operator==(const UWORD &al, const FLOAT64 &b);
  friend int operator==(const FLOAT64 &a, const USWORD &bl);
  friend int operator==(const USWORD &al, const FLOAT64 &b);
  friend int operator==(const FLOAT64 &a, const SBYTE &bl);
  friend int operator==(const SBYTE &al, const FLOAT64 &b);
  friend int operator==(const FLOAT64 &a, const UBYTE &bl);
  friend int operator==(const UBYTE &al, const FLOAT64 &b);

  friend int operator!=(const FLOAT64 &a, const FLOAT64 &b);
  friend int operator!=(const FLOAT64 &a, const DPFLOAT &bf);
  friend int operator!=(const DPFLOAT &af, const FLOAT64 &b);
  friend int operator!=(const FLOAT64 &a, const LWORD &bl);
  friend int operator!=(const LWORD &al, const FLOAT64 &b);
  friend int operator!=(const FLOAT64 &a, const ULWORD &bl);
  friend int operator!=(const ULWORD &al, const FLOAT64 &b);
  friend int operator!=(const FLOAT64 &a, const WORD &bl);
  friend int operator!=(const WORD &al, const FLOAT64 &b);
  friend int operator!=(const FLOAT64 &a, const SWORD &bl);
  friend int operator!=(const SWORD &al, const FLOAT64 &b);
  friend int operator!=(const FLOAT64 &a, const UWORD &bl);
  friend int operator!=(const UWORD &al, const FLOAT64 &b);
  friend int operator!=(const FLOAT64 &a, const USWORD &bl);
  friend int operator!=(const USWORD &al, const FLOAT64 &b);
  friend int operator!=(const FLOAT64 &a, const SBYTE &bl);
  friend int operator!=(const SBYTE &al, const FLOAT64 &b);
  friend int operator!=(const FLOAT64 &a, const UBYTE &bl);
  friend int operator!=(const UBYTE &al, const FLOAT64 &b);

  friend int operator<(const FLOAT64 &a, const FLOAT64 &b);
  friend int operator<(const FLOAT64 &a, const DPFLOAT &bf);
  friend int operator<(const DPFLOAT &af, const FLOAT64 &b);
  friend int operator<(const FLOAT64 &a, const LWORD &bl);
  friend int operator<(const LWORD &al, const FLOAT64 &b);
  friend int operator<(const FLOAT64 &a, const ULWORD &bl);
  friend int operator<(const ULWORD &al, const FLOAT64 &b);
  friend int operator<(const FLOAT64 &a, const WORD &bl);
  friend int operator<(const WORD &al, const FLOAT64 &b);
  friend int operator<(const FLOAT64 &a, const SWORD &bl);
  friend int operator<(const SWORD &al, const FLOAT64 &b);
  friend int operator<(const FLOAT64 &a, const UWORD &bl);
  friend int operator<(const UWORD &al, const FLOAT64 &b);
  friend int operator<(const FLOAT64 &a, const USWORD &bl);
  friend int operator<(const USWORD &al, const FLOAT64 &b);
  friend int operator<(const FLOAT64 &a, const SBYTE &bl);
  friend int operator<(const SBYTE &al, const FLOAT64 &b);
  friend int operator<(const FLOAT64 &a, const UBYTE &bl);
  friend int operator<(const UBYTE &al, const FLOAT64 &b);

  friend int operator>(const FLOAT64 &a, const FLOAT64 &b);
  friend int operator>(const FLOAT64 &a, const DPFLOAT &bf);
  friend int operator>(const DPFLOAT &af, const FLOAT64 &b);
  friend int operator>(const FLOAT64 &a, const LWORD &bl);
  friend int operator>(const LWORD &al, const FLOAT64 &b);
  friend int operator>(const FLOAT64 &a, const ULWORD &bl);
  friend int operator>(const ULWORD &al, const FLOAT64 &b);
  friend int operator>(const FLOAT64 &a, const WORD &bl);
  friend int operator>(const WORD &al, const FLOAT64 &b);
  friend int operator>(const FLOAT64 &a, const SWORD &bl);
  friend int operator>(const SWORD &al, const FLOAT64 &b);
  friend int operator>(const FLOAT64 &a, const UWORD &bl);
  friend int operator>(const UWORD &al, const FLOAT64 &b);
  friend int operator>(const FLOAT64 &a, const USWORD &bl);
  friend int operator>(const USWORD &al, const FLOAT64 &b);
  friend int operator>(const FLOAT64 &a, const SBYTE &bl);
  friend int operator>(const SBYTE &al, const FLOAT64 &b);
  friend int operator>(const FLOAT64 &a, const UBYTE &bl);
  friend int operator>(const UBYTE &al, const FLOAT64 &b);

  friend int operator<=(const FLOAT64 &a, const FLOAT64 &b);
  friend int operator<=(const FLOAT64 &a, const DPFLOAT &bf);
  friend int operator<=(const DPFLOAT &af, const FLOAT64 &b);
  friend int operator<=(const FLOAT64 &a, const LWORD &bl);
  friend int operator<=(const LWORD &al, const FLOAT64 &b);
  friend int operator<=(const FLOAT64 &a, const ULWORD &bl);
  friend int operator<=(const ULWORD &al, const FLOAT64 &b);
  friend int operator<=(const FLOAT64 &a, const WORD &bl);
  friend int operator<=(const WORD &al, const FLOAT64 &b);
  friend int operator<=(const FLOAT64 &a, const SWORD &bl);
  friend int operator<=(const SWORD &al, const FLOAT64 &b);
  friend int operator<=(const FLOAT64 &a, const UWORD &bl);
  friend int operator<=(const UWORD &al, const FLOAT64 &b);
  friend int operator<=(const FLOAT64 &a, const USWORD &bl);
  friend int operator<=(const USWORD &al, const FLOAT64 &b);
  friend int operator<=(const FLOAT64 &a, const SBYTE &bl);
  friend int operator<=(const SBYTE &al, const FLOAT64 &b);
  friend int operator<=(const FLOAT64 &a, const UBYTE &bl);
  friend int operator<=(const UBYTE &al, const FLOAT64 &b);

  friend int operator>=(const FLOAT64 &a, const FLOAT64 &b);
  friend int operator>=(const FLOAT64 &a, const DPFLOAT &bf);
  friend int operator>=(const DPFLOAT &af, const FLOAT64 &b);
  friend int operator>=(const FLOAT64 &a, const LWORD &bl);
  friend int operator>=(const LWORD &al, const FLOAT64 &b);
  friend int operator>=(const FLOAT64 &a, const ULWORD &bl);
  friend int operator>=(const ULWORD &al, const FLOAT64 &b);
  friend int operator>=(const FLOAT64 &a, const WORD &bl);
  friend int operator>=(const WORD &al, const FLOAT64 &b);
  friend int operator>=(const FLOAT64 &a, const SWORD &bl);
  friend int operator>=(const SWORD &al, const FLOAT64 &b);
  friend int operator>=(const FLOAT64 &a, const UWORD &bl);
  friend int operator>=(const UWORD &al, const FLOAT64 &b);
  friend int operator>=(const FLOAT64 &a, const USWORD &bl);
  friend int operator>=(const USWORD &al, const FLOAT64 &b);
  friend int operator>=(const FLOAT64 &a, const SBYTE &bl);
  friend int operator>=(const SBYTE &al, const FLOAT64 &b);
  friend int operator>=(const FLOAT64 &a, const UBYTE &bl);
  friend int operator>=(const UBYTE &al, const FLOAT64 &b);
  
private:
  SBYTE byte[8];
};

#endif // __FLOAT64_HPP
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
