// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- //
// C++ Header File Name: int32.hpp 
// Compiler Used: MSVC40, DJGPP 2.7.2.1, GCC 2.7.2.1, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 09/05/97  
// Date Last Modified: 12/03/97
// Copyright (c) 1997 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ---------- Include File Description and Details  ---------- // 
// ----------------------------------------------------------- // 
/*
The VBD C++ classes are copyright (c) 1997, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The INT32 class is used to represent 32 bit signed integers
independently of the operating system or hardware platform
used. It was created to overcome the big and little endian
problem encountered when writing integer values to a common
database file accessed by several different types of machines.

The term "endian" is used to describe the order in which
multibyte numbers are stored in the computers's memory. This
is specifc to each microprocessor. The Intel x86 family is
little-endian, meaning the lowest-order byte is stored first.
Hewlett-Packard's PA-RISC and Sun's SuperSPARC are big-endian,
meaning the highest-order byte is stored first. The Silicon
Graphics MIPS and IBM/Motorola PowerPC processors are both
little and big endian (bi-endian).

 Multibyte Value     Big-Endian    Little-Endian
   0x12345678        0x12345678    0x78563412
   0x1234            0x1234        0x3412
   0x5678            0x5678        0x7856
   "ABC"             41 42 42 00   41 42 43 00

With big-endian ordering, the address of the multibyte value
is its most significant byte (its big end). With little-endian
ordering, the address is its least significant byte (its little
end). In a struct the order of bytes in memory will differ
depending on the byte ordering and the particular data type.
*/
// ----------------------------------------------------------- //   
#ifndef __INT32_HPP
#define __INT32_HPP

#include "dtypes.hpp"

// Data structure for signed 32 bit integer values.
class INT32
{
public:
  INT32(LWORD val = 0);
  INT32(const INT32& ob);
  INT32& operator=(const INT32& ob);
  INT32& operator=(const LWORD ob);

public:
  void UnPackBits(LWORD val);
  LWORD PackBits() const;

public:
  operator LWORD() const;
  
public: // Arithmetic operators that modify their operand
  INT32 operator++(int);  // Postfix
  INT32 operator--(int);  // Postfix
  INT32 &operator++() { operator=(*this + 1); return *this; } // Prefix
  INT32 &operator--() { operator=(*this - 1); return *this; } // Prefix
  void operator+=(const INT32 &i) { operator=(*this + i); }
  void operator-=(const INT32 &i) { operator=(*this - i); }
  void operator*=(const INT32 &i) { operator=(*this * i); }
  void operator/=(const INT32 &i);

  void operator+=(const LWORD &i) { operator=(*this + i); }
  void operator-=(const LWORD &i) { operator=(*this - i); }
  void operator*=(const LWORD &i) { operator=(*this * i); }
  void operator/=(const LWORD &i);

  void operator+=(const ULWORD &i) { operator=(*this + i); }
  void operator-=(const ULWORD &i) { operator=(*this - i); }
  void operator*=(const ULWORD &i) { operator=(*this * i); }
  void operator/=(const ULWORD &i);

  void operator+=(const WORD &i) { operator=(*this + i); }
  void operator-=(const WORD &i) { operator=(*this - i); }
  void operator*=(const WORD &i) { operator=(*this * i); }
  void operator/=(const WORD &i);

  void operator+=(const SWORD &i) { operator=(*this + i); }
  void operator-=(const SWORD &i) { operator=(*this - i); }
  void operator*=(const SWORD &i) { operator=(*this * i); }
  void operator/=(const SWORD &i);

  void operator+=(const UWORD &i) { operator=(*this + i); }
  void operator-=(const UWORD &i) { operator=(*this - i); }
  void operator*=(const UWORD &i) { operator=(*this * i); }
  void operator/=(const UWORD &i);

  void operator+=(const USWORD &i) { operator=(*this + i); }
  void operator-=(const USWORD &i) { operator=(*this - i); }
  void operator*=(const USWORD &i) { operator=(*this * i); }
  void operator/=(const USWORD &i);

  void operator+=(const SBYTE &i) { operator=(*this + (LWORD)i); }
  void operator-=(const SBYTE &i) { operator=(*this - (LWORD)i); }
  void operator*=(const SBYTE &i) { operator=(*this * (LWORD)i); }
  void operator/=(const SBYTE &i);

  void operator+=(const UBYTE &i) { operator=(*this + (LWORD)i); }
  void operator-=(const UBYTE &i) { operator=(*this - (LWORD)i); }
  void operator*=(const UBYTE &i) { operator=(*this * (LWORD)i); }
  void operator/=(const UBYTE &i);

public: // Comparison operators
  friend int operator==(const INT32 &a, const INT32 &b);
  friend int operator==(const INT32 &a, const LWORD &bl);
  friend int operator==(const LWORD &al, const INT32 &b);
  friend int operator==(const INT32 &a, const ULWORD &bl);
  friend int operator==(const ULWORD &al, const INT32 &b);
  friend int operator==(const INT32 &a, const WORD &bl);
  friend int operator==(const WORD &al, const INT32 &b);
  friend int operator==(const INT32 &a, const SWORD &bl);
  friend int operator==(const SWORD &al, const INT32 &b);
  friend int operator==(const INT32 &a, const UWORD &bl);
  friend int operator==(const UWORD &al, const INT32 &b);
  friend int operator==(const INT32 &a, const USWORD &bl);
  friend int operator==(const USWORD &al, const INT32 &b);
  friend int operator==(const INT32 &a, const SBYTE &bl);
  friend int operator==(const SBYTE &al, const INT32 &b);
  friend int operator==(const INT32 &a, const UBYTE &bl);
  friend int operator==(const UBYTE &al, const INT32 &b);

  friend int operator!=(const INT32 &a, const INT32 &b);
  friend int operator!=(const INT32 &a, const LWORD &bl);
  friend int operator!=(const LWORD &al, const INT32 &b);
  friend int operator!=(const INT32 &a, const ULWORD &bl);
  friend int operator!=(const ULWORD &al, const INT32 &b);
  friend int operator!=(const INT32 &a, const WORD &bl);
  friend int operator!=(const WORD &al, const INT32 &b);
  friend int operator!=(const INT32 &a, const SWORD &bl);
  friend int operator!=(const SWORD &al, const INT32 &b);
  friend int operator!=(const INT32 &a, const UWORD &bl);
  friend int operator!=(const UWORD &al, const INT32 &b);
  friend int operator!=(const INT32 &a, const USWORD &bl);
  friend int operator!=(const USWORD &al, const INT32 &b);
  friend int operator!=(const INT32 &a, const SBYTE &bl);
  friend int operator!=(const SBYTE &al, const INT32 &b);
  friend int operator!=(const INT32 &a, const UBYTE &bl);
  friend int operator!=(const UBYTE &al, const INT32 &b);

  friend int operator<(const INT32 &a, const INT32 &b);
  friend int operator<(const INT32 &a, const LWORD &bl);
  friend int operator<(const LWORD &al, const INT32 &b);
  friend int operator<(const INT32 &a, const ULWORD &bl);
  friend int operator<(const ULWORD &al, const INT32 &b);
  friend int operator<(const INT32 &a, const WORD &bl);
  friend int operator<(const WORD &al, const INT32 &b);
  friend int operator<(const INT32 &a, const SWORD &bl);
  friend int operator<(const SWORD &al, const INT32 &b);
  friend int operator<(const INT32 &a, const UWORD &bl);
  friend int operator<(const UWORD &al, const INT32 &b);
  friend int operator<(const INT32 &a, const USWORD &bl);
  friend int operator<(const USWORD &al, const INT32 &b);
  friend int operator<(const INT32 &a, const SBYTE &bl);
  friend int operator<(const SBYTE &al, const INT32 &b);
  friend int operator<(const INT32 &a, const UBYTE &bl);
  friend int operator<(const UBYTE &al, const INT32 &b);

  friend int operator>(const INT32 &a, const INT32 &b);
  friend int operator>(const INT32 &a, const LWORD &bl);
  friend int operator>(const LWORD &al, const INT32 &b);
  friend int operator>(const INT32 &a, const ULWORD &bl);
  friend int operator>(const ULWORD &al, const INT32 &b);
  friend int operator>(const INT32 &a, const WORD &bl);
  friend int operator>(const WORD &al, const INT32 &b);
  friend int operator>(const INT32 &a, const SWORD &bl);
  friend int operator>(const SWORD &al, const INT32 &b);
  friend int operator>(const INT32 &a, const UWORD &bl);
  friend int operator>(const UWORD &al, const INT32 &b);
  friend int operator>(const INT32 &a, const USWORD &bl);
  friend int operator>(const USWORD &al, const INT32 &b);
  friend int operator>(const INT32 &a, const SBYTE &bl);
  friend int operator>(const SBYTE &al, const INT32 &b);
  friend int operator>(const INT32 &a, const UBYTE &bl);
  friend int operator>(const UBYTE &al, const INT32 &b);

  friend int operator<=(const INT32 &a, const INT32 &b);
  friend int operator<=(const INT32 &a, const LWORD &bl);
  friend int operator<=(const LWORD &al, const INT32 &b);
  friend int operator<=(const INT32 &a, const ULWORD &bl);
  friend int operator<=(const ULWORD &al, const INT32 &b);
  friend int operator<=(const INT32 &a, const WORD &bl);
  friend int operator<=(const WORD &al, const INT32 &b);
  friend int operator<=(const INT32 &a, const SWORD &bl);
  friend int operator<=(const SWORD &al, const INT32 &b);
  friend int operator<=(const INT32 &a, const UWORD &bl);
  friend int operator<=(const UWORD &al, const INT32 &b);
  friend int operator<=(const INT32 &a, const USWORD &bl);
  friend int operator<=(const USWORD &al, const INT32 &b);
  friend int operator<=(const INT32 &a, const SBYTE &bl);
  friend int operator<=(const SBYTE &al, const INT32 &b);
  friend int operator<=(const INT32 &a, const UBYTE &bl);
  friend int operator<=(const UBYTE &al, const INT32 &b);

  friend int operator>=(const INT32 &a, const INT32 &b);
  friend int operator>=(const INT32 &a, const LWORD &bl);
  friend int operator>=(const LWORD &al, const INT32 &b);
  friend int operator>=(const INT32 &a, const ULWORD &bl);
  friend int operator>=(const ULWORD &al, const INT32 &b);
  friend int operator>=(const INT32 &a, const WORD &bl);
  friend int operator>=(const WORD &al, const INT32 &b);
  friend int operator>=(const INT32 &a, const SWORD &bl);
  friend int operator>=(const SWORD &al, const INT32 &b);
  friend int operator>=(const INT32 &a, const UWORD &bl);
  friend int operator>=(const UWORD &al, const INT32 &b);
  friend int operator>=(const INT32 &a, const USWORD &bl);
  friend int operator>=(const USWORD &al, const INT32 &b);
  friend int operator>=(const INT32 &a, const SBYTE &bl);
  friend int operator>=(const SBYTE &al, const INT32 &b);
  friend int operator>=(const INT32 &a, const UBYTE &bl);
  friend int operator>=(const UBYTE &al, const INT32 &b);
  
private:
  SBYTE byte[4];
};

#endif // __INT32_HPP
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
