// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: persist.cpp 
// Compiler Used: MSVC40, DJGPP 2.7.2.1, GCC 2.7.2.1, HP CPP 10.24
// Produced By: Doug Gaer  
// File Creation Date: 09/18/97  
// Date Last Modified: 12/03/97
// Copyright (c) 1997 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBD C++ classes are copyright (c) 1997, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.
  
The (P)ersistent base class is used to define the interface 
that makes the object persistent. 
*/
// ----------------------------------------------------------- // 
#include "persist.hpp"

void Persistent::Connect(POD *DB) 
{
  if(!DB->OpenDatabase())
#ifdef CPP_EXCEPTIONS
    throw CNoDatabaseOpen();
#else
    Error->SignalException(EHandler::NoDatabaseOpen);
#endif
    pod = DB;
}

void Persistent::Connect(const POD *DB) 
{
  if(!DB->OpenDatabase())
#ifdef CPP_EXCEPTIONS
    throw CNoDatabaseOpen();
#else
    Error->SignalException(EHandler::NoDatabaseOpen);
#endif
    pod = (POD *)DB;
}

void Persistent::WriteObjHeader(const ObjectHeader &oh, FAU addr)
{
  pod->OpenDatabase()->Write(&oh, sizeof(ObjectHeader), addr);
}

void Persistent::ReadObjHeader(ObjectHeader &oh, FAU addr)
{
  pod->OpenDatabase()->Read(&oh, sizeof(ObjectHeader), addr);
}

unsigned Persistent::StringFileLength(const UString &s)
// Calculates the total number of bytes to be allocated for
// the string in the file.
{
  unsigned len = s.length();
  return len + sizeof(INT32); 
}

void Persistent::WriteString(const UString &s, FAU addr)
{
  INT32 len = s.length(); // Record the logical length of the string
  pod->OpenDatabase()->Write(&len, sizeof(INT32), addr);
  pod->OpenDatabase()->Write(s.c_str(), len, addr+sizeof(INT32));  
}

void Persistent::ReadString(UString &s, FAU addr)
{
  INT32 len ;
  pod->OpenDatabase()->Read(&len, sizeof(INT32), addr);
  char *buf = new char[len + 1];
  pod->OpenDatabase()->Read(buf, len, addr+sizeof(INT32));   
  buf[len] = '\0';
  s = buf;
  delete buf;
}

FAU Persistent::AddObject(int find)
// Will search the entire database for the object if find is true
{
  FAU addr;
  if(find) {
    addr = Find();
    if(!addr) addr = Write();
    return addr;
  }
  addr = Write();
  return addr;
}

FAU Persistent::DeleteObject()
{
  FAU addr = Find();
  if(!addr) return 0; // Object does not exist
  pod->OpenDatabase()->Delete(addr);
  return addr;
}

void Persistent::DeleteObject(FAU addr)
// Delete the object at the specifed address
{
  pod->OpenDatabase()->Delete(addr);
}

FAU Persistent::RemoveObject()
{
  FAU addr = Find();
  if(!addr) return 0; // Object does not exist
  pod->OpenDatabase()->Remove(addr);
  return addr;
}

void Persistent::RemoveObject(FAU addr)
// Remove the object at the specifed address
{
  pod->OpenDatabase()->Remove(addr);
}

FAU Persistent::ObjectAddress(int find)
// Will search the entire database for the object if find is true
{
  FAU addr;

  if(find) {
    addr = Find();
    if(!addr) return 0; // Object does not exist
    SetObjectAddress(addr);
    return addr;
  }

  return objectaddress;
}
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
