// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: strdb.cpp 
// Compiler Used: MSVC40, DJGPP 2.7.2.1, GCC 2.7.2.1, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 09/17/97  
// Date Last Modified: 12/04/97
// Copyright (c) 1997 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBD C++ classes are copyright (c) 1997, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The StrDB class is a general purpose string database used to
represent one unique key member, 7 general purpose members,
and one comment block. All the string members can grow and
shrink as needed
*/
// ----------------------------------------------------------- //   
#include "strdb.hpp"
#include "os_type.hpp"

// String compare routines that need to be ported from UNIX to DOS for
int NoCaseCmp(const char *s1, const char *s2)
// Compare two strings without regard to the case of the letters
{
#ifdef __DOS__
  return stricmp(s1, s2);
#endif

#ifdef __UNIX__
  return strcasecmp(s1, s2);
#endif
}

int NoCaseCmp(char *s1, char *s2)
// Compare two strings without regard to the case of the letters
{
#ifdef __DOS__
  return stricmp(s1, s2);
#endif

#ifdef __UNIX__
  return strcasecmp(s1, s2);
#endif
}

int NoCaseCmp(const UString &s1, const UString &s2)
// Compare two UString objects without regard to the case of the letters
{
#ifdef __DOS__
  return stricmp(s1.c_str(), s2.c_str());
#endif

#ifdef __UNIX__
  return strcasecmp(s1.c_str(), s2.c_str());
#endif
}

int NoCaseCmp(UString &s1, UString &s2)
// Compare two UString objects without regard to the case of the letters
{
#ifdef __DOS__
  return stricmp(s1.c_str(), s2.c_str());
#endif

#ifdef __UNIX__
  return strcasecmp(s1.c_str(), s2.c_str());
#endif
}

unsigned StrDB::ObjectLength()
{
  unsigned len = StringFileLength(KM) + \
    StringFileLength(M2) + \
    StringFileLength(M3)  + \
    StringFileLength(M4)  + \
    StringFileLength(M5)  + \
    StringFileLength(M6)  + \
    StringFileLength(M7)  + \
    StringFileLength(M8)  + \
    StringFileLength(CM);

  return len;
}

FAU StrDB::Write()
{
  ObjectHeader oh;

  FAU addr = pod->OpenDatabase()->Alloc(ObjectLength() + sizeof(ObjectHeader));
  if(!addr) return 0;
  
  oh.ClassID = GetClassID();
  oh.ObjectID = addr;
  WriteObjHeader(oh, addr);

  FAU offset = addr+sizeof(ObjectHeader);
  WriteString(KM, offset);
  offset = offset + StringFileLength(KM);
  WriteString(M2, offset);
  offset = offset + StringFileLength(M2);
  WriteString(M3, offset);
  offset = offset + StringFileLength(M3);
  WriteString(M4, offset);
  offset = offset + StringFileLength(M4);
  WriteString(M5, offset);
  offset = offset + StringFileLength(M5);
  WriteString(M6, offset);
  offset = offset + StringFileLength(M6);
  WriteString(M7, offset);
  offset = offset + StringFileLength(M7);
  WriteString(M8, offset);
  offset = offset + StringFileLength(M8);
  WriteString(CM, offset);
  objectaddress = addr;  
  return addr;
}

void StrDB::Read(FAU Address)
{
  ObjectHeader oh;

  ReadObjHeader(oh, Address);
  if(oh.ClassID != GetClassID()) return; // Incorrect object type

  FAU offset = Address+sizeof(ObjectHeader);
  ReadString(KM, offset);
  offset = offset + StringFileLength(KM);
  ReadString(M2, offset);
  offset = offset + StringFileLength(M2);
  ReadString(M3, offset);
  offset = offset + StringFileLength(M3);
  ReadString(M4, offset);
  offset = offset + StringFileLength(M4);
  ReadString(M5, offset);
  offset = offset + StringFileLength(M5);
  ReadString(M6, offset);
  offset = offset + StringFileLength(M6);
  ReadString(M7, offset);
  offset = offset + StringFileLength(M7);
  ReadString(M8, offset);
  offset = offset + StringFileLength(M8);
  ReadString(CM, offset);
  objectaddress = Address;
}

FAU StrDB::Find()
// Find object by seaching the database for its primary data members
{
  UString buf;
  StrDB strdb;
  FAU addr = 0;
  ObjectHeader oh;
  int result;
  
  strdb.KM = this->KM; 
  strdb.M2 = this->M2;
  strdb.M3 = this->M3;
  strdb.M4 = this->M4;
  strdb.M5 = this->M5;
  strdb.M6 = this->M6;
  strdb.M7 = this->M7;
  strdb.M8 = this->M8;
  strdb.CM = this->CM;
  
  while(1)
    {
      addr = pod->OpenDatabase()->FindFirstObject(addr);
      if(!addr) break;
      ReadObjHeader(oh, addr);
      if(oh.ClassID == GetClassID()) {
	Read(addr);
	result = NoCaseCmp(KM, strdb.KM);
	if(result == 0) {
	  objectaddress = addr;
	  return addr; // Found unique data member
	}
	else {
	  // Reset the objects data
	  this->KM = strdb.KM;
	  this->M2 = strdb.M2;
	  this->M3 = strdb.M3;
	  this->M4 = strdb.M4;
	  this->M5 = strdb.M5;
	  this->M6 = strdb.M6;
	  this->M7 = strdb.M7;
	  this->M8 = strdb.M8;
	  this->CM = strdb.CM;
	}
      }
    }
  return 0; // Could not find 
}

void StrDB::Copy(const StrDB &ob)
{
  KM = ob.KM;
  M2 = ob.M2;
  M3 = ob.M3;
  M4 = ob.M4;
  M5 = ob.M5;
  M6 = ob.M6;
  M7 = ob.M7;
  M8 = ob.M8;
  CM = ob.CM;
}

int StrDB::FullCompare(const StrDB &ob)
{
  if(KM != ob.KM) return 0;
  if(M2 != ob.M2) return 0;
  if(M3 != ob.M3) return 0;
  if(M4 != ob.M4) return 0;
  if(M5 != ob.M5) return 0;
  if(M6 != ob.M6) return 0;
  if(M7 != ob.M7) return 0;
  if(M8 != ob.M8) return 0;
  if(CM != ob.CM) return 0;
  return 1;
}

int operator==(const StrDB &a, const StrDB &b)
{
  return NoCaseCmp(a.GetKM(), b.GetKM()) == 0;
}

int operator!=(const StrDB &a, const StrDB &b)
{
  return NoCaseCmp(a.GetKM(), b.GetKM()) != 0;
}

int operator>(const StrDB &a, const StrDB &b)
{
  return NoCaseCmp(a.GetKM(), b.GetKM()) > 0;
}

int operator>=(const StrDB &a, const StrDB &b)
{
  return NoCaseCmp(a.GetKM(), b.GetKM()) >= 0;
}
  
int operator<(const StrDB &a, const StrDB &b)
{
  return NoCaseCmp(a.GetKM(), b.GetKM()) < 0;
}
  
int operator<=(const StrDB &a, const StrDB &b)
{
  return NoCaseCmp(a.GetKM(), b.GetKM()) <= 0;
}
// ----------------------------------------------------------- // 
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
// ------------------------------- //
