// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: strdbt.cpp 
// Compiler Used: MSVC40, DJGPP 2.7.2.1, GCC 2.7.2.1, HP CPP 10.24
// Produced By: Doug Gaer  
// File Creation Date: 09/18/97  
// Date Last Modified: 12/04/97
// Copyright (c) 1997 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBD C++ classes are copyright (c) 1997, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The string database is a general purpose object-oriented
database. This version is designed to work with a terminal
interface.

- Modify "dbconfig.hpp" file to customize the database
- Modify "dbvers.hpp" file to uniquely identify the database
- Modify "dbtcfg.hpp" file to customize this program
- Modify "version.hpp" file to report changes and fixes
- Modify "os_type.hpp" file to compile for a different OS
*/
// ----------------------------------------------------------- // 
#include <fstream.h> // Using ofstream to print to a text file
#include "rbtree.hpp"
#include "twalk.hpp"
#include "strdb.hpp"
#include "terminal.hpp"
#include "config.hpp"
#include "dbconfig.hpp"
#include "timer.hpp"
#include "version.hpp"
#include "dbtcfg.hpp"

static POD *DB;                // Global database pointer
static RBTree<StrDB> *cache;   // Global cache pointer
const int KeyBuf = 25;         // Users input limit per string
const int CBuf = 81;           // Input limit for comments
const int StringOffset = 15;   // Display output limit per string
const int IntOffset = 8;       // Display output limit per number
const char *WildCard = "*";    // Used for wild card searches 
const int retrys = 3;          // Times to retry user input operations

// Menu menu setup
typedef void (*MMF) (); // Pointer to (M)ain (M)enu (F)unctions
const int MMSelections = 6; // Main menu selections

// Main menu functions
void Exit();
void ISRMenu();     // Insertion/Removal menu
void FindMenu();    // Query menu
void DisplayMenu(); // Display menu
void FileMenu();    // File operations menu
void PrintMenu();   // Print menu

const char *mainmenu[MMSelections] = {
  "(0) - Exit this program",
  "(1) - Add, Remove, or Change items in the database",
  "(2) - Find items in the database",
  "(3) - Display items in the database",
  "(4) - File Operations",
  "(5) - Print Menu"
};

MMF MMFunctions[MMSelections] = {
  &Exit,
  &ISRMenu,
  &FindMenu,
  &DisplayMenu,
  &FileMenu,
  &PrintMenu
};

// Insertion/Removal menu setup
typedef void (*ISRMF) (); // Pointer to (A)dd (M)enu (F)unctions 
const char *ISRMTitle = "--------- DATABASE INSERTION/REMOVAL MENU ---------";
const int ISRMSelections = 5; // Insertion/Removal menu selections

// Insertion/Removal menu functions
void Return();
void Add();
void Change();
void Delete();
void Remove();

const char *isrmenu[ISRMSelections] = {
  "(0) - Return to main menu",
  "(1) - Add items to the database",
  "(2) - Change an item in the database",
  "(3) - Delete an item (delete without destroying)",
  "(4) - Remove an item (permanently remove object)"
};

ISRMF ISRMFunctions[ISRMSelections] = {
  &Return,
  &Add,
  &Change,
  &Delete,
  &Remove
};

// Find menu setup
typedef void (*FMF) (); // Pointer to (F)ind (M)enu (F)unctions 
const char *FMTitle = "---------- DATABASE QUERY MENU ----------";
const int FMSelections = 10; // Find menu selections

// Find menu functions
void Return();
void FindByKeyName(); 
void FindByM2Name(); 
void FindByM3Name(); 
void FindByM4Name(); 
void FindByM5Name(); 
void FindByM6Name(); 
void FindByM7Name(); 
void FindByM8Name();
void FindByComments();

FMF FMFunctions[FMSelections] = {
  &Return,
  &FindByKeyName,
  &FindByM2Name,
  &FindByM3Name,
  &FindByM4Name,
  &FindByM5Name,
  &FindByM6Name,
  &FindByM7Name,
  &FindByM8Name,
  &FindByComments
};

// Display menu setup
typedef void (*DMF) (); // Pointer to (D)isplay (M)enu (F)unctions 
const char *DMTitle = "---------- DATABASE DISPLAY MENU ----------";
const int DMSelections = 3; // Display menu selections

// Display menu functions
void DisplayLBL(); // Display all objects line by line
void DisplayAll(); // Display all objects one at a time
  
const char *displaymenu[DMSelections] = {
  "(0) - Return to main menu",
  "(1) - Display all items line by line",
  "(2) - Display all items one a time"
};

DMF DMFunctions[DMSelections] = {
  &Return,
  &DisplayLBL,
  &DisplayAll
};

// File operations menu setup
typedef void (*FOMF) (); // Pointer to (F)ile (M)enu (F)unctions 
const char *FOMTitle = "---------- DATABASE FILE OPERATIONS MENU ----------";
const int FOMSelections = 8; // File operations menu selections

// File operations menu functions
void FileStats();
void LoadCache();
void ExportToASCII();
void ImportFromASCII();
void BackUp();
void Merge();
void CreateTemplate();

const char *filemenu[FOMSelections] = {
  "(0) - Return to main menu",
  "(1) - Display Variable Block Database statistics",
  "(2) - Reload memory cache",
  "(3) - Export to ASCII file (delimited by tabs)",
  "(4) - Import ASCII file (delimited by tabs)",
  "(5) - Backup the database to another file",
  "(6) - Merge the contents of another database file",
  "(7) - Create a template for an import file"
};

FOMF FOMFunctions[FOMSelections] = {
  &Return,
  &FileStats,
  &LoadCache,
  &ExportToASCII,
  &ImportFromASCII,
  &BackUp,
  &Merge,
  &CreateTemplate
};

// Print menu setup
typedef void (*PMF) (); // Pointer to (P)rint (M)enu (F)unctions 
const char *PMTitle = "---------- DATABASE PRINTING MENU ----------";
const int PMSelections = 4; // Print menu selections

// Print menu functions
void ASCIIPrintAll();     // Print database to an ASCII file (abbreviated)
void ASCIIPrintAllLong(); // Print database to an ASCII file (full listing) 
void ASCIIPrintObject();  // Print a single object to an ASCII file

const char *printmenu[PMSelections] = {
  "(0) - Return to main menu",
  "(1) - Print to text file (abbreviated,  80 cols)",
  "(2) - Print to text file (full listing, 132 cols)",
  "(3) - Print a single object to a text file"
};

PMF PMFunctions[PMSelections] = {
  &Return,
  &ASCIIPrintAll,
  &ASCIIPrintAllLong,
  &ASCIIPrintObject
};

// Function prototypes for functions use by all menu functions
void AddObject(StrDB &strdb, Coords *p);
void ChangeObject(StrDB &strdb, Coords *p, int adding = 0);
void DisplayObject(StrDB &strdb, Coords *p);
void LineByLine(StrDB &strdb, Coords *p);
void DisplayStats(const VBDFilePtr &f);
void DisplayString(const char *s);
RBNode<StrDB> *FindKeyMember(char *s);
RBNode<StrDB> *FindKeyMember(const char *s);

// General purpose string parser 
const int MAXWORDLENGTH = 80;
const int MAXWORDS = 10;
int parse(char *string, char words[MAXWORDS][MAXWORDLENGTH],
	  int*numwords, char sepchar);

enum StrDBItem { // Database items used in global searches
  KEYMEMBER,
  M2NAME,
  M3NAME,
  M4NAME,
  M5NAME,
  M6NAME,
  M7NAME,
  M8NAME,
  COMMENTS
};

void PlainSearch(StrDB &strdb, StrDBItem item, UString &str, Coords *p);
void WildCardSearch(StrDB &strdb, StrDBItem item, UString &str, Coords *p);

// Function prototypes and constants for print functions
const int PrintCols = 80;      // Columns for portrait print outs
const int PrintColsLong = 132; // Columns for lanscape print outs
const int PrintRows = 25;      // Rows for portrait print outs
const int PrintRowsLong = 25;  // Rows for lanscape print outs
const char *LineFeed = "\n";

enum PrintMode {
  PORTRAIT,
  LANDSCAPE
};

void FPrint(const char *s, ofstream &stream, int offset, int pos = 0);
void FPrint(const char c, ofstream &stream, int offset, int pos = 0);
void FPrint(int val, ofstream &stream, int offset, int pos = 0);
void FPrint(long val, ofstream &stream, int offset, int pos = 0);
void FPrint(double val, ofstream &stream, int offset, int pos = 0);
void PrintItemBar(ofstream &stream, int abv = 1);
void PrintLineByLine(StrDB &strdb, ofstream &stream, int abv = 1);
int ASCIIPrint(PrintMode mode);
void PrintObject(StrDB &strdb, Coords *p);

void MainMenu()
{
  int c;
  Coords *p = new Coords(0, 0);

  while(1) {
    p->SetXY(0, 0);
    p->SetX(terminal->Center(MMTitle));
    p->SetY(terminal->ScreenCenter(MMSelections));
    terminal->ClearScreen();
    terminal->Write(MMTitle, p->XPos(), p->YPrev());

    for(int i = 0; i < MMSelections; i++)
      terminal->Write(mainmenu[i], p->XPos(), p->YOffset(1));
    
    terminal->Write("Press the number of your selection -> ",
		    p->XPos(), p->YOffset(2));
    
    c = terminal->GetChar();
    switch(c) {
      case '0': 
	(*MMFunctions[0]) (); // Call the appropriate menu function
	return;
      case '1':
	(*MMFunctions[1]) (); // Call the appropriate menu function
	break;
      case '2':
	(*MMFunctions[2]) (); // Call the appropriate menu function
	break;
      case '3':
	(*MMFunctions[3]) (); // Call the appropriate menu function
	break;
      case '4':
	(*MMFunctions[4]) (); // Call the appropriate menu function
	break;
      case '5':
	(*MMFunctions[5]) (); // Call the appropriate menu function
	break;
      default:
	break;
    }
  }
}

void ISRMenu()
{
  int c;
  Coords *p = new Coords(0, 0);

  while(1) {
    p->SetXY(0, 0);
    p->SetX(terminal->Center(ISRMTitle));
    p->SetY(terminal->ScreenCenter(ISRMSelections));
    terminal->ClearScreen();
    terminal->Write(ISRMTitle, p->XPos(), p->YPrev());

    for(int i = 0; i < ISRMSelections; i++)
      terminal->Write(isrmenu[i], p->XPos(), p->YOffset(1));
    
    terminal->Write("Press the number of your selection -> ",
		    p->XPos(), p->YOffset(2));
    
    c = terminal->GetChar();
    switch(c) {
      case '0': 
	(*ISRMFunctions[0]) (); // Call the appropriate menu function
	return;
      case '1':
	(*ISRMFunctions[1]) (); // Call the appropriate menu function
	break;
      case '2':
	(*ISRMFunctions[2]) (); // Call the appropriate menu function
	break;
      case '3':
	(*ISRMFunctions[3]) (); // Call the appropriate menu function
	break;
      case '4':
	(*ISRMFunctions[4]) (); // Call the appropriate menu function
	break;
      default:
	break;
    }
  }
}

void FindMenu()
{
  int c;
  Coords *p = new Coords(0, 0);

  while(1) {
    p->SetXY(0, 0);
    p->SetX(terminal->Center(FMTitle));
    p->SetY(terminal->ScreenCenter(FMSelections));
    terminal->ClearScreen();
    terminal->Write(FMTitle, p->XPos(), p->YPrev());

    terminal->Write("(0) - Return to main menu", p->XPos(), p->YOffset(1));
    if(KeyName != 0) {
      terminal->Write("(1) - Find by ", p->XPos(), p->YOffset(1));
      terminal->Write(KeyName);
    }
    if(M2Name != 0) {
      terminal->Write("(2) - Find by ", p->XPos(), p->YOffset(1));
      terminal->Write(M2Name);
    }
    if(M3Name != 0) {
      terminal->Write("(3) - Find by ", p->XPos(), p->YOffset(1));
      terminal->Write(M3Name);
    }
    if(M4Name != 0) {
      terminal->Write("(4) - Find by ", p->XPos(), p->YOffset(1));
      terminal->Write(M4Name);
    }
    if(M5Name != 0) {
      terminal->Write("(5) - Find by ", p->XPos(), p->YOffset(1));
      terminal->Write(M5Name);
    }
    if(M6Name != 0) {
      terminal->Write("(6) - Find by ", p->XPos(), p->YOffset(1));
      terminal->Write(M6Name);
    }
    if(M7Name != 0) {
      terminal->Write("(7) - Find by ", p->XPos(), p->YOffset(1));
      terminal->Write(M7Name);
    }
    if(M8Name != 0) {
      terminal->Write("(8) - Find by ", p->XPos(), p->YOffset(1));
      terminal->Write(M8Name);
    }
    if(Comments != 0) {
      terminal->Write("(9) - Find by ", p->XPos(), p->YOffset(1));
      terminal->Write(Comments);
    }
    
    terminal->Write("Press the number of your selection -> ",
		    p->XPos(), p->YOffset(2));
    
    c = terminal->GetChar();
    switch(c) {
      case '0': 
	(*FMFunctions[0]) (); // Call the appropriate menu function
	return;
      case '1':
	(*FMFunctions[1]) (); // Call the appropriate menu function
	break;
      case '2':
	(*FMFunctions[2]) (); // Call the appropriate menu function
	break;
      case '3':
	(*FMFunctions[3]) (); // Call the appropriate menu function
	break;
      case '4':
	(*FMFunctions[4]) (); // Call the appropriate menu function
	break;
      case '5':
	(*FMFunctions[5]) (); // Call the appropriate menu function
	break;
      case '6':
	(*FMFunctions[6]) (); // Call the appropriate menu function
	break;
      case '7':
	(*FMFunctions[7]) (); // Call the appropriate menu function
	break;
      case '8':
	(*FMFunctions[8]) (); // Call the appropriate menu function
	break;
      case '9':
	(*FMFunctions[9]) (); // Call the appropriate menu function
	break;
      default:
	break;
    }
  }
}

void DisplayMenu()
{
  int c;
  Coords *p = new Coords(0, 0);

  while(1) {
    p->SetXY(0, 0);
    p->SetX(terminal->Center(DMTitle));
    p->SetY(terminal->ScreenCenter(DMSelections));
    terminal->ClearScreen();
    terminal->Write(DMTitle, p->XPos(), p->YPrev());

    for(int i = 0; i < DMSelections; i++)
      terminal->Write(displaymenu[i], p->XPos(), p->YOffset(1));
    
    terminal->Write("Press the number of your selection -> ",
		    p->XPos(), p->YOffset(2));
    
    c = terminal->GetChar();
    switch(c) {
      case '0': 
	(*DMFunctions[0]) (); // Call the appropriate menu function
	return;
      case '1':
	(*DMFunctions[1]) (); // Call the appropriate menu function
	break;

      case '2':
	(*DMFunctions[2]) (); // Call the appropriate menu function
	break;
	
      default:
	break;
    }
  }
}

void FileMenu()
{
  int c;
  Coords *p = new Coords(0, 0);

  while(1) {
    p->SetXY(0, 0);
    p->SetX(terminal->Center(FOMTitle));
    p->SetY(terminal->ScreenCenter(FOMSelections));
    terminal->ClearScreen();
    terminal->Write(FOMTitle, p->XPos(), p->YPrev());

    for(int i = 0; i < FOMSelections; i++)
      terminal->Write(filemenu[i], p->XPos(), p->YOffset(1));
    
    terminal->Write("Press the number of your selection -> ",
		    p->XPos(), p->YOffset(2));
    
    c = terminal->GetChar();
    switch(c) {
      case '0': 
	(*FOMFunctions[0]) (); // Call the appropriate menu function
	return;
      case '1':
	(*FOMFunctions[1]) (); // Call the appropriate menu function
	break;
      case '2':
	(*FOMFunctions[2]) (); // Call the appropriate menu function
	break;
      case '3':
	(*FOMFunctions[3]) (); // Call the appropriate menu function
	break;
      case '4':
	(*FOMFunctions[4]) (); // Call the appropriate menu function
	break;
      case '5':
	(*FOMFunctions[5]) (); // Call the appropriate menu function
	break;
      case '6':
	(*FOMFunctions[6]) (); // Call the appropriate menu function
	break;
      case '7':
	(*FOMFunctions[7]) (); // Call the appropriate menu function
	break;
      default:
	break;
    }
  }
}

void PrintMenu()
{
  int c;
  Coords *p = new Coords(0, 0);

  while(1) {
    p->SetXY(0, 0);
    p->SetX(terminal->Center(PMTitle));
    p->SetY(terminal->ScreenCenter(PMSelections));
    terminal->ClearScreen();
    terminal->Write(PMTitle, p->XPos(), p->YPrev());

    for(int i = 0; i < PMSelections; i++)
      terminal->Write(printmenu[i], p->XPos(), p->YOffset(1));
    
    terminal->Write("Press the number of your selection -> ",
		    p->XPos(), p->YOffset(2));
    
    c = terminal->GetChar();
    switch(c) {
      case '0': 
	(*PMFunctions[0]) (); // Call the appropriate menu function
	return;
      case '1':
	(*PMFunctions[1]) (); // Call the appropriate menu function
	break;
      case '2':
	(*PMFunctions[2]) (); // Call the appropriate menu function
	break;
      case '3':
	(*PMFunctions[3]) (); // Call the appropriate menu function
	break;
      default:
	break;
    }
  }
}

void Exit()
{
  // Nothing to do
}

void Return()
{
  // Nothing to do
}

RBNode<StrDB> *FindKeyMember(char *s)
// Non-case sensitive search. Returns false if found.
{
  int result = 0;
  StrDB ob; 
  BNodeBase *nxt;     // BSTree base node pointer
  RBNode<StrDB> *ptr; // RB Tree Node pointer 
    
  // Using iterator object to walk throught the tree
  TreeWalkerb tw(cache->GetRoot(), INORDER);
  
  while((nxt = tw.Next()) != 0) {
    ptr = (RBNode<StrDB> *)nxt;
    ob.Copy(ptr->Data);
    result = NoCaseCmp(s, ob.GetKM());
    if(result == 0) return ptr; // Found primary circuit number
  }
  return 0; // No match found
}

RBNode<StrDB> *FindKeyMember(const char *s)
// Non-case sensitive search. Returns false if not found.
{
  int result = 0;
  StrDB ob; 
  BNodeBase *nxt;     // BSTree base node pointer
  RBNode<StrDB> *ptr; // RB Tree Node pointer 
    
  // Using iterator object to walk throught the tree
  TreeWalkerb tw(cache->GetRoot(), INORDER);
  
  while((nxt = tw.Next()) != 0) {
    ptr = (RBNode<StrDB> *)nxt;
    ob.Copy(ptr->Data);
    result = NoCaseCmp(s, ob.GetKM());
    if(result == 0) return ptr; // Found primary circuit number
  }
  return 0; // No match found
}

void DisplayAll()
{
  StrDB strdb(DB);
  Coords *p = new Coords(0, 0);
  int c, yn;
  BNodeBase *nxt; // BSTree base node pointer

  // Using iterator object to walk throught the tree
  TreeWalkerb tw(cache->GetRoot(), INORDER);

  while((nxt = tw.Next()) != 0) {
    terminal->ClearScreen();      
    p->SetXY(0, 0); // Reset x and y positions
    RBNode<StrDB> *ptr = (RBNode<StrDB> *)nxt;
    DisplayObject(ptr->Data, p);
    terminal->Write("(C)-Change, (D)-Delete, (P)-Print, (X)-Exit",
		    p->XPos(), p->YOffset(2));
    terminal->Write(", Any other key to continue");
    c = terminal->GetChar();
    switch(c) {
      case 'c': case 'C': 
	terminal->ClearScreen();
	p->SetXY(0, 0);
        strdb.Copy(ptr->Data);
        strdb.ReadObject(strdb.ObjectAddress());
        ChangeObject(strdb, p);
	break;
	
      case 'd': case 'D':
	yn = terminal->YesNo("Are you sure you want to delete (y/n)",
			     p->XPos(), p->YOffset(2));
	if(!yn) break;
        strdb.Copy(ptr->Data);
        cache->Delete(strdb); // Update the memory cache
        strdb.DeleteObject(strdb.ObjectAddress());
	break;
	
      case 'p': case 'P':
        strdb.Copy(ptr->Data);
	p->YOffset(1);
        PrintObject(strdb, p);
	break;

      case 'x': case 'X':
	return;

      default:
	break;
    }
  }
}

void ItemBar(Coords *p)
// Displays the Key data member and the data member 2 - 5
{
  int x = p->XPos(); 
  if(KeyName != 0) terminal->Write(KeyName, p->XPos(), p->YPos());
  if(M2Name != 0)
    terminal->Write(M2Name, p->XOffset(StringOffset+1), p->YPos());
  if(M3Name != 0)
    terminal->Write(M3Name, p->XOffset(StringOffset+1), p->YPos());
  if(M4Name != 0)
    terminal->Write(M4Name, p->XOffset(StringOffset+1), p->YPos());
  if(M5Name != 0)
    terminal->Write(M5Name, p->XOffset(StringOffset+1), p->YPos());
  terminal->MoveCursor(0, p->YOffset(1));
  int i = 0;
  while (i++ < terminal->MaxCols()-1) terminal->Write('=');
  p->YOffset(1);
  p->SetX(x);
}

void DisplayLBL()
{
  StrDB strdb(DB);
  int c;
  unsigned count = 0;
  Coords *p = new Coords(0, 0);
  terminal->ClearScreen();
  ItemBar(p);
  BNodeBase *nxt; // BSTree base node pointer

  // Using iterator object to walk throught the tree
  TreeWalkerb tw(cache->GetRoot(), INORDER);

  while((nxt = tw.Next()) != 0) {
    RBNode<StrDB> *ptr = (RBNode<StrDB> *)nxt;
    LineByLine(ptr->Data, p);
    p->YOffset(1);
    count++;
    if(count > 5) {
    terminal->Write("(X)-Exit", p->XPos(), p->YOffset(2));
    terminal->Write(", Any other key to continue");
    c = terminal->GetChar();
    switch(c) {
      case 'x': case 'X':
	return;
	
      default:
	break;
    }
    terminal->ClearScreen();
    p->SetXY(0, 0);
    ItemBar(p);
    count = 0;
    }
  }
  terminal->AnyKey(p->XPos(), p->YOffset(2));
  delete p; // Free Coords pointer
}

void Delete()
{
  if(KeyName == 0) return;
  
  terminal->ClearScreen();
  Coords *p = new Coords(0, 0);
  char buf[KeyBuf]; 
  StrDB strdb(DB);
  terminal->Write(KeyName, p->XPos(), p->YPos());
  terminal->Write(" to delete -> ");
  terminal->GetString(buf);

  RBNode<StrDB> *ptr = FindKeyMember(buf);
  if(!ptr) { 
    terminal->Write("Could not find: ", p->XPos(), p->YOffset(2));
    terminal->Write(buf);
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    delete p; // Free the Coord pointer
    return;
  }
  int yn = terminal->YesNo("Are you sure you want to delete (y/n)",
			   p->XPos(), p->YOffset(2));
  if(!yn) {
    delete p; // Free Coord pointer
    return;
  }
  
  strdb.Copy(ptr->Data);                   
  cache->Delete(strdb); // Update the memory cache
  strdb.DeleteObject(strdb.ObjectAddress());
  terminal->Write("Deleted entry for: ", p->XPos(), p->YOffset(2));
  terminal->Write(buf);
  terminal->AnyKey(p->XPos(), p->YOffset(2));
  delete p; // Free Coord pointer
}

void Remove()
{
  if(KeyName == 0) return;
  
  terminal->ClearScreen();
  Coords *p = new Coords(0, 0);
  char buf[KeyBuf]; 
  StrDB strdb(DB);
  terminal->Write(KeyName, p->XPos(), p->YPos());
  terminal->Write(" to remove -> ");
  terminal->GetString(buf);

  RBNode<StrDB> *ptr = FindKeyMember(buf);

  if(!ptr) { 
    terminal->Write("Could not find: ", p->XPos(), p->YOffset(2));
    terminal->Write(buf);
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    delete p; // Free the Coord pointer
    return;
  }
  int yn = terminal->YesNo("Are you sure you want to remove (y/n)",
			   p->XPos(), p->YOffset(2));
  if(!yn) {
    delete p; // Free Coord pointer
    return;
  }

  strdb.Copy(ptr->Data);                   
  cache->Delete(strdb); // Update the memory cache
  strdb.RemoveObject(strdb.ObjectAddress());
  terminal->Write("Removed entry for: ", p->XPos(), p->YOffset(2));
  terminal->Write(buf);
  terminal->AnyKey(p->XPos(), p->YOffset(2));
  delete p; // Free Coord pointer
}

void FileStats()
{
  DisplayStats(DB->OpenDatabase());
}

void FPrint(const char *s, ofstream &stream, int offset, int pos)
{
  int i = 0;
  int len = strlen(s);
  char *buf = new char[len + 1];
  buf[len] = '\0';
  strcpy(buf, s);
  if(len > offset) len = offset-1; // Subtract 1 to avoid overlaps
  if(pos) for(i = 0; i < pos; i++) stream << ' '; // Position the text
  for(i = 0; i < len; i++) stream << buf[i];
  if(len < offset) {
    offset = offset - len;
    for(i = 0; i < offset; i++) stream << ' ';
  }
  delete[] buf;
}

void FPrint(const char c, ofstream &stream, int offset, int pos)
{
  char s[1];
  s[0] = c;    // Copy char into a string buffer
  s[1] = '\0'; // Null terminate the string
  FPrint(s, stream, offset, pos);
}
  
void FPrint(int val, ofstream &stream, int offset, int pos)
{
  char *buf = new char[offset+1];
  buf[offset] = '\0';
  sprintf(buf, "%d", val);
  FPrint(buf, stream, offset, pos);
  delete[] buf; 
}

void FPrint(long val, ofstream &stream, int offset, int pos)
{
  char *buf = new char[offset+1];
  buf[offset] = '\0';
  sprintf(buf, "%d", val);
  FPrint(buf, stream, offset, pos);
  delete[] buf; 
}

void FPrint(double val, ofstream &stream, int offset, int pos)
{
  char *buf = new char[offset+1];
  buf[offset] = '\0';
  sprintf(buf, "%g", val);
  FPrint(buf, stream, offset, pos);
  delete[] buf; 
}

void PrintItemBar(ofstream &stream, int abv)
{

  int i = 0;
  if(KeyName != 0) FPrint(KeyName, stream, StringOffset+1);
  if(M2Name != 0) FPrint(M2Name, stream, StringOffset+1);
  if(M3Name != 0) FPrint(M3Name, stream, StringOffset+1);
  if(M4Name != 0) FPrint(M4Name, stream, StringOffset+1);
  if(abv) { // Print abbreviated list for portrait print outs
    if(M5Name != 0) FPrint(M5Name, stream, StringOffset);
    stream << LineFeed;
    i = 0;
    while (i++ < PrintCols-1) stream << '=';
    stream << LineFeed;
    return;
  }
  else { // Print full listing for landscape print outs
    if(M5Name != 0) FPrint(M5Name, stream, StringOffset+1);
    if(M6Name != 0) FPrint(M6Name, stream, StringOffset+1);
    if(M7Name != 0) FPrint(M7Name, stream, StringOffset+1);
    if(M8Name != 0) FPrint(M8Name, stream, StringOffset);
    stream << LineFeed;
    i = 0;
    while (i++ < PrintColsLong-1) stream << '=';
    stream << LineFeed;
  }
}

void PrintLineByLine(StrDB &strdb, ofstream &stream, int abv)
{
  int i = 0;
  
  if(KeyName != 0) FPrint(strdb.GetKM(), stream, StringOffset+1);
  if(M2Name != 0) FPrint(strdb.GetM2(), stream, StringOffset+1);
  if(M3Name != 0) FPrint(strdb.GetM3(), stream, StringOffset+1);
  if(M4Name != 0) FPrint(strdb.GetM4(), stream, StringOffset+1);
  if(abv) { // Print abbreviated list for portrait print outs
    if(M5Name != 0) FPrint(strdb.GetM5(), stream, StringOffset);
    stream << LineFeed;
    i = 0;
    while (i++ < PrintCols-1) stream << '-';
    stream << LineFeed;
    return;
  }
  else { // Print full listing for landscape print outs
    if(M5Name != 0) FPrint(strdb.GetM5(), stream, StringOffset+1);
    if(M6Name != 0) FPrint(strdb.GetM6(), stream, StringOffset+1);
    if(M7Name != 0) FPrint(strdb.GetM7(), stream, StringOffset+1);
    if(M8Name != 0) FPrint(strdb.GetM8(), stream, StringOffset);
    stream << LineFeed;
    i = 0;
    while (i++ < PrintColsLong-1) stream << '-';
    stream << LineFeed;
  }
}

void ASCIIPrintAll()
{
  ASCIIPrint(PORTRAIT);
}

void ASCIIPrintAllLong()
{
  ASCIIPrint(LANDSCAPE);
}

int ASCIIPrint(PrintMode mode)
{
  char buf[CBuf];
  char *FileName;
  terminal->ClearScreen();
  Coords *p = new Coords(0, 0);

  terminal->Write("Enter file name to print to: ",
		  p->XPos(), p->YPos());
  terminal->GetString(buf);

  char *tmp = new char[strlen(buf)+1];
  tmp[strlen(buf)] = '\0';
  strcpy(tmp, buf);
  
  if(VBDFile::Exists(tmp)) {
    terminal->Write("File: ",
		    p->XPos(), p->YOffset(2));
    terminal->Write(tmp);
    terminal->Write(" already exists.");
    int yn = terminal->YesNo("Overwrite it (y/n)?", p->XPos(), p->YOffset(2));
    if(!yn) {
      delete p;
      delete[] tmp;
      return 0;
    }
  }

  FileName = tmp;
  delete[] tmp;
  
  ofstream stream(FileName, ios::out); // Open file and truncate it
  terminal->Write("Printing to: ",
		  p->XPos(), p->YOffset(2));
  terminal->Write(FileName);
  
  if(!stream) { // Could not open the stream
    terminal->Write("Could not write to: ",
		    p->XPos(), p->YOffset(2));
    terminal->Write(FileName);
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    delete p; // Free the Coords pointer
    return 0; 
  }

  terminal->Write("Printing...", p->XPos(), p->YOffset(2));
		  
  StrDB strdb(DB);
  unsigned count = 0;
  BNodeBase *nxt; // BSTree base node pointer

  // Using iterator object to walk throught the tree
  TreeWalkerb tw(cache->GetRoot(), INORDER);

  switch(mode) {
    case PORTRAIT: 
      PrintItemBar(stream);
      while((nxt = tw.Next()) != 0) {
        RBNode<StrDB> *ptr = (RBNode<StrDB> *)nxt;
	PrintLineByLine(ptr->Data, stream);
	count++; // Used to count the items printed
      }
      break;
      
    case LANDSCAPE: 
      PrintItemBar(stream, 0);
      while((nxt = tw.Next()) != 0) {
        RBNode<StrDB> *ptr = (RBNode<StrDB> *)nxt;
	PrintLineByLine(ptr->Data, stream, 0);
	count++; // Used to count the items printed
      }
      break;
      
    default:
      delete p;          // Free the Coords pointer
      return 0;
  }
  terminal->Write("Finished", p->XPos(), p->YOffset(2));
  terminal->AnyKey(p->XPos(), p->YOffset(2));
  delete p;          // Free the Coords pointer
  stream.close();
  return count;
}

void ASCIIPrintObject()
{
  if(KeyName == 0) return;
  
  StrDB strdb(DB);
  Coords *p = new Coords(0, 0);
  char buf[KeyBuf];
  
  terminal->ClearScreen();
  terminal->Write(KeyName, p->XPos(), p->YPos());
  terminal->Write(" to print -> ");
  terminal->GetString(buf);

  RBNode<StrDB> *ptr = FindKeyMember(buf);
  if(!ptr) { 
    terminal->Write("Could not find: ", p->XPos(), p->YOffset(2));
    terminal->Write(buf);
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    delete p; // Free Coords pointer
    return;
  }

  strdb.Copy(ptr->Data);                   
  PrintObject(strdb, p);
  delete p; // Free Coords pointer
}

void PrintObject(StrDB &strdb, Coords *p)
{
  char buf[CBuf];
  char *FileName;
  const char *separator = " = ";
  const char *cseparator = ": ";
  
  terminal->Write("Enter file name to print to: ",
		  p->XPos(), p->YOffset(1));
  terminal->GetString(buf);

  char *tmp = new char[strlen(buf)+1];
  tmp[strlen(buf)] = '\0';
  strcpy(tmp, buf);
  
  if(VBDFile::Exists(tmp)) {
    terminal->Write("File: ",
		    p->XPos(), p->YOffset(1));
    terminal->Write(tmp);
    terminal->Write(" already exists.");
    int yn = terminal->YesNo("Overwrite it (y/n)?", p->XPos(), p->YOffset(1));
    if(!yn) {
      delete[] tmp;
      return;
    }
  }

  FileName = tmp;
  delete[] tmp;
  
  ofstream stream(FileName, ios::out); // Open file and truncate it
  terminal->Write("Printing to: ",
		  p->XPos(), p->YOffset(1));
  terminal->Write(FileName);
  
  if(!stream) { // Could not open the stream
    terminal->Write("Could not write to: ",
		    p->XPos(), p->YOffset(1));
    terminal->Write(FileName);
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    return;
  }

  if(KeyName != 0) {
    FPrint(KeyName, stream, strlen(KeyName));
    FPrint(separator, stream, strlen(separator));
    FPrint(strdb.GetKM(), stream, strlen(strdb.GetKM()));
    stream << LineFeed;
  }
  if(M2Name != 0) {
    FPrint(M2Name, stream, strlen(M2Name));
    FPrint(separator, stream, strlen(separator));
    FPrint(strdb.GetM2(), stream, strlen(strdb.GetM2()));
    stream << LineFeed;
  }
  if(M3Name != 0) {
    FPrint(M3Name, stream, strlen(M3Name));
    FPrint(separator, stream, strlen(separator));
    FPrint(strdb.GetM3(), stream, strlen(strdb.GetM3()));
    stream << LineFeed;
  }
  if(M4Name != 0) {
    FPrint(M4Name, stream, strlen(M4Name));
    FPrint(separator, stream, strlen(separator));
    FPrint(strdb.GetM4(), stream, strlen(strdb.GetM4()));
    stream << LineFeed;
  }
  if(M5Name != 0) {
    FPrint(M5Name, stream, strlen(M5Name));
    FPrint(separator, stream, strlen(separator));
    FPrint(strdb.GetM5(), stream, strlen(strdb.GetM5()));
    stream << LineFeed;
  }
  if(M6Name != 0) {
    FPrint(M6Name, stream, strlen(M6Name));
    FPrint(separator, stream, strlen(separator));
    FPrint(strdb.GetM6(), stream, strlen(strdb.GetM6()));
    stream << LineFeed;
  }
  if(M7Name != 0) {
    FPrint(M7Name, stream, strlen(M7Name));
    FPrint(separator, stream, strlen(separator));
    FPrint(strdb.GetM7(), stream, strlen(strdb.GetM7()));
    stream << LineFeed;
  }
  if(M8Name != 0) {
    FPrint(M8Name, stream, strlen(M8Name));
    FPrint(separator, stream, strlen(separator));
    FPrint(strdb.GetM8(), stream, strlen(strdb.GetM8()));
    stream << LineFeed;
  }
  if(Comments != 0) {
    FPrint(Comments, stream, strlen(Comments));
    FPrint(cseparator, stream, strlen(cseparator));
    FPrint(strdb.GetCM(), stream, strlen(strdb.GetCM()));
    stream << LineFeed;
  }
  terminal->Write("Finished", p->XPos(), p->YOffset(1));
  terminal->AnyKey(p->XPos(), p->YOffset(2));
  stream.close();
}

void Add()
{
  if(KeyName == 0) return;
  int r = retrys;
  
  Coords *p = new Coords(0, 0);
  char buf[KeyBuf];
  StrDB strdb(DB);
  terminal->ClearScreen();

  while(1) {
    terminal->Write(KeyName, p->XPos(), p->YPos());
    terminal->Write(" to add -> ");
    terminal->GetString(buf);
    if(!*buf) r--; else break;
    if(!r) {
      delete p;
      return;
    }
  }
  
  if(FindKeyMember(buf)) { 
    terminal->Write(buf, p->XPos(), p->YOffset(2));
    terminal->Write(" entry already exists."); 
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    delete p; // Free Coords pointer
    return;
  }

  strdb.SetKM(buf);
  AddObject(strdb, p);
  delete p; // Free Coords pointer
}

void AddObject(StrDB &strdb, Coords *p)
{
  char buf[KeyBuf];
  char cbuf[CBuf];

  if(M2Name != 0) {
  terminal->Write(M2Name, p->XPos(), p->YOffset(2));
  terminal->Write(" -> ");
  terminal->GetString(buf);
  strdb.SetM2(buf);
  }
  if(M3Name != 0) {
  terminal->Write(M3Name, p->XPos(), p->YOffset(1));
  terminal->Write(" -> ");
  terminal->GetString(buf);
  strdb.SetM3(buf);
  }
  if(M4Name != 0) {
  terminal->Write(M4Name, p->XPos(), p->YOffset(1));
  terminal->Write(" -> ");
  terminal->GetString(buf);
  strdb.SetM4(buf);
  }
  if(M5Name != 0) {
    terminal->Write(M5Name, p->XPos(), p->YOffset(1));
    terminal->Write(" -> ");
    terminal->GetString(buf);
    strdb.SetM5(buf);
  }
  if(M6Name != 0) {
    terminal->Write(M6Name, p->XPos(), p->YOffset(1));
    terminal->Write(" -> ");
    terminal->GetString(buf);
    strdb.SetM6(buf);
  }
  if(M7Name != 0) {
    terminal->Write(M7Name, p->XPos(), p->YOffset(1));
    terminal->Write(" -> ");
    terminal->GetString(buf);
    strdb.SetM7(buf);
  }
  if(M8Name != 0) {
    terminal->Write(M8Name, p->XPos(), p->YOffset(1));
    terminal->Write(" -> ");
    terminal->GetString(buf);
    strdb.SetM8(buf);
  }
  if(Comments != 0) {
    terminal->Write(Comments, p->XPos(), p->YOffset(1));
    terminal->Write(" -> ");
    terminal->GetString(cbuf);
    strdb.SetCM(cbuf);
  }
  terminal->Write("(A)-Abort (C)-Change, Any other key to accept entry",
		  p->XPos(), p->YOffset(2));
  int c = terminal->GetChar();
  switch(c) {
    case 'c': case 'C':
      terminal->ClearScreen();
      p->SetXY(0, 0);
      ChangeObject(strdb, p, 1); 
      break;
    case 'a': case 'A':
      return;
    default:
      break;
  }

  FAU addr = strdb.AddObject(0); // Write the object to the file
  if(!addr) {
    terminal->Write("Could not add item to database",
                    p->XPos(), p->YOffset(2));
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    return;
  }
  cache->Add(strdb); // Update the cache
  
  terminal->Write("Item was added to the database",
                  p->XPos(), p->YOffset(2));

  terminal->Write("Add another (y/n)", p->XPos(), p->YOffset(2));
  if(terminal->GetYesNo()) Add(); else return; 
}

void FindByKeyName()
{
  if(KeyName == 0) return;
  
  Coords *p = new Coords(0, 0);
  char buf[KeyBuf];
  unsigned offset = 0;
  
  terminal->ClearScreen();
  StrDB strdb(DB);

  terminal->Write("Enter complete string or use ", p->XPos(), p->YPos());
  terminal->Write(WildCard);
  terminal->Write(" for a wild card.");
  terminal->Write(KeyName, p->XPos(), p->YOffset(2));
  terminal->Write(" -> ");
  terminal->GetString(buf);
  UString str(buf);
  offset = str.Find(WildCard, offset); // Look for wild card character

  if(offset == UString::NoMatch) { // No wild cards found
    strdb.SetKM(buf);
    PlainSearch(strdb, KEYMEMBER, str, p);
    delete p; // Free the Coord pointer
    return;
  }

  WildCardSearch(strdb, KEYMEMBER, str, p);
  delete p; // Free the Coord pointer
}

void FindByM2Name()
{
  if(M2Name == 0) return;
  
  Coords *p = new Coords(0, 0);
  char buf[KeyBuf];
  unsigned offset = 0;
  
  terminal->ClearScreen();
  StrDB strdb(DB);

  terminal->Write("Enter complete string or use ", p->XPos(), p->YPos());
  terminal->Write(WildCard);
  terminal->Write(" for a wild card.");
  terminal->Write(M2Name, p->XPos(), p->YOffset(2));
  terminal->Write(" -> ");
  terminal->GetString(buf);
  UString str(buf);
  offset = str.Find(WildCard, offset); // Look for wild card character

  if(offset == UString::NoMatch) { // No wild cards found
    strdb.SetKM(buf);
    PlainSearch(strdb, M2NAME, str, p);
    delete p; // Free the Coord pointer
    return;
  }

  WildCardSearch(strdb, M2NAME, str, p);
  delete p; // Free the Coord pointer
}

void FindByM3Name()
{
  if(M3Name == 0) return;
  
  Coords *p = new Coords(0, 0);
  char buf[KeyBuf];
  unsigned offset = 0;
  
  terminal->ClearScreen();
  StrDB strdb(DB);

  terminal->Write("Enter complete string or use ", p->XPos(), p->YPos());
  terminal->Write(WildCard);
  terminal->Write(" for a wild card.");
  terminal->Write(M3Name, p->XPos(), p->YOffset(2));
  terminal->Write(" -> ");
  terminal->GetString(buf);
  UString str(buf);
  offset = str.Find(WildCard, offset); // Look for wild card character

  if(offset == UString::NoMatch) { // No wild cards found
    strdb.SetKM(buf);
    PlainSearch(strdb, M3NAME, str, p);
    delete p; // Free the Coord pointer
    return;
  }

  WildCardSearch(strdb, M3NAME, str, p);
  delete p; // Free the Coord pointer
}

void FindByM4Name()
{
  if(M4Name == 0) return;
  
  Coords *p = new Coords(0, 0);
  char buf[KeyBuf];
  unsigned offset = 0;
  
  terminal->ClearScreen();
  StrDB strdb(DB);

  terminal->Write("Enter complete string or use ", p->XPos(), p->YPos());
  terminal->Write(WildCard);
  terminal->Write(" for a wild card.");
  terminal->Write(M4Name, p->XPos(), p->YOffset(2));
  terminal->Write(" -> ");
  terminal->GetString(buf);
  UString str(buf);
  offset = str.Find(WildCard, offset); // Look for wild card character

  if(offset == UString::NoMatch) { // No wild cards found
    strdb.SetKM(buf);
    PlainSearch(strdb, M4NAME, str, p);
    delete p; // Free the Coord pointer
    return;
  }

  WildCardSearch(strdb, M4NAME, str, p);
  delete p; // Free the Coord pointer
}

void FindByM5Name()
{
  if(M5Name == 0) return;
  
  Coords *p = new Coords(0, 0);
  char buf[KeyBuf];
  unsigned offset = 0;
  
  terminal->ClearScreen();
  StrDB strdb(DB);

  terminal->Write("Enter complete string or use ", p->XPos(), p->YPos());
  terminal->Write(WildCard);
  terminal->Write(" for a wild card.");
  terminal->Write(M5Name, p->XPos(), p->YOffset(2));
  terminal->Write(" -> ");
  terminal->GetString(buf);
  UString str(buf);
  offset = str.Find(WildCard, offset); // Look for wild card character

  if(offset == UString::NoMatch) { // No wild cards found
    strdb.SetKM(buf);
    PlainSearch(strdb, M5NAME, str, p);
    delete p; // Free the Coord pointer
    return;
  }

  WildCardSearch(strdb, M5NAME, str, p);
  delete p; // Free the Coord pointer
}

void FindByM6Name()
{
  if(M6Name == 0) return;
  
  Coords *p = new Coords(0, 0);
  char buf[KeyBuf];
  unsigned offset = 0;
  
  terminal->ClearScreen();
  StrDB strdb(DB);

  terminal->Write("Enter complete string or use ", p->XPos(), p->YPos());
  terminal->Write(WildCard);
  terminal->Write(" for a wild card.");
  terminal->Write(M6Name, p->XPos(), p->YOffset(2));
  terminal->Write(" -> ");
  terminal->GetString(buf);
  UString str(buf);
  offset = str.Find(WildCard, offset); // Look for wild card character

  if(offset == UString::NoMatch) { // No wild cards found
    strdb.SetKM(buf);
    PlainSearch(strdb, M6NAME, str, p);
    delete p; // Free the Coord pointer
    return;
  }

  WildCardSearch(strdb, M6NAME, str, p);
  delete p; // Free the Coord pointer
}

void FindByM7Name()
{
  if(M7Name == 0) return;
  
  Coords *p = new Coords(0, 0);
  char buf[KeyBuf];
  unsigned offset = 0;
  
  terminal->ClearScreen();
  StrDB strdb(DB);

  terminal->Write("Enter complete string or use ", p->XPos(), p->YPos());
  terminal->Write(WildCard);
  terminal->Write(" for a wild card.");
  terminal->Write(M7Name, p->XPos(), p->YOffset(2));
  terminal->Write(" -> ");
  terminal->GetString(buf);
  UString str(buf);
  offset = str.Find(WildCard, offset); // Look for wild card character

  if(offset == UString::NoMatch) { // No wild cards found
    strdb.SetKM(buf);
    PlainSearch(strdb, M7NAME, str, p);
    delete p; // Free the Coord pointer
    return;
  }

  WildCardSearch(strdb, M7NAME, str, p);
  delete p; // Free the Coord pointer
}

void FindByM8Name()
{
  if(M8Name == 0) return;
  
  Coords *p = new Coords(0, 0);
  char buf[KeyBuf];
  unsigned offset = 0;
  
  terminal->ClearScreen();
  StrDB strdb(DB);

  terminal->Write("Enter complete string or use ", p->XPos(), p->YPos());
  terminal->Write(WildCard);
  terminal->Write(" for a wild card.");
  terminal->Write(M8Name, p->XPos(), p->YOffset(2));
  terminal->Write(" -> ");
  terminal->GetString(buf);
  UString str(buf);
  offset = str.Find(WildCard, offset); // Look for wild card character

  if(offset == UString::NoMatch) { // No wild cards found
    strdb.SetKM(buf);
    PlainSearch(strdb, M8NAME, str, p);
    delete p; // Free the Coord pointer
    return;
  }

  WildCardSearch(strdb, M8NAME, str, p);
  delete p; // Free the Coord pointer
}

void FindByComments()
{
  if(Comments == 0) return;
  
  Coords *p = new Coords(0, 0);
  char buf[KeyBuf];
  unsigned offset = 0;
  
  terminal->ClearScreen();
  StrDB strdb(DB);

  terminal->Write("Enter complete string or use ", p->XPos(), p->YPos());
  terminal->Write(WildCard);
  terminal->Write(" for a wild card.");
  terminal->Write(Comments, p->XPos(), p->YOffset(2));
  terminal->Write(" -> ");
  terminal->GetString(buf);
  UString str(buf);
  offset = str.Find(WildCard, offset); // Look for wild card character

  if(offset == UString::NoMatch) { // No wild cards found
    strdb.SetKM(buf);
    PlainSearch(strdb, COMMENTS, str, p);
    delete p; // Free the Coord pointer
    return;
  }

  WildCardSearch(strdb, COMMENTS, str, p);
  delete p; // Free the Coord pointer
}

void WildCardSearch(StrDB &strdb, StrDBItem item, UString &str, Coords *p)
{
  int offset, count, c, yn;
  offset = count = c = 0;
  BNodeBase *nxt;         // BSTree base node pointer
  SLList<StrDB> list;     // Linked list for objects found
  SNode<StrDB> *listptr;  // Linked list node pointer
  
  while(1) { // Remove the wild card characters from the string
    offset = str.Find(WildCard, offset);
    if (offset == UString::NoMatch) break;
    str.DeleteAt(offset, strlen(WildCard));
    offset++;
  }

  // Using iterator object to walk throught the tree
  TreeWalkerb tw(cache->GetRoot(), INORDER);
  UString buf;
  
  while((nxt = tw.Next()) != 0) {
    RBNode<StrDB> *ptr = (RBNode<StrDB> *)nxt;
    strdb.Copy(ptr->Data);
    switch(item) {
      case KEYMEMBER:
        buf = strdb.GetKM();
	break;
      case M2NAME:
        buf = strdb.GetM2();
	break;
      case M3NAME:
        buf = strdb.GetM3();
	break;
      case M4NAME:
        buf = strdb.GetM4();
	break;
      case M5NAME:
        buf = strdb.GetM5();
	break;
      case M6NAME:
        buf = strdb.GetM6();
	break;
      case M7NAME:
        buf = strdb.GetM7();
	break;
      case M8NAME:
        buf = strdb.GetM8();
	break;
      case COMMENTS:
        buf = strdb.GetCM();
	break;
      default:
	return;
    }

    offset = buf.Find(str.c_str(), 0);
    if(offset == UString::NoMatch)
      continue;
    else { // Found match 
      list.StoreNode(strdb);
      count++;
    }
  }

  if(count == 0) {
    terminal->Write("No matches found.", p->XPos(), p->YOffset(2));
    terminal->AnyKey(p->XPos(), p->YOffset(1));
    return;
  }

  terminal->Write("Found ", p->XPos(), p->YOffset(2));
  terminal->Write(count);
  terminal->Write(" matching.");
  terminal->AnyKey(p->XPos(), p->YOffset(1));
  
  // Display all the matches
  listptr = list.GetFront();
  while(!list.IsHeader(listptr)) {
    terminal->ClearScreen();
    p->SetXY(0, 0);
    DisplayObject(listptr->Data, p);
    terminal->Write("(C)-Change, (D)-Delete, (P)-Print, (X)-Exit",
		    p->XPos(), p->YOffset(2));
    terminal->Write(", Any other key to continue");
    c = terminal->GetChar();
    switch(c) {
      case 'c': case 'C': 
	terminal->ClearScreen();
	p->SetXY(0, 0);
        strdb.Copy(listptr->Data);
        ChangeObject(strdb, p);
	break;
	
      case 'd': case 'D':
	yn = terminal->YesNo("Are you sure you want to delete (y/n)",
			     p->XPos(), p->YOffset(2));
	if(!yn) break;
        strdb.Copy(listptr->Data);
        cache->Delete(strdb); // Update the memory cache
        strdb.DeleteObject(strdb.ObjectAddress());
	break;
	
      case 'p': case 'P':
        strdb.Copy(listptr->Data);
	p->YOffset(1);
        PrintObject(strdb, p);
	break;

      case 'x': case 'X':
	list.Clear(); // Free list memory
	return;

      default:
	break;
    }
    listptr = listptr->GetNext();
  }
  list.Clear(); // Free list memory
}

void PlainSearch(StrDB &strdb, StrDBItem item, UString &str, Coords *p)
// Return true if match found
{
  int c, result, yn;
  int exists = 0;
  BNodeBase *nxt;         // BSTree base node pointer
  UString buf;
  
  // Using iterator object to walk throught the tree
  TreeWalkerb tw(cache->GetRoot(), INORDER);
  RBNode<StrDB> *ptr;
  
  while((nxt = tw.Next()) != 0) {
    ptr = (RBNode<StrDB> *)nxt;
    strdb.Copy(ptr->Data);
    switch(item) {
      case KEYMEMBER:
        buf = strdb.GetKM();
	break;
      case M2NAME:
        buf = strdb.GetM2();
	break;
      case M3NAME:
        buf = strdb.GetM3();
	break;
      case M4NAME:
        buf = strdb.GetM4();
	break;
      case M5NAME:
        buf = strdb.GetM5();
	break;
      case M6NAME:
        buf = strdb.GetM6();
	break;
      case M7NAME:
        buf = strdb.GetM7();
	break;
      case M8NAME:
        buf = strdb.GetM8();
	break;
      case COMMENTS:
        buf = strdb.GetCM();
	break;
      default:
	return;
    }
    result = NoCaseCmp(buf, str);
    if(result == 0) { // Found unique data member
      exists = 1;
      break;
    }
  }

  if(!exists || !*buf.c_str()) {
    terminal->Write("Could not find entry for: ", p->XPos(), p->YOffset(2));
    terminal->Write(str.c_str());
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    return;
  }

  terminal->Write("Found entry for: ", p->XPos(), p->YOffset(2));
  terminal->Write(str.c_str());
  terminal->AnyKey(p->XPos(), p->YOffset(1));
  
  p->YOffset(2);
  DisplayObject(strdb, p);
  terminal->Write("(C)-Change, (D)-Delete, (P)-Print",
		  p->XPos(), p->YOffset(2));
  terminal->Write(", Any other key to continue");
  c = terminal->GetChar();
  switch(c) {
    case 'c': case 'C':
      terminal->ClearScreen();
      p->SetXY(0, 0);
      ChangeObject(strdb, p);
      break;
      
    case 'd': case 'D':
      yn = terminal->YesNo("Are you sure you want to delete (y/n)",
			     p->XPos(), p->YOffset(2));
      if(!yn) break;
      cache->Delete(strdb); // Update the memory cache
      strdb.DeleteObject(strdb.ObjectAddress());
      break;
      
      case 'p': case 'P':
        strdb.Copy(ptr->Data);
	p->YOffset(1);
        PrintObject(strdb, p);
	break;
	
    default:
      break;
  }
}

void Change()
{
  if(KeyName == 0) return;
  
  StrDB strdb(DB);
  Coords *p = new Coords(0, 0);
  char buf[KeyBuf];
  
  terminal->ClearScreen();
  terminal->Write(KeyName, p->XPos(), p->YPos());
  terminal->Write(" to change -> ");
  terminal->GetString(buf);

  RBNode<StrDB> *ptr = FindKeyMember(buf);
  if(!ptr) { 
    terminal->Write("Could not find: ", p->XPos(), p->YOffset(2));
    terminal->Write(buf);
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    delete p; // Free Coords pointer
    return;
  }

  strdb.Copy(ptr->Data);
  ChangeObject(strdb, p);
  delete p; // Free Coords pointer
}

void ChangeObject(StrDB &strdb, Coords *p, int adding)
{
  char buf[KeyBuf];
  char cbuf[CBuf];
  
  // Buffer the object's data to allow changes to be canceled
  StrDB ob;
  ob.Copy(strdb);

  StrDB changed(DB); // Temp buffer used to record changes
  
  char *prompt = "Press the number/letter of your selection -> ";  
  int c, x, y, x_prev, y_prev, x_prompt, y_prompt;
  x_prev = p->XPos(); y_prev = p->YPos();

  while(1) {
    p->SetXY(x_prev, y_prev);
    terminal->Write("(C) Cancel change", p->XPos(), p->YOffset(2));
    if(adding) {
      if(KeyName != 0) {
	terminal->Write("(0) ", p->XPos(), p->YOffset(1));
	terminal->Write(KeyName); terminal->Write(" = ");
	terminal->Write(strdb.GetKM());
	terminal->ClearLine();
      }
    }
    else {
      if(KeyName != 0) {
	terminal->Write("(-) ", p->XPos(), p->YOffset(1));
	terminal->Write(KeyName); terminal->Write(" = ");
	terminal->Write(strdb.GetKM());
	terminal->ClearLine();
      }
    }
    if(M2Name != 0) {
      terminal->Write("(1) ", p->XPos(), p->YOffset(1));
      terminal->Write(M2Name); terminal->Write(" = ");
      terminal->Write(strdb.GetM2());
      terminal->ClearLine();
    }
    if(M3Name != 0) {
      terminal->Write("(2) ", p->XPos(), p->YOffset(1));
      terminal->Write(M3Name); terminal->Write(" = ");
      terminal->Write(strdb.GetM3());
      terminal->ClearLine();
    }
    if(M4Name != 0) {
      terminal->Write("(3) ", p->XPos(), p->YOffset(1));
      terminal->Write(M4Name); terminal->Write(" = ");
      terminal->Write(strdb.GetM4());
      terminal->ClearLine();
    }
    if(M5Name != 0) {
      terminal->Write("(4) ", p->XPos(), p->YOffset(1));
      terminal->Write(M5Name); terminal->Write(" = ");
      terminal->Write(strdb.GetM5());
      terminal->ClearLine();
    }
    if(M6Name != 0) {
      terminal->Write("(5) ", p->XPos(), p->YOffset(1));
      terminal->Write(M6Name); terminal->Write(" = ");
      terminal->Write(strdb.GetM6());
      terminal->ClearLine();
    }
    if(M7Name != 0) {
      terminal->Write("(6) ", p->XPos(), p->YOffset(1));
      terminal->Write(M7Name); terminal->Write(" = ");
      terminal->Write(strdb.GetM7());
      terminal->ClearLine();
    }
    if(M8Name != 0) {
      terminal->Write("(7) ", p->XPos(), p->YOffset(1));
      terminal->Write(M8Name); terminal->Write(" = ");
      terminal->Write(strdb.GetM8());
      terminal->ClearLine();
    }
    if(Comments != 0) {
      terminal->Write("(8) ", p->XPos(), p->YOffset(1));
      terminal->Write(Comments); terminal->Write(" = ");
      terminal->Write(strdb.GetCM());
      terminal->ClearLine();
    }
    terminal->Write("(A) Accept changes", p->XPos(), p->YOffset(1));
    
    terminal->Write(prompt, p->XPos(), p->YOffset(2));
    x_prompt = p->XPos() + strlen(prompt);
    y_prompt = p->YPos();
  
    x = p->XPos();
    y = p->YPos() + 2;

    c = terminal->GetChar();
    switch(c) {
      case 'C': case 'c': 
        strdb.Copy(ob);
	return;

      case '0':
	// User not allowed to change name unless adding a new object
	if(!adding) break; 
	if(KeyName != 0) { 
	  terminal->Write(KeyName, x, y);
	  terminal->Write(" -> ");
	  terminal->GetString(buf);
	  if(FindKeyMember(buf)) {
	    terminal->Write("Entry already exists!", x, y+1);
	    terminal->AnyKey();
	    terminal->ClearLine(x, y+1);
	    terminal->ClearLine(x, y);
	    terminal->MoveCursor(x_prev, y_prev);
	    break;
	  }
	  strdb.SetKM(buf);
	  terminal->ClearLine(x, y);
	  terminal->MoveCursor(x_prev, y_prev);
	}
	break;

      case '1':
	if(M2Name != 0) { 
	  terminal->Write(M2Name, x, y);
	  terminal->Write(" -> ");
	  terminal->GetString(buf);
	  strdb.SetM2(buf);
	  terminal->ClearLine(x, y);
	  terminal->MoveCursor(x_prev, y_prev);
	}
	break;

      case '2':
	if(M3Name != 0) { 
	  terminal->Write(M3Name, x, y);
	  terminal->Write(" -> ");
	  terminal->GetString(buf);
	  strdb.SetM3(buf);
	  terminal->ClearLine(x, y);
	  terminal->MoveCursor(x_prev, y_prev);
	}
	break;
	
      case '3':
	if(M4Name != 0) { 
	  terminal->Write(M4Name, x, y);
	  terminal->Write(" -> ");
	  terminal->GetString(buf);
	  strdb.SetM4(buf);
	  terminal->ClearLine(x, y);
	  terminal->MoveCursor(x_prev, y_prev);
	}
	break;

      case '4':
	if(M5Name != 0) { 
	  terminal->Write(M5Name, x, y);
	  terminal->Write(" -> ");
	  terminal->GetString(buf);
	  strdb.SetM5(buf);
	  terminal->ClearLine(x, y);
	  terminal->MoveCursor(x_prev, y_prev);
	}
	break;

      case '5':
	if(M6Name != 0) { 
	  terminal->Write(M6Name, x, y);
	  terminal->Write(" -> ");
	  terminal->GetString(buf);
	  strdb.SetM6(buf);
	  terminal->ClearLine(x, y);
	  terminal->MoveCursor(x_prev, y_prev);
	}
	break;

      case '6':
	if(M7Name != 0) { 
	  terminal->Write(M7Name, x, y);
	  terminal->Write(" -> ");
	  terminal->GetString(buf);
	  strdb.SetM7(buf);
	  terminal->ClearLine(x, y);
	  terminal->MoveCursor(x_prev, y_prev);
	}
	break;

      case '7':
	if(M8Name != 0) { 
	  terminal->Write(M8Name, x, y);
	  terminal->Write(" -> ");
	  terminal->GetString(buf);
	  strdb.SetM8(buf);
	  terminal->ClearLine(x, y);
	  terminal->MoveCursor(x_prev, y_prev);
	}
	break;

      case '8':
	if(Comments != 0) { 
	  terminal->Write(Comments, x, y);
	  terminal->Write(" -> ");
	  terminal->GetString(cbuf);
	  strdb.SetCM(cbuf);
	  terminal->ClearLine(x, y);
	  terminal->MoveCursor(x_prev, y_prev);
	}
	break;
	
      case 'A': case 'a':
	if(adding) return; // New object, no need to reallocate space
        if(strdb.FullCompare(ob)) return; // The object has not changed
        changed.Copy(strdb);  // Make a copy of the changes
        cache->Delete(strdb); // Update the memory cache
	// Remove the original object 
        strdb.DeleteObject(strdb.ObjectAddress()); 
	changed.AddObject(0); // Add the changed object back to the database
	cache->Add(changed);  // Update the memory cache   
	return;
	
      default:
	break;
    }
  }
}

void DisplayStats(const VBDFilePtr &f)
// Passing VBDFilePtr by reference to avoid calling the Refcount
// copy constructor. This will avoid any unnecessary aliasing.
{
  terminal->ClearScreen();
  int tab = 5;
  const char *FileName = f->VBDFileName();
  SBYTE FileStatus = f->GetFileStatus(); // Get the current file status
  char *s1 = "----- (V)ariable (B)lock (D)atabase file statistics -----";
  terminal->Write(s1, terminal->Center(s1), 0);
  terminal->Write("----- Static information -----", 0, 2);
  terminal->Write("VBD File Name:          ", tab, 3);
  terminal->Write(FileName);
  terminal->Write("VBDFile Signature:      ", tab, 4);
  terminal->Write(f->GetSignature());
  terminal->Write("VBDFile Version Number: ", tab, 5);
  terminal->Write(f->GetVersion());
  terminal->Write("File Header Size :      ", tab, 6);
  terminal->Write((int)f->FileHeaderSize());
  terminal->Write("VB Header Size:         ", tab, 7);
  terminal->Write((int)f->VBHeaderSize());
  terminal->Write("Static area size:       ", tab, 8);
  terminal->Write(f->StaticArea());
  terminal->Write("Static file size:       ", tab, 9);
  terminal->Write(f->FileSize(FileName));
  terminal->Write("----- Dynamic Information -----", 0, 11);
  terminal->Write("File status Byte:       ", tab, 12);
  if(FileStatus & 0x01)
    terminal->Write("Good ");
  else
    terminal->Write("Bad ");
  if(FileStatus & 0x02)
    terminal->Write("Open ");
  else
    terminal->Write("Closed ");
  if(FileStatus & 0x04)
    terminal->Write("Read/Write ");
  else
    terminal->Write("Read/Only ");
  terminal->Write("VBDFile Free Space:     ", tab, 13);
  terminal->Write(f->GetFreeSpace());
  terminal->Write("VBDFile End of File:    ", tab, 14);
  terminal->Write(f->GetEOF());
  terminal->Write("VBDFile Heap Start:     ", tab, 15);
  terminal->Write(f->GetHeapStart());
  terminal->Write("VBDFile Highest VB:     ", tab, 16);
  terminal->Write(f->GetHighestVB());
  terminal->Write("Total VBs in file:      ", tab, 17);
  terminal->Write((int)f->VBTotal());
  terminal->Write("Total deleted VBs:      ", tab, 18);
  terminal->Write((int)f->VBDeleted());
  terminal->AnyKey(0, 20);
  terminal->ClearScreen();
}

void DisplayObject(StrDB &strdb, Coords *p)
// Display's the complete object with all details.
{
  if(KeyName != 0) {
    terminal->Write(KeyName, p->XPos(), p->YPos());
    terminal->Write(" = ");
    terminal->Write(strdb.GetKM());
  }
  if(M2Name != 0) {
    terminal->Write(M2Name, p->XPos(), p->YOffset(1));
    terminal->Write(" = ");
    terminal->Write(strdb.GetM2());
  }
  if(M3Name != 0) {
    terminal->Write(M3Name, p->XPos(), p->YOffset(1));
    terminal->Write(" = ");
    terminal->Write(strdb.GetM3());
  }
  if(M4Name != 0) {
    terminal->Write(M4Name, p->XPos(), p->YOffset(1));
    terminal->Write(" = ");
    terminal->Write(strdb.GetM4());
  }
  if(M5Name != 0) { 
    terminal->Write(M5Name, p->XPos(), p->YOffset(1));
    terminal->Write(" = ");
    terminal->Write(strdb.GetM5());
  }
  if(M6Name != 0) {
    terminal->Write(M6Name, p->XPos(), p->YOffset(1));
    terminal->Write(" = ");
    terminal->Write(strdb.GetM6());
  }
  if(M7Name != 0) {
    terminal->Write(M7Name, p->XPos(), p->YOffset(1));
    terminal->Write(" = ");
    terminal->Write(strdb.GetM7());
  }
  if(M8Name != 0) {
    terminal->Write(M8Name, p->XPos(), p->YOffset(1));
    terminal->Write(" = ");
    terminal->Write(strdb.GetM8());
  }
  if(Comments != 0) {
    terminal->Write(Comments, p->XPos(), p->YOffset(1));
    terminal->Write(": ");
    terminal->Write(strdb.GetCM());
  }
}

void DisplayString(const char *s)
{
  int len = strlen(s);
  int offset = StringOffset;
  char *buf = new char[len + 1];
  buf[len] = '\0';
  strcpy(buf, s);
  if(len < StringOffset) offset = len;
  for(int i = 0; i < offset; i++)
    terminal->Write(buf[i]);
  delete[] buf;
}

void LineByLine(StrDB &strdb, Coords *p)
// Display objects line by line with partial details
{
  int x = p->XPos();

  if(KeyName != 0) {
    terminal->MoveCursor(p->XPos(), p->YPos());
    DisplayString(strdb.GetKM());
  }

  if(M2Name != 0) {
    terminal->MoveCursor(p->XOffset(StringOffset+1), p->YPos()); 
    DisplayString(strdb.GetM2());
  }
  if(M3Name != 0) {
    terminal->MoveCursor(p->XOffset(StringOffset+1), p->YPos());
    DisplayString(strdb.GetM3());
  }
  if(M4Name != 0) {
    terminal->MoveCursor(p->XOffset(StringOffset+1), p->YPos());
    DisplayString(strdb.GetM4());
  }
  if(M5Name != 0) {
    terminal->MoveCursor(p->XOffset(StringOffset+1), p->YPos());
    DisplayString(strdb.GetM5());
  }

  int i = 0;
  terminal->MoveCursor(0, p->YOffset(1));
  while (i++ < terminal->MaxCols()-1) terminal->Write('-');
  p->SetX(x);
}

void LoadCache()
{
  FAU addr;
  StrDB strdb(DB);
  cache->Clear(); // Empty the existing memory buffers
  terminal->ClearScreen();
  const char *mesg = "Loading the database into memory...";
  int x = terminal->Center(mesg);
  terminal->Write(mesg, x, terminal->ScreenCenter(1));

  while(1)
    {
      addr = DB->OpenDatabase()->FindFirstObject(addr);
      if(!addr) break;
      strdb.ReadObject(addr);
      cache->Add(strdb);
    }
}

void Version()
{
  printf("\n%s program version %g\n", ProgramName, VersionNumber);
  exit(0);
}

void ExportToASCII()
{
  char buf[CBuf];
  char *FileName;
  const char dchar = '\t';   // Text delimiter
  const char *Fill = "None"; // Fill character string

  terminal->ClearScreen();
  Coords *p = new Coords(0, 0);

  terminal->Write("Export database to ASCII file delimited by tabs",
		  p->XPos(), p->YPos());

  terminal->Write("Enter file name to export to: ",
		  p->XPos(), p->YOffset(2));
  terminal->GetString(buf);

  char *tmp = new char[strlen(buf)+1];
  tmp[strlen(buf)] = '\0';
  strcpy(tmp, buf);
  
  if(VBDFile::Exists(tmp)) {
    terminal->Write("File: ",
		    p->XPos(), p->YOffset(2));
    terminal->Write(tmp);
    terminal->Write(" already exists.");
    int yn = terminal->YesNo("Overwrite it (y/n)?", p->XPos(), p->YOffset(2));
    if(!yn) {
      delete p;
      delete[] tmp;
      return;
    }
  }

  FileName = tmp;
  delete[] tmp;
  ofstream stream(FileName, ios::out); // Open file and truncate it
  terminal->Write("Exporting database to: ",
		  p->XPos(), p->YOffset(2));
  terminal->Write(FileName);
  
  if(!stream) { // Could not open the stream
    terminal->Write("Could not write to: ",
		    p->XPos(), p->YOffset(2));
    terminal->Write(FileName);
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    delete p; // Free the Coords pointer
    return; 
  }

  StrDB strdb(DB);
  BNodeBase *nxt; // BSTree base node pointer

  // Using iterator object to walk throught the tree
  TreeWalkerb tw(cache->GetRoot(), INORDER);

  // Write the item bar to the file first
  if(KeyName != 0) {
    FPrint(KeyName, stream, strlen(KeyName));
    stream << dchar;
  }
  if(M2Name != 0) {
    FPrint(M2Name, stream, strlen(M2Name));
    stream << dchar;
  }
  if(M3Name != 0) {
    FPrint(M3Name, stream, strlen(M3Name));
    stream << dchar;
  }
  if(M4Name != 0) {
    FPrint(M4Name, stream, strlen(M4Name));
    stream << dchar;
  }
  if(M5Name != 0) {
    FPrint(M5Name, stream, strlen(M5Name));
    stream << dchar;
  }
  if(M6Name != 0) {
    FPrint(M6Name, stream, strlen(M6Name));
    stream << dchar;
  }
  if(M7Name != 0) {
    FPrint(M7Name, stream, strlen(M7Name));
    stream << dchar;
  }
  if(M8Name != 0) {
    FPrint(M8Name, stream, strlen(M8Name));
    stream << dchar;
  }
  if(Comments != 0) {
    FPrint(Comments, stream, strlen(Comments));
  }
  stream << LineFeed;
  
  int exports = 0;
  while((nxt = tw.Next()) != 0) {
    RBNode<StrDB> *ptr = (RBNode<StrDB> *)nxt;
    strdb.Copy(ptr->Data);
    if(KeyName != 0) {
      FPrint(strdb.GetKM(), stream, strlen(strdb.GetKM()));
      stream << dchar;
    }
    if(M2Name != 0) {
      if(!*strdb.GetM2()) FPrint(Fill, stream, strlen(Fill));
      else FPrint(strdb.GetM2(), stream, strlen(strdb.GetM2()));
      stream << dchar;
    }
    if(M3Name != 0) {
      if(!*strdb.GetM3()) FPrint(Fill, stream, strlen(Fill));
      else FPrint(strdb.GetM3(), stream, strlen(strdb.GetM3()));
      stream << dchar;
    }
    if(M4Name != 0) {
      if(!*strdb.GetM4()) FPrint(Fill, stream, strlen(Fill));
      else FPrint(strdb.GetM4(), stream, strlen(strdb.GetM4()));
      stream << dchar;
    }
    if(M5Name != 0) {
      if(!*strdb.GetM5()) FPrint(Fill, stream, strlen(Fill));
      else FPrint(strdb.GetM5(), stream, strlen(strdb.GetM5()));
      stream << dchar;
    }
    if(M6Name != 0) {
      if(!*strdb.GetM6()) FPrint(Fill, stream, strlen(Fill));
      else FPrint(strdb.GetM6(), stream, strlen(strdb.GetM6()));
      stream << dchar;
    }
    if(M7Name != 0) {
      if(!*strdb.GetM7()) FPrint(Fill, stream, strlen(Fill));
      else FPrint(strdb.GetM7(), stream, strlen(strdb.GetM7()));
      stream << dchar;
    }
    if(M8Name != 0) {
      if(!*strdb.GetM8()) FPrint(Fill, stream, strlen(Fill));
      else FPrint(strdb.GetM8(), stream, strlen(strdb.GetM8()));
      stream << dchar;
    }
    if(Comments != 0) {
      if(!*strdb.GetCM()) FPrint(Fill, stream, strlen(Fill));
      else FPrint(strdb.GetCM(), stream, strlen(strdb.GetCM()));
    }
    stream << LineFeed;
    exports++;
  }

  terminal->Write("Exported ", p->XPos(), p->YOffset(2));
  terminal->Write(exports);
  terminal->Write(" items.");
  terminal->AnyKey(p->XPos(), p->YOffset(2));
  delete p;          // Free the Coords pointer
  stream.close();
}

void ImportFromASCII()
{
  if(KeyName == 0) return;
  
  char buf[CBuf];
  char words[MAXWORDS][MAXWORDLENGTH];
  int num;
  const int MaxLine = 512;
  char LineBuffer[MaxLine];
  char *sbuf;
  char *FileName;
  const char dchar = '\t';  // Text delimiter

  terminal->ClearScreen();
  Coords *p = new Coords(0, 0);

  terminal->Write("Importing database from ASCII file delimited by tabs",
		  p->XPos(), p->YPos());

  terminal->Write("Enter file name to import from: ",
		  p->XPos(), p->YOffset(2));
  terminal->GetString(buf);
  char *tmp = new char[strlen(buf)+1];
  tmp[strlen(buf)] = '\0';
  strcpy(tmp, buf);
  
  if(!VBDFile::Exists(tmp)) {
    terminal->Write("File: ",
		    p->XPos(), p->YOffset(2));
    terminal->Write(tmp);
    terminal->Write(" does not exists.");
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    delete p;
    delete[] tmp;
    return;
  }

  FileName = tmp;
  ifstream stream(FileName, ios::in|ios::nocreate);
  terminal->Write("Importing database from: ",
		  p->XPos(), p->YOffset(2));
  terminal->Write(FileName);
  
  if(!stream) { // Could not open the stream
    terminal->Write("Could not open file: ",
		    p->XPos(), p->YOffset(2));
    terminal->Write(FileName);
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    delete[] tmp;
    delete p; // Free the Coords pointer
    return; 
  }

  terminal->Write("Importing...", p->XPos(), p->YOffset(2));

  ChSNode *listptr; // Linked list node pointer
  ChSList list;     // Linked list
  
  while(!stream.eof())
  {
    // Read in file line by line
    stream.getline(LineBuffer, MaxLine);

    // Allocate memory for temporary holding buffer
    sbuf = new char[strlen(LineBuffer)+1];
    
    // Copy contents of the array to temporary holding buffer
    strcpy(sbuf, LineBuffer);

    // Load file data into singly-linked list
    listptr = list.Store(sbuf); 

    if(!listptr) { // Out of memory
      stream.close();
      delete[] sbuf;
      list.Clear();
      return;
   }
  }
  stream.close();
  delete[] sbuf;
  
  StrDB strdb(DB);
  StrDB ob(DB);
  
  int linecount = 0;
  int imports = 0;
  int updates = 0;
  int updateall = 0;
  int ObjectItems = 1; // Number of class data members 
  int x_prev, y_prev, c;

  // Calculate the number of object items
  if(M2Name != 0) ObjectItems++;
  if(M3Name != 0) ObjectItems++;
  if(M4Name != 0) ObjectItems++;
  if(M5Name != 0) ObjectItems++;
  if(M6Name != 0) ObjectItems++;
  if(M7Name != 0) ObjectItems++;
  if(M8Name != 0) ObjectItems++;
  if(Comments != 0) ObjectItems++;

  listptr = list.GetFront();
  while(!list.IsHeader(listptr)) {
    if(parse(listptr->Data, words, &num, dchar) == 1) {
      terminal->Write("Parse error!", p->XPos(), p->YOffset(2));
      break;
    }

    linecount++;
    if(num != ObjectItems && (num != 0 && num != 1)) {
      x_prev = p->XPos(); y_prev = p->YPos();
      terminal->Write("Error in ", p->XPos(), p->YOffset(2));
      terminal->Write(FileName);
      terminal->Write(" import file!");
      if(num > ObjectItems) 
	terminal->Write("To many items on line: ", p->XPos(), p->YOffset(2));
      if(num < ObjectItems) 
	terminal->Write("Not enough items on line: ",
			p->XPos(), p->YOffset(2));
      terminal->Write(linecount);
      terminal->Write("(X)-Exit, Any other key to continue",
		    p->XPos(), p->YOffset(2));
      c = terminal->GetChar();
      switch(c) {
	case 'x': case 'X':
	  delete p;          // Free the Coords pointer
	  delete[] tmp;
	  list.Clear();
	  return;
	default:
	  break;
      }
      terminal->ClearLine(p->XPos(), p->YPos());
      terminal->ClearLine(p->XPos(), p->YPos()-4);
      terminal->ClearLine(p->XPos(), p->YPos()-2);
      p->SetXY(x_prev, y_prev);
    }

    if(num == ObjectItems && *words[0] != 0) {
      strdb.SetKM(words[0]);
      if(M2Name != 0) strdb.SetM2(words[1]);
      if(M3Name != 0) strdb.SetM3(words[2]);
      if(M4Name != 0) strdb.SetM4(words[3]);
      if(M5Name != 0) strdb.SetM5(words[4]);
      if(M6Name != 0) strdb.SetM6(words[5]);
      if(M7Name != 0) strdb.SetM7(words[6]);
      if(M8Name != 0) strdb.SetM8(words[7]);
      if(Comments != 0) strdb.SetCM(words[8]);

      RBNode<StrDB> *cacheptr = FindKeyMember(strdb.GetKM());  
      if(cacheptr) {
	x_prev = p->XPos(); y_prev = p->YPos();
	if(updateall == 0) {
	  terminal->Write(strdb.GetKM(), p->XPos(), p->YOffset(2));
	  terminal->Write(" entry already exists."); 
	  terminal->Write("(U)-Update, (A)-Update all, (X)-Exit",
			  p->XPos(), p->YOffset(2));
	  terminal->Write(", Any other key to continue");
	  c = terminal->GetChar();
	}
	if(updateall == 1) c = 'a';
	switch(c) {
	  case 'a': case 'A':
	    updateall = 1;
	    updates++;
	    ob.Copy(cacheptr->Data);
            if(strdb.FullCompare(ob)) break; // The object has not changed
	    cache->Delete(ob);
	    ob.DeleteObject(ob.ObjectAddress()); 
            strdb.AddObject(0);
            cache->Add(strdb);  
	    break;
	  case 'u': case 'U': 
	    updates++;
	    ob.Copy(cacheptr->Data);
            if(strdb.FullCompare(ob)) break; // The object has not changed
	    cache->Delete(ob);
	    ob.DeleteObject(ob.ObjectAddress()); 
            strdb.AddObject(0);
            cache->Add(strdb);  
	    break;
	  case 'x': case 'X':
	    delete p;          // Free the Coords pointer
	    delete[] tmp;
	    list.Clear();
	    return;
	  default:
	    break;
	}
	if(updateall == 1) {
	  p->SetXY(x_prev, y_prev);
	  terminal->ClearLine(x_prev, y_prev+2);
	  terminal->ClearLine(x_prev, y_prev+4);
	  terminal->Write("Updating entry for: ", p->XPos(), p->YOffset(2)); 
	  terminal->Write(strdb.GetKM(), p->XPos(), p->YOffset(2)); 
	  Sleep((clock_t)1);
	  terminal->ClearLine(p->XPos(), p->YPos());
	  terminal->ClearLine(p->XPos(), p->YPos()-2);
	  p->SetXY(x_prev, y_prev);
	}
	else {
	  terminal->ClearLine(p->XPos(), p->YPos());
	  terminal->ClearLine(p->XPos(), p->YPos()-2);
	  p->SetXY(x_prev, y_prev);
	}
      }
      else {
	// Ensure that the template does not get added to the file
	if(NoCaseCmp(KeyName, words[0]) != 0) {
	  FAU addr = strdb.AddObject(0); // Write the object to the file
	  imports++;
	  if(!addr) {
	    x_prev = p->XPos(); y_prev = p->YPos();
	    terminal->Write("Could not add:", p->XPos(), p->YOffset(2));
	    terminal->Write(strdb.GetKM());
	    terminal->Write(" to database");
	    terminal->AnyKey(p->XPos(), p->YOffset(2));
	    terminal->ClearLine(p->XPos(), p->YPos());
	    terminal->ClearLine(p->XPos(), p->YPos()-2);
	    p->SetXY(x_prev, y_prev);
	    imports--;
	  }
	}
        cache->Add(strdb); // Update the cache
      }
    }

    listptr = listptr->GetNext();
  }

  terminal->Write("Imported ", p->XPos(), p->YOffset(2));
  terminal->Write(imports);
  terminal->Write(" items.");
  if(updates) {
    terminal->Write("Updated ", p->XPos(), p->YOffset(2));
    terminal->Write(updates);
    terminal->Write(" items.");
  }
  
  terminal->AnyKey(p->XPos(), p->YOffset(2));
  delete p;          // Free the Coords pointer
  delete[] tmp;
  list.Clear();
}

void BackUp()
{
  char buf[CBuf];
  char *FileName;

  terminal->ClearScreen();
  Coords *p = new Coords(0, 0);

  terminal->Write("Backing up the database to another file",
		  p->XPos(), p->YPos());

  terminal->Write("Enter new file name to create: ",
		  p->XPos(), p->YOffset(2));
  terminal->GetString(buf);

  char *tmp = new char[strlen(buf)+1];
  tmp[strlen(buf)] = '\0';
  strcpy(tmp, buf);
  
  if(VBDFile::Exists(tmp)) {
    terminal->Write("File: ",
		    p->XPos(), p->YOffset(2));
    terminal->Write(tmp);
    terminal->Write(" already exists.");
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    delete p;
    delete[] tmp;
    return;
  }

  FileName = tmp;
  POD *newdb = new POD(FileName);
  int items = 0;
  StrDB strdb(newdb);
  BNodeBase *nxt;     // BSTree base node pointer
  RBNode<StrDB> *ptr; // RB Tree Node pointer 
    
  // Using iterator object to walk throught the tree
  TreeWalkerb tw(cache->GetRoot(), INORDER);
  
  while((nxt = tw.Next()) != 0) {
    ptr = (RBNode<StrDB> *)nxt;
    strdb.Copy(ptr->Data);
    strdb.AddObject(0);
    items++;
  }

  terminal->Write("Backed up ",
		  p->XPos(), p->YOffset(2));
  terminal->Write(items);
  terminal->Write(" objects to ");
  terminal->Write(tmp);
  terminal->AnyKey(p->XPos(), p->YOffset(2));

  delete newdb;
  delete[] tmp;
  delete p;
}

void Merge()
{
  char buf[CBuf];
  char *FileName;
  FAU addr;
  
  terminal->ClearScreen();
  Coords *p = new Coords(0, 0);

  terminal->Write("Merging the contents of another database", 
		  p->XPos(), p->YPos());

  terminal->Write("Enter file name to merge: ",
		  p->XPos(), p->YOffset(2));
  terminal->GetString(buf);
  char *tmp = new char[strlen(buf)+1];
  tmp[strlen(buf)] = '\0';
  strcpy(tmp, buf);
  
  if(!VBDFile::Exists(tmp)) {
    terminal->Write("File: ",
		    p->XPos(), p->YOffset(2));
    terminal->Write(tmp);
    terminal->Write(" does not exists.");
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    delete p;
    delete[] tmp;
    return;
  }

  FileName = tmp;
  POD *newdb = new POD(FileName);
  RBTree<StrDB> *newcache = new RBTree<StrDB>;
  StrDB strdb(newdb);
  StrDB ob(DB);
  
  while(1) { // Load new database into memory
      addr = newdb->OpenDatabase()->FindFirstObject(addr);
      if(!addr) break;
      strdb.ReadObject(addr);
      newcache->Add(strdb);
    }

  BNodeBase *nxt;          // BSTree base node pointer
  RBNode<StrDB> *ptr;      // RB Tree Node pointer to new cache 
  RBNode<StrDB> *existing; // RB Tree Node pointer existing cache
  int x_prev, y_prev, c;
  int imports = 0;
  int updates = 0;
  int updateall = 0;
  
  // Using iterator object to walk throught the tree
  TreeWalkerb tw(newcache->GetRoot(), INORDER);
  
  while((nxt = tw.Next()) != 0) {
    ptr = (RBNode<StrDB> *)nxt;
    strdb.Copy(ptr->Data);
    existing = FindKeyMember(strdb.GetKM());  
    if(existing) {
      x_prev = p->XPos(); y_prev = p->YPos();
      if(updateall == 0) {
	terminal->Write(strdb.GetKM(), p->XPos(), p->YOffset(2));
	terminal->Write(" entry already exists."); 
	terminal->Write("(U)-Update, (A)-Update all, (X)-Exit",
			p->XPos(), p->YOffset(2));
	terminal->Write(", Any other key to continue");
	c = terminal->GetChar();
      }
      if(updateall == 1) c = 'a';
      switch(c) {
	case 'a': case 'A': 
	  updateall = 1;
	  updates++;
	  ob.Copy(existing->Data);
          if(strdb.FullCompare(ob)) break; // The object has not changed
	  cache->Delete(ob);
	  ob.DeleteObject(ob.ObjectAddress()); 
          ob.Copy(strdb);
	  ob.AddObject(0);
	  cache->Add(ob);  
	  break;
	case 'u': case 'U': 
	  updates++;
	  ob.Copy(existing->Data);
          if(strdb.FullCompare(ob)) break; // The object has not changed
	  cache->Delete(ob);
	  ob.DeleteObject(ob.ObjectAddress()); 
          ob.Copy(strdb);
	  ob.AddObject(0);
	  cache->Add(ob);  
	  break;
	case 'x': case 'X':
	  newcache->Clear();
	  delete newdb;
	  delete newcache;
	  delete p;
	  delete[] tmp;
	  return;
	default:
	  break;
      }
      if(updateall == 1) {
	p->SetXY(x_prev, y_prev);
	terminal->ClearLine(x_prev, y_prev+1);
	terminal->ClearLine(x_prev, y_prev+4);
	terminal->Write("Updating entry for: ", p->XPos(), p->YOffset(2)); 
	terminal->Write(strdb.GetKM(), p->XPos(), p->YOffset(2)); 
	Sleep((clock_t)1);
	terminal->ClearLine(p->XPos(), p->YPos());
	terminal->ClearLine(p->XPos(), p->YPos()-2);
	p->SetXY(x_prev, y_prev);
      }
      else {
	terminal->ClearLine(p->XPos(), p->YPos());
	terminal->ClearLine(p->XPos(), p->YPos()-2);
	p->SetXY(x_prev, y_prev);
      }
    }
    else {
      ob.Copy(ptr->Data);
      addr = ob.AddObject(0); // Write the object to the file
      imports++;
      if(!addr) {
	x_prev = p->XPos(); y_prev = p->YPos();
	terminal->Write("Could not add:", p->XPos(), p->YOffset(2));
	terminal->Write(ob.GetKM());
	terminal->Write(" to database");
	terminal->AnyKey(p->XPos(), p->YOffset(2));
	terminal->ClearLine(p->XPos(), p->YPos());
	terminal->ClearLine(p->XPos(), p->YPos()-2);
	p->SetXY(x_prev, y_prev);
	imports--;
      }
      cache->Add(strdb); // Update the cache
    }
  }

  terminal->Write("Imported ", p->XPos(), p->YOffset(2));
  terminal->Write(imports);
  terminal->Write(" items.");
  if(updates) {
    terminal->Write("Updated ", p->XPos(), p->YOffset(2));
    terminal->Write(updates);
    terminal->Write(" items.");
  }
  terminal->AnyKey(p->XPos(), p->YOffset(2));

  newcache->Clear();
  delete newdb;
  delete newcache;
  delete p;
  delete[] tmp;
}

void CreateTemplate()
{
  char buf[CBuf];
  char *FileName;
  const char dchar = '\t';   // Text delimiter

  terminal->ClearScreen();
  Coords *p = new Coords(0, 0);

  terminal->Write("Creating template file delimited by tabs",
		  p->XPos(), p->YPos());

  terminal->Write("Enter file name to create: ",
		  p->XPos(), p->YOffset(2));
  terminal->GetString(buf);

  char *tmp = new char[strlen(buf)+1];
  tmp[strlen(buf)] = '\0';
  strcpy(tmp, buf);
  
  if(VBDFile::Exists(tmp)) {
    terminal->Write("File: ",
		    p->XPos(), p->YOffset(2));
    terminal->Write(tmp);
    terminal->Write(" already exists.");
    int yn = terminal->YesNo("Overwrite it (y/n)?", p->XPos(), p->YOffset(2));
    if(!yn) {
      delete p;
      delete[] tmp;
      return;
    }
  }

  FileName = tmp;
  delete[] tmp;
  
  ofstream stream(FileName, ios::out); // Open file and truncate it
  terminal->Write("Creating template file: ",
		  p->XPos(), p->YOffset(2));
  terminal->Write(FileName);
  
  if(!stream) { // Could not open the stream
    terminal->Write("Could not write to: ",
		    p->XPos(), p->YOffset(2));
    terminal->Write(FileName);
    terminal->AnyKey(p->XPos(), p->YOffset(2));
    delete p; // Free the Coords pointer
    return; 
  }

  if(KeyName != 0) {
    FPrint(KeyName, stream, strlen(KeyName));
    stream << dchar;
  }
  if(M2Name != 0) {
    FPrint(M2Name, stream, strlen(M2Name));
    stream << dchar;
  }
  if(M3Name != 0) {
    FPrint(M3Name, stream, strlen(M3Name));
    stream << dchar;
  }
  if(M4Name != 0) {
    FPrint(M4Name, stream, strlen(M4Name));
    stream << dchar;
  }
  if(M5Name != 0) {
    FPrint(M5Name, stream, strlen(M5Name));
    stream << dchar;
  }
  if(M6Name != 0) {
    FPrint(M6Name, stream, strlen(M6Name));
    stream << dchar;
  }
  if(M7Name != 0) {
    FPrint(M7Name, stream, strlen(M7Name));
    stream << dchar;
  }
  if(M8Name != 0) {
    FPrint(M8Name, stream, strlen(M8Name));
    stream << dchar;
  }
  if(Comments != 0) {
    FPrint(Comments, stream, strlen(Comments));
  }
  stream << LineFeed;
  
  terminal->Write("Finished", p->XPos(), p->YOffset(2));
  terminal->AnyKey(p->XPos(), p->YOffset(2));
  delete p;          // Free the Coords pointer
  stream.close();
}

int main(int argc, char **argv)
{
  // Display the program version information and exit the program
  if(argc >= 2) {
    if(strcmp(argv[1], "version") == 0) Version();
  }

  char *FileName;
  char *CurrentCfgFile;
  char *CfgFile;
  Config *CfgData = new Config;

  // Look for CfgFile name in environment
  if((CurrentCfgFile = getenv(EnvSetting)) == 0)
     CfgFile = (char *)DefaultCfgFile;
  else
    CfgFile = CurrentCfgFile;

  // Use default file name if no file is specifed on command line
  if(argc < 2) {
    int FileStatus = CfgData->Load((char *)CfgFile);
    if(!FileStatus) {
       printf("\nCannot locate any database files!\n");
       exit(0);
    }
    else {
      FileName = CfgData->GetStrValue("DBFileName");
      if(!FileName) {
       printf("\nDBFileName section missing in config file: %s\n", CfgFile);
       exit(0);
      }
    }
  }
  else {
    FileName = argv[1];
  }

  CfgData->UnLoad(); // Unload the Config file from memory
  UString FN(FileName);
  DB = new POD(FN);          // Initialize global database pointer
  cache = new RBTree<StrDB>; // Initialize global cache pointer
  terminal->init();
  LoadCache();
  MainMenu();
  terminal->finish();
  delete DB;
  delete cache;
  return 0;
}

int parse(char *string, char words[MAXWORDS][MAXWORDLENGTH],
	  int*numwords, char sepchar)
// General purpose string parser
{
  int i = 0;
  char newword[MAXWORDLENGTH];
  *numwords = 0;

  // First skip over leading blanks. Stop if an ASCII NULL is seen
  while (1) {
    if (*string == '\0') return 0;
    if (*string != ' ') break;
    string++;
  }
  
  while(1) {
    // Check to see if there is room for another word in the array
    if(*numwords == MAXWORDS) return 1;

    i = 0;
    while (i < MAXWORDLENGTH) {
      if(*string == 0 || *string == sepchar) break;
      newword[i] = *string;
      string++;
      i++;
    }
    newword[i] = 0;   // Ensure an ASCII null at end of newword. 
    strcpy (words[*numwords], newword);  // Install into array 
    (*numwords)++;
    
    // If stopped by an ASCII NULL above, exit loop
    if(*string == 0) break;
    
    string++;
    if(*string == 0) break;
  }
  return 0;
}
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
