// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: int32.cpp 
// Compiler Used: MSVC40, DJGPP 2.7.2.1, GCC 2.7.2.1, HP CPP 10.24
// Produced By: Doug Gaer   
// File Creation Date: 09/05/97  
// Date Last Modified: 12/03/97
// Copyright (c) 1997 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBD C++ classes are copyright (c) 1997, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

The INT32 class is used to represent 32 bit signed integers
independently of the operating system or hardware platform
used. It was created to overcome the big and little endian
problem encountered when writing integer values to a common
database file accessed by several different types of machines.

The term "endian" is used to describe the order in which
multibyte numbers are stored in the computers's memory. This
is specifc to each microprocessor. The Intel x86 family is
little-endian, meaning the lowest-order byte is stored first.
Hewlett-Packard's PA-RISC and Sun's SuperSPARC are big-endian,
meaning the highest-order byte is stored first. The Silicon
Graphics MIPS and IBM/Motorola PowerPC processors are both
little and big endian (bi-endian).

 Multibyte Value     Big-Endian    Little-Endian
   0x12345678        0x12345678    0x78563412
   0x1234            0x1234        0x3412
   0x5678            0x5678        0x7856
   "ABC"             41 42 42 00   41 42 43 00

With big-endian ordering, the address of the multibyte value
is its most significant byte (its big end). With little-endian
ordering, the address is its least significant byte (its little
end). In a struct the order of bytes in memory will differ
depending on the byte ordering and the particular data type.
*/
// ----------------------------------------------------------- // 
#include <string.h>
#include <memory.h>
#include "int32.hpp"
#include "ehandler.hpp"

INT32::INT32(LWORD val)
{
  UnPackBits(val);
}

INT32::INT32(const INT32& ob)
{
  memmove((void *)byte, (const void *)ob.byte, 4);
}

INT32& INT32::operator=(const INT32& ob)
{
  memmove((void *)byte, (const void *)ob.byte, 4);
  return *this;
}

INT32& INT32::operator=(const LWORD val)
{
  UnPackBits(val);
  return *this;
}

INT32::operator LWORD() const
{
  return PackBits();
}

LWORD INT32::PackBits() const
{
  LWORD a, b, c, d;
  
  a = (LWORD)byte[0];
  b = (LWORD)byte[1];
  c = (LWORD)byte[2];
  d = (LWORD)byte[3];

  a = a & 0xFF;
  b = (b<<8) & 0xFF00;
  c = (c<<16) & 0xFF0000;
  d = (d<<24) & 0xFF000000;

  return a + b + c + d;
}

void INT32::UnPackBits(LWORD val)
{
  byte[0] = val & 0xFF;
  byte[1] = (val & 0xFF00)>>8;
  byte[2] = (val & 0xFF0000)>>16;
  byte[3] = (val & 0xFF000000)>>24;
}

INT32 INT32::operator++(int) // Postfix
{
  INT32 val_before(*this); 
  operator=(*this + 1);
  return val_before;
}
 
INT32 INT32::operator--(int) // Postfix
{
  INT32 val_before(*this); 
  operator=(*this - 1);
  return val_before;
}

void INT32::operator/=(const INT32 &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void INT32::operator/=(const LWORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

    operator=(*this / i);
}

void INT32::operator/=(const ULWORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void INT32::operator/=(const WORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void INT32::operator/=(const SWORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void INT32::operator/=(const UWORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void INT32::operator/=(const USWORD &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / i);
}

void INT32::operator/=(const SBYTE &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / (LWORD)i);
}

void INT32::operator/=(const UBYTE &i)
{
  if(i == 0)
#ifdef CPP_EXCEPTIONS
    throw CDivideByZero();
#else
    Error->SignalException(EHandler::DivideByZero);
#endif

  operator=(*this / (LWORD)i);
}

int operator==(const INT32 &a, const INT32 &b)
{
  return a.PackBits() == b.PackBits();
}

int operator==(const INT32 &a, const LWORD &bl)
{
  return a.PackBits() == bl;
}

int operator==(const LWORD &al, const INT32 &b)
{
  return al == b.PackBits(); 
}

int operator==(const INT32 &a, const ULWORD &bl)
{
  return a.PackBits() == bl;
}

int operator==(const ULWORD &al, const INT32 &b)
{
  return al == b.PackBits(); 
}

int operator==(const INT32 &a, const WORD &bl)
{
  return a.PackBits() == bl;
}

int operator==(const WORD &al, const INT32 &b)
{
  return al == b.PackBits();
}

int operator==(const INT32 &a, const SWORD &bl)
{
  return a.PackBits() == bl;
}

int operator==(const SWORD &al, const INT32 &b)
{
  return al == b.PackBits(); 
}

int operator==(const INT32 &a, const UWORD &bl)
{
  return a.PackBits() == bl;
}

int operator==(const UWORD &al, const INT32 &b)
{
  return al == b.PackBits(); 
}

int operator==(const INT32 &a, const USWORD &bl)
{
  return  a.PackBits() == bl;
}

int operator==(const USWORD &al, const INT32 &b)
{
  return al == b.PackBits(); 
}

int operator==(const INT32 &a, const SBYTE &bl)
{
  return a.PackBits() == (LWORD)bl;
}

int operator==(const SBYTE &al, const INT32 &b)
{
  return (LWORD)al == b.PackBits(); 
}

int operator==(const INT32 &a, const UBYTE &bl)
{
  return a.PackBits() == (LWORD)bl;
}

int operator==(const UBYTE &al, const INT32 &b)
{
  return (LWORD)al == b.PackBits(); 
}

int operator!=(const INT32 &a, const INT32 &b)
{
  return a.PackBits() != b.PackBits();
}

int operator!=(const INT32 &a, const LWORD &bl)
{
  return a.PackBits() != bl;
}

int operator!=(const LWORD &al, const INT32 &b)
{
  return al != b.PackBits();
}

int operator!=(const INT32 &a, const ULWORD &bl)
{
  return a.PackBits() != bl;
}

int operator!=(const ULWORD &al, const INT32 &b)
{
  return al != b.PackBits();
}

int operator!=(const INT32 &a, const WORD &bl)
{
  return a.PackBits() != bl;
}

int operator!=(const WORD &al, const INT32 &b)
{
  return al != b.PackBits();
}

int operator!=(const INT32 &a, const SWORD &bl)
{
  return a.PackBits() != bl;
}

int operator!=(const SWORD &al, const INT32 &b)
{
  return al != b.PackBits();
}

int operator!=(const INT32 &a, const UWORD &bl)
{
  return a.PackBits() != bl;
}

int operator!=(const UWORD &al, const INT32 &b)
{
  return al != b.PackBits();
}

int operator!=(const INT32 &a, const USWORD &bl)
{
  return a.PackBits() != bl;
}

int operator!=(const USWORD &al, const INT32 &b)
{
  return al != b.PackBits();
}

int operator!=(const INT32 &a, const SBYTE &bl)
{
  return a.PackBits() != (LWORD)bl;
}

int operator!=(const SBYTE &al, const INT32 &b)
{
  return (LWORD)al != b.PackBits();
}

int operator!=(const INT32 &a, const UBYTE &bl)
{
  return a.PackBits() != (LWORD)bl;
}

int operator!=(const UBYTE &al, const INT32 &b)
{
  return (LWORD)al != b.PackBits();
}

int operator<(const INT32 &a, const INT32 &b)
{
  return a.PackBits() < b.PackBits();
}

int operator<(const INT32 &a, const LWORD &bl)
{
  return a.PackBits() < bl;
}

int operator<(const LWORD &al, const INT32 &b)
{
  return al < b.PackBits();
}

int operator<(const INT32 &a, const ULWORD &bl)
{
  return a.PackBits() < bl;
}

int operator<(const ULWORD &al, const INT32 &b)
{
  return al < b.PackBits();
}

int operator<(const INT32 &a, const WORD &bl)
{
  return a.PackBits() < bl;
}

int operator<(const WORD &al, const INT32 &b)
{
  return al < b.PackBits();
}

int operator<(const INT32 &a, const SWORD &bl)
{
  return a.PackBits() < bl;
}

int operator<(const SWORD &al, const INT32 &b)
{
  return al < b.PackBits();
}

int operator<(const INT32 &a, const UWORD &bl)
{
  return a.PackBits() < bl;
}

int operator<(const UWORD &al, const INT32 &b)
{
  return al < b.PackBits();
}

int operator<(const INT32 &a, const USWORD &bl)
{
  return a.PackBits() < bl;
}

int operator<(const USWORD &al, const INT32 &b)
{
  return al < b.PackBits();
}

int operator<(const INT32 &a, const SBYTE &bl)
{
  return a.PackBits() < (LWORD)bl;
}

int operator<(const SBYTE &al, const INT32 &b)
{
  return (LWORD)al < b.PackBits();
}

int operator<(const INT32 &a, const UBYTE &bl)
{
  return a.PackBits() < (LWORD)bl;
}

int operator<(const UBYTE &al, const INT32 &b)
{
  return (LWORD)al < b.PackBits();
}

int operator>(const INT32 &a, const INT32 &b)
{
  return a.PackBits() > b.PackBits();
}

int operator>(const INT32 &a, const LWORD &bl)
{
  return a.PackBits() > bl;
}

int operator>(const LWORD &al, const INT32 &b)
{
  return al > b.PackBits();
}

int operator>(const INT32 &a, const ULWORD &bl)
{
  return a.PackBits() > bl;
}

int operator>(const ULWORD &al, const INT32 &b)
{
  return al > b.PackBits();
}

int operator>(const INT32 &a, const WORD &bl)
{
  return a.PackBits() > bl;
}

int operator>(const WORD &al, const INT32 &b)
{
  return al > b.PackBits();
}

int operator>(const INT32 &a, const SWORD &bl)
{
  return a.PackBits() > bl;
}

int operator>(const SWORD &al, const INT32 &b)
{
  return al > b.PackBits();
}

int operator>(const INT32 &a, const UWORD &bl)
{
  return a.PackBits() > bl;
}

int operator>(const UWORD &al, const INT32 &b)
{
  return al > b.PackBits();
}

int operator>(const INT32 &a, const USWORD &bl)
{
  return a.PackBits() > bl;
}

int operator>(const USWORD &al, const INT32 &b)
{
  return al > b.PackBits();
}

int operator>(const INT32 &a, const SBYTE &bl)
{
  return a.PackBits() > (LWORD)bl;
}

int operator>(const SBYTE &al, const INT32 &b)
{
  return (LWORD)al > b.PackBits();
}

int operator>(const INT32 &a, const UBYTE &bl)
{
  return a.PackBits() > (LWORD)bl;
}

int operator>(const UBYTE &al, const INT32 &b)
{
  return (LWORD)al > b.PackBits();
}

int operator<=(const INT32 &a, const INT32 &b)
{
  return a.PackBits() == b.PackBits() || a.PackBits() < b.PackBits();
}

int operator<=(const INT32 &a, const LWORD &bl)
{
  return a.PackBits() == bl || a.PackBits() < bl;
}

int operator<=(const LWORD &al, const INT32 &b)
{
  return al == b.PackBits() || al < b.PackBits();
}

int operator<=(const INT32 &a, const ULWORD &bl)
{
  return a.PackBits() == bl || a.PackBits() < bl;
}

int operator<=(const ULWORD &al, const INT32 &b)
{
  return al == b.PackBits() || al < b.PackBits();
}

int operator<=(const INT32 &a, const WORD &bl)
{
  return a.PackBits() == bl || a.PackBits() < bl;
}

int operator<=(const WORD &al, const INT32 &b)
{
  return al == b.PackBits() || al < b.PackBits();
}

int operator<=(const INT32 &a, const SWORD &bl)
{
  return a.PackBits() == bl || a.PackBits() < bl;
}

int operator<=(const SWORD &al, const INT32 &b)
{
  return al == b.PackBits() || al < b.PackBits();
}

int operator<=(const INT32 &a, const UWORD &bl)
{
  return a.PackBits() == bl || a.PackBits() < bl;
}

int operator<=(const UWORD &al, const INT32 &b)
{
  return al == b.PackBits() || al < b.PackBits();
}

int operator<=(const INT32 &a, const USWORD &bl)
{
  return a.PackBits() == bl || a.PackBits() < bl;
}

int operator<=(const USWORD &al, const INT32 &b)
{
  return al == b.PackBits() || al < b.PackBits();
}

int operator<=(const INT32 &a, const SBYTE &bl)
{
  return a.PackBits() == (LWORD)bl ||  a.PackBits() < (LWORD)bl;
}

int operator<=(const SBYTE &al, const INT32 &b)
{
  return (LWORD)al == b.PackBits() || (LWORD)al < b.PackBits();
}

int operator<=(const INT32 &a, const UBYTE &bl)
{
  return a.PackBits() == (LWORD)bl ||  a.PackBits() < (LWORD)bl;
}

int operator<=(const UBYTE &al, const INT32 &b)
{
  return (LWORD)al == b.PackBits() || (LWORD)al < b.PackBits();
}

int operator>=(const INT32 &a, const INT32 &b)
{
  return a.PackBits() == b.PackBits() || a.PackBits() > b.PackBits();
}

int operator>=(const INT32 &a, const LWORD &bl)
{
  return a.PackBits() == bl || a.PackBits() > bl;
}

int operator>=(const LWORD &al, const INT32 &b)
{
  return al == b.PackBits() || al > b.PackBits();
}

int operator>=(const INT32 &a, const ULWORD &bl)
{
  return a.PackBits() == bl || a.PackBits() > bl;
}

int operator>=(const ULWORD &al, const INT32 &b)
{
  return al == b.PackBits() || al > b.PackBits();
}

int operator>=(const INT32 &a, const WORD &bl)
{
  return a.PackBits() == bl || a.PackBits() > bl;
}

int operator>=(const WORD &al, const INT32 &b)
{
  return al == b.PackBits() || al > b.PackBits();
}

int operator>=(const INT32 &a, const SWORD &bl)
{
  return a.PackBits() == bl || a.PackBits() > bl;
}

int operator>=(const SWORD &al, const INT32 &b)
{
  return al == b.PackBits() || al > b.PackBits();
}

int operator>=(const INT32 &a, const UWORD &bl)
{
  return a.PackBits() == bl || a.PackBits() > bl;
}

int operator>=(const UWORD &al, const INT32 &b)
{
  return al == b.PackBits() || al > b.PackBits();
}

int operator>=(const INT32 &a, const USWORD &bl)
{
  return a.PackBits() == bl || a.PackBits() > bl;
}

int operator>=(const USWORD &al, const INT32 &b)
{
  return al == b.PackBits() || al > b.PackBits();
}

int operator>=(const INT32 &a, const SBYTE &bl)
{
  return a.PackBits() == (LWORD)bl || a.PackBits() > (LWORD)bl;
}

int operator>=(const SBYTE &al, const INT32 &b)
{
  return (LWORD)al == b.PackBits() || (LWORD)al > b.PackBits();
}

int operator>=(const INT32 &a, const UBYTE &bl)
{
  return a.PackBits() == (LWORD)bl || a.PackBits() > (LWORD)bl;
}

int operator>=(const UBYTE &al, const INT32 &b)
{
  return (LWORD)al == b.PackBits() || (LWORD)al > b.PackBits();
}
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //
