// ------------------------------- //
// -------- Start of File -------- //
// ------------------------------- //
// ----------------------------------------------------------- // 
// C++ Source Code File Name: vb_debug.cpp 
// Compiler Used: MSVC40, DJGPP 2.7.2.1, GCC 2.7.2.1, HP CPP 10.24
// Produced By: Doug Gaer 
// File Creation Date: 02/04/97  
// Date Last Modified: 12/04/97
// Copyright (c) 1997 Douglas M. Gaer
// ----------------------------------------------------------- // 
// ------------- Program Description and Details ------------- // 
// ----------------------------------------------------------- // 
/*
The VBD C++ classes are copyright (c) 1997, by Douglas M. Gaer.
All those who put this code or its derivatives in a commercial
product MUST mention this copyright in their documentation for
users of the products in which this code or its derivative
classes are used. Otherwise, you have the freedom to redistribute
verbatim copies of this source code, adapt it to your specific
needs, or improve the code and release your improvements to the
public provided that the modified files carry prominent notices
stating that you changed the files and the date of any change.

THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.
THE ENTIRE RISK OF THE QUALITY AND PERFORMANCE OF THIS SOFTWARE
IS WITH YOU. SHOULD ANY ELEMENT OF THIS SOFTWARE PROVE DEFECTIVE,
YOU WILL ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR, OR
CORRECTION.

This program is used to display a detailed analysis of every
Variable Block allocated in a VBD file.
*/
// ----------------------------------------------------------- // 
#include <iostream.h>
#include <string.h>
#include "dvbdfile.hpp"

// Program name and version number
const int version = 1024;
const char *name = "VBD file debug utility program";

// Function prototypes for display menu
void SkipToEol(istream &s);
int Quit();
void Menu(const VBDFilePtr &f);
void pause();
void Version();

// Function prototypes for VBD utilities 
void FindVB(const VBDFilePtr &f, int verbose = 0);
void AnalyzeHeader(const VBDFilePtr &f);
void DisplayStats(const VBDFilePtr &f);
void VBStats(const VBDFilePtr &f, FAU VBAddress);
void Rebuild(const VBDFilePtr &f);

void DisplayStats(const VBDFilePtr &f)
// Passing VBDFilePtr by reference to avoid calling the Refcount
// copy constructor. This will avoid any unnecessary aliasing.
{
  const char *FName = f->VBDFileName();
  FileHeader fh;

  f->Read(&fh, sizeof(FileHeader), 0);
  if (memcmp(fh.Signature, VBDFile::VBDSignature, 8)) { // Test file type
    cout << endl;
    cout << "VBD File Header is damaged or does not exist" <<endl;
    cout << endl;
    FindVB(f, 0);
    return;
  }

  cout << endl;
  SBYTE FileStatus = f->GetFileStatus(); // Get the current file status
  cout << "----- VBD file statistics -----" << endl;
  cout << endl;
  cout << "----- Static information -----" << endl;
  cout << "VBD File Name:          " << FName << endl;
  cout << "VBDFile Signature:      " << f->GetSignature() << endl; 
  cout << "VBDFile Version Number: " << f->GetVersion() << endl;
  cout << "File Header Size :      " << f->FileHeaderSize() << endl;
  cout << "VB Header Size:         " << f->VBHeaderSize() << endl;
  cout << "Static area size:       " << f->StaticArea() << endl;
  cout << "Static file size:       " << f->FileSize(FName) << endl;
  cout << endl;
  cout << "----- Dynamic Information -----" << endl;
  cout << "File status Byte:       ";
  if(FileStatus & 0x01) cout << "Good "; else cout << "Bad ";
  if(FileStatus & 0x02) cout << "Open "; else cout << "Closed ";
  if(FileStatus & 0x04) cout << "Read/Write "; else cout << "Read/Only ";
  cout << endl;
  cout << "VBDFile Free Space:     " << f->GetFreeSpace() << endl;
  cout << "VBDFile End of File:    " << f->GetEOF() << endl;
  cout << "VBDFile Heap Start:     " << f->GetHeapStart() << endl;
  cout << "VBDFile Highest VB:     " << f->GetHighestVB() << endl;
  cout << "Total VBs in file:      " << f->VBTotal() << endl;
  cout << "Total deleted VBs:      " << f->VBDeleted() << endl;
  pause();
}

void VBStats(const VBDFilePtr &f, FAU VBAddress)
// Passing VBDFilePtr by reference to avoid calling the Refcount
// copy constructor. This will avoid any unnecessary aliasing.
{
  VBHeader vb;
  f->Read(&vb, sizeof(VBHeader), VBAddress);

  cout << endl;
  cout << "----- (V)ariable (B)lock statistics -----" << endl;
  cout << "VB check word = " << vb.CkWord << endl;
  cout << "VB length     = " << vb.Length << endl;
  cout << "VB status     = " << (SBYTE)vb.Status << endl;
  cout << "VB NDB        = " << vb.NextDeletedVB << endl;
  cout << endl;
  cout << "----- Object statistics -----" << endl;
  cout << "Object length  = " << vb.Length - sizeof(VBHeader) << endl;
  cout << endl;
  cout << "----- File Address Info -----" << endl;
  cout << "VB File Address   = " << VBAddress << endl;
  cout << "Next File Address = " << VBAddress + vb.Length << endl;
}

void AnalyzeHeader(const VBDFilePtr &f)
{
  FileHeader fh;
  VBHeader vb;
  int errors = 0;
  
  f->Read(&fh, sizeof(FileHeader), 0);
  cout << endl;
  cout << "Analyzing VBD file header..." << endl;
  cout << endl;

  cout << "Checking VBD signature..." << endl;
  if (memcmp(fh.Signature, VBDFile::VBDSignature, 8)) { // Test file type
    cout << endl;
    cout << "VBD File Header is damaged or does not exist" <<endl;
    cout << endl;
    FindVB(f, 0);
    return;
  }

  cout << "Checking for End of File errors..." << endl;
  FAU StaticEOF = f->FileSize(f->VBDFileName());
  if(StaticEOF > fh.EndOfFile) {
    cout << endl;
    cout << "End of file error in file: " << f->VBDFileName() << endl;
    cout << "The actual length is longer then the allocated length!" << endl;
    cout << endl;
    errors++;
  }

  cout << "Checking the HeapStart value..." << endl;
  f->Read(&vb, sizeof(VBHeader), fh.HeapStart);
  if (vb.CkWord != CheckWord) {
    cout << endl;
    cout << "Bad HeapStart value in file: " << f->VBDFileName() << endl;
    cout << "No variable block found at file address: "
	 << fh.HeapStart << endl;
    cout << endl;
    errors++;
  }

  cout << "Checking the FreeSpace value..." << endl;
  if(fh.FreeSpace != 0) {
    f->Read(&vb, sizeof(VBHeader), fh.FreeSpace);
    if (vb.CkWord != CheckWord) {
      cout << endl;
      cout << "Bad FreeSpace value in file: " << f->VBDFileName() << endl;
      cout << "No variable block found at file address: "
	   << fh.FreeSpace << endl;
      cout << endl;
    }
    errors++;
  }

  cout << "Checking the HighestVB value..." << endl;
  f->Read(&vb, sizeof(VBHeader), fh.HighestVB);
  if (vb.CkWord != CheckWord) {
    cout << endl;
    cout << "Bad HeapStart value in file: " << f->VBDFileName() << endl;
    cout << "No variable block found at file address: "
	 << fh.HighestVB << endl;
    cout << endl;
    errors++;
  }
  
  cout << "Checking the VBD version number..." << endl;
  if(fh.Version != VBDFile::VBDVersion) {
    cout << endl;
    cout << "The version numbers do not match!" << endl;
    cout << "Current VBD File Version = " << VBDFile::VBDVersion << endl;
    cout << "The file reads: " << fh.Version << endl;
    cout << endl;
    errors++;
  }

  if(errors) {
    cout << endl;
    cout << "VBD file header has errors!" << endl;
  }
  else {
    cout << endl;
    cout << "VBD file header checks good." << endl;
  }
  pause();
}

void FindVB(const VBDFilePtr &f, int verbose)
// Serach the file for all variable block.
{
  VBHeader vb;
  int count = 0;
  int badvb = 0;
  
  cout << endl;
  cout << "Searching file for all variable blocks..." << endl;
  
  FAU StaticEOF = f->FileSize(f->VBDFileName());
  FAU addr = f->VBSearch(0); // Search the entire file
  
  if(addr == 0) {
    cout << endl;
    cout << "No variable blocks found in file: "
	 << f->VBDFileName() << endl;
    cout << endl;
    return;
  }
  
  while(1) { 
    if(addr >= StaticEOF) break;
    f->Read(&vb, sizeof(VBHeader), addr);

    if (vb.CkWord == CheckWord) {
      count++; // Increment the variable block count
      if(verbose) {
	VBStats(f, addr);
	char c;
	cout << endl;
	cout << "Press enter to continue or enter 'X' to exit >";
	cin.clear();
	cin.get(c);
	switch(c) {
	  case 'x' : case 'X' : Menu(f); return;
	  default: break;
	}
      }
      addr = addr + vb.Length; // Goto the next variable block
    }
    else {
      badvb++;
      cout << endl;
      cout << "Found bad variable block at address " << addr << endl;
      cout << "Searching for next good variable block..." << endl;
      addr = f->VBSearch(addr); // Search for the next good block
      if(!addr) {
	cout << "None found!" << endl;
	cout << endl;
	return;
      }
    }
  }
  cout << endl;
  cout << "Found " << count << " good varible blocks." << endl;
  if(badvb) cout << "Found " << badvb << " bad variable blocks." << endl;
  pause();
}

void Rebuild(const VBDFilePtr &f)
{
  cout << endl;
  cout << "Rebuilding damaged VBD file..." << endl;
  cout << endl;

  char buf[255];
  int retry = 3;
  
  while(1) { // Loop until a good file name is found
    cout << "Enter new name of file to build >";
    cin.getline(buf, sizeof(buf));
    cout << endl;
    if(!*buf) { // Return if nothing is entered
      Menu(f);
      return;
    }
    if(VBDFile::Exists(buf)) {
      cout << "File already exists!" << endl << endl;
       retry--;
      if(!retry) {
	Menu(f);
	return;
      }
    }
    else
      break;
  }

  VBDFilePtr NewFile(new VBDFile);
  NewFile->Create(buf);

  VBHeader vb;
  FAU StaticEOF = f->FileSize(f->VBDFileName());
  FAU addr = f->VBSearch(0); // Search the entire file
  
  if(addr == 0) {
    cout << endl;
    cout << "No variable blocks found in file: "
	 << f->VBDFileName() << endl;
    pause();
    return;
  }
  
  int count = 0;
  int badvb = 0;

  char *v;
  FAU ObjectLength;
  FAU NewAddr;

  while(1) { 
    if(addr >= StaticEOF) break;

    f->Read(&vb, sizeof(VBHeader), addr);
    if (vb.CkWord == CheckWord) {
      count++; // Increment the variable block count
      ObjectLength = vb.Length - sizeof(VBHeader);
      v = new char[ObjectLength];
      f->Read(v, ObjectLength, addr + sizeof(VBHeader));
      NewAddr = NewFile->Alloc(ObjectLength);
      NewFile->Write(v, ObjectLength, NewAddr);
      delete v;
      addr = addr + vb.Length; // Goto the next variable block
    }
    else {
      badvb++;
      addr = f->VBSearch(addr); // Search for the next good block
      if(!addr) break; 
    }
  }
  cout << endl;
  cout << "Wrote " << count << " good variable blocks to file: "
       << NewFile->VBDFileName() << endl;
  if(badvb) 
    cout << "Did not write " << badvb << " bad variables found in file: "
	 << f->VBDFileName() << endl;
  cout << endl;
  NewFile->Close();
}

int main(int argc, char **argv)
{
  VBDFilePtr f(new VBDFile);   // Should always create dynamically

  // Display the program version information and exit the program
  if(argc >= 2) {
    if(strcmp(argv[1], "version") == 0) {
      Version();
      return 0;
    }
  }

  if(argc < 2) {
       printf("Usage %s <pathname>\n", argv[0]);
       return 1;
   }

  const char *FName = argv[1];
  if(!VBDFile::Exists(FName)) 
    Error->SignalException(EHandler::NoFileExists);
  else
    f->BlindOpen(FName, VBDFile::READONLY);

  char key;
  Menu(f);
  
  int rv = 1;
  while(rv) {
    if (!cin) { // Input is in fail state
       SkipToEol(cin); // Go to end of line
       if (!cin) {  // Can't fix
          cout << "Input stream is broken" << endl;
          return 0;
       }
    }
    cout << '>';
    cin >> key;
    if (!cin) continue; // Fix at top of loop
    switch(key) {
      case 'a' : case 'A' : SkipToEol(cin); AnalyzeHeader(f); break;
      case 'f' : case 'F' : SkipToEol(cin); FindVB(f); break;
      case 'h' : case 'H' : Menu(f); break;
      case '?' : Menu(f); break; 
      case 'q' : case 'Q' : rv = Quit(); break;
      case 'r' : case 'R' : SkipToEol(cin); Rebuild(f); break;
      case 's' : case 'S' : SkipToEol(cin); DisplayStats(f); break;
      case 'v' : case 'V' : SkipToEol(cin); FindVB(f, 1); break;
      case 'z' : case 'Z' : Version(); break;
      default:
        cout << "Unrecognized command" << endl;
    }
  }

  return 0;
}

void Menu(const VBDFilePtr &f)
{
  cout << endl;
  cout << "Analyzing file: " << f->VBDFileName() << endl;
  cout << "Enter the letter of your selection at the prompt." << endl;
  cout << endl;
  cout << "(A) - Analyze the VBD file header" << endl;
  cout << "(F) - Find every variable block in the file" << endl;
  cout << "(H) - Displays this menu" << endl;
  cout << "(Q) - Quit this program" << endl;
  cout << "(R) - Rebuild a damaged VBD file" << endl;
  cout << "(S) - Display VBD file statistics" << endl;
  cout << "(V) - Find every variable block in verbose mode" << endl;
  cout << "(Z) - Display this program's version number" << endl;
  cout << endl;
}

void SkipToEol(istream &s)
// Used to clear istream
{
  char c;
  s.clear();
  while(s.get(c) && c != '\n') { ; }
}

void pause()
{
  cout << endl;
  cout << "Press enter to continue..." << endl;
  cin.get();
}

int Quit()
{
  cout << "Exiting..." << endl;
  return 0;
}

void Version()
{
  cout << endl;
  cout << name << " version number: " << version << endl;
  cout << endl;
}
// ----------------------------------------------------------- //
// ------------------------------- //
// --------- End of File --------- //
// ------------------------------- //

